@@ -28,10 +28,12 @@
 #include <qgssearchtreenode.h>
 
 #include "qgisapp.h"
+#include "qgsaddattrdialog.h"
+#include "qgsdelattrdialog.h"
 #include "qgssearchquerybuilder.h"
 #include "qgslogger.h"
 #include "qgsmapcanvas.h"
-
+#include "qgsfieldcalculator.h"
 
 class QgsAttributeTableDock : public QDockWidget
 {
@@ -76,6 +78,7 @@ QgsAttributeTableDialog::QgsAttributeTableDialog( QgsVectorLayer *theLayer, QWid
     mDock = new QgsAttributeTableDock( tr( "Attribute table - %1" ).arg( mLayer->name() ), QgisApp::instance() );
     mDock->setAllowedAreas( Qt::BottomDockWidgetArea | Qt::TopDockWidgetArea );
     mDock->setWidget( this );
+    connect( this, SIGNAL( destroyed() ), mDock, SLOT( close() ) );
     QgisApp::instance()->addDockWidget( Qt::BottomDockWidgetArea, mDock );
   }
 
@@ -87,9 +90,23 @@ QgsAttributeTableDialog::QgsAttributeTableDialog( QgsVectorLayer *theLayer, QWid
   mZoomMapToSelectedRowsButton->setIcon( getThemeIcon( "/mActionZoomToSelected.png" ) );
   mInvertSelectionButton->setIcon( getThemeIcon( "/mActionInvertSelection.png" ) );
   mToggleEditingButton->setIcon( getThemeIcon( "/mActionToggleEditing.png" ) );
+  mDeleteSelectedButton->setIcon( getThemeIcon( "/mActionDeleteSelected.png" ) );
+  mOpenFieldCalculator->setIcon( getThemeIcon( "/mActionCalculateField.png" ) );
+  mAddAttribute->setIcon( getThemeIcon( "/mActionNewAttribute.png" ) );
+  mRemoveAttribute->setIcon( getThemeIcon( "/mActionDeleteAttribute.png" ) );
+
   // toggle editing
+  bool canChangeAttributes = mLayer->dataProvider()->capabilities() & QgsVectorDataProvider::ChangeAttributeValues;
+  bool canDeleteFeatures = mLayer->dataProvider()->capabilities() & QgsVectorDataProvider::DeleteFeatures;
+  bool canAddAttributes = mLayer->dataProvider()->capabilities() & QgsVectorDataProvider::AddAttributes;
+  bool canDeleteAttributes = mLayer->dataProvider()->capabilities() & QgsVectorDataProvider::DeleteAttributes;
   mToggleEditingButton->setCheckable( true );
-  mToggleEditingButton->setEnabled( mLayer->dataProvider()->capabilities() & QgsVectorDataProvider::ChangeAttributeValues );
+  mToggleEditingButton->setChecked( mLayer->isEditable() );
+  mToggleEditingButton->setEnabled( canChangeAttributes );
+  mOpenFieldCalculator->setEnabled( canChangeAttributes && mLayer->isEditable() );
+  mDeleteSelectedButton->setEnabled( canDeleteFeatures && mLayer->isEditable() );
+  mAddAttribute->setEnabled( canAddAttributes && mLayer->isEditable() );
+  mRemoveAttribute->setEnabled( canDeleteAttributes && mLayer->isEditable() );
 
   // info from table to application
   connect( this, SIGNAL( editingToggled( QgsMapLayer * ) ), QgisApp::instance(), SLOT( toggleEditing( QgsMapLayer * ) ) );
@@ -102,6 +119,7 @@ QgsAttributeTableDialog::QgsAttributeTableDialog( QgsVectorLayer *theLayer, QWid
   connect( mLayer, SIGNAL( selectionChanged() ), this, SLOT( updateSelectionFromLayer() ) );
   connect( mLayer, SIGNAL( layerDeleted() ), this, SLOT( close() ) );
   connect( mView->verticalHeader(), SIGNAL( sectionClicked( int ) ), this, SLOT( updateRowSelection( int ) ) );
+  connect( mView->verticalHeader(), SIGNAL( sectionPressed( int ) ), this, SLOT( updateRowPressed( int ) ) );
   connect( mModel, SIGNAL( modelChanged() ), this, SLOT( updateSelection() ) );
 
   mLastClickedHeaderIndex = 0;
@@ -216,9 +234,14 @@ void QgsAttributeTableDialog::on_mRemoveSelectionButton_clicked()
   mLayer->removeSelection();
 }
 
+void QgsAttributeTableDialog::on_mDeleteSelectedButton_clicked()
+{
+  QgisApp::instance()->deleteSelected( mLayer );
+}
+
 void QgsAttributeTableDialog::on_cbxShowSelectedOnly_toggled( bool theFlag )
 {
-  mFilterModel->mHideUnselected = theFlag;
+  mFilterModel->setHideUnselected( theFlag );
   mFilterModel->invalidate();
   //TODO: weird
   //mModel->changeLayout();
@@ -249,8 +272,6 @@ int QgsAttributeTableDialog::columnBoxColumnId()
 void QgsAttributeTableDialog::updateSelection()
 {
   QModelIndex index;
-  mView->setSelectionMode( QAbstractItemView::MultiSelection );
-
   QItemSelection selection;
 
   QgsFeatureIds::Iterator it = mSelectedFeatures.begin();
@@ -264,7 +285,6 @@ void QgsAttributeTableDialog::updateSelection()
 
   mSelectionModel->select( selection, QItemSelectionModel::ClearAndSelect );// | QItemSelectionModel::Columns);
   mView->setSelectionModel( mSelectionModel );
-  mView->setSelectionMode( QAbstractItemView::NoSelection );
 
   /*for (int i = 0; i < mModel->rowCount(); ++i)
   {
@@ -275,127 +295,208 @@ void QgsAttributeTableDialog::updateSelection()
   */
 }
 
+void QgsAttributeTableDialog::updateRowPressed( int index )
+{
+  mView->setSelectionMode( QAbstractItemView::ExtendedSelection );
+  mIndexPressed = index;
+}
+
 void QgsAttributeTableDialog::updateRowSelection( int index )
 {
-  // map index to filter model
-  //index = mFilterModel->mapFromSource(mModel->index(index, 0)).row();
+  bool bDrag;
 
-  if ( mView->shiftPressed() )
+  if ( mIndexPressed == index )
+    bDrag = false;
+  else
+    bDrag = true;
+
+  QString key = "";
+  if ( QApplication::keyboardModifiers() == Qt::ControlModifier )
+    key = "Control";
+  else if ( QApplication::keyboardModifiers() == Qt::ShiftModifier )
+    key = "Shift";
+
+  int first, last;
+
+  if ( bDrag )
   {
-    QgsDebugMsg( "shift" );
-    // get the first and last index of the rows to be selected/deselected
-    int first, last;
-    if ( index > mLastClickedHeaderIndex )
+    if ( mIndexPressed < index )
     {
-      first = mLastClickedHeaderIndex + 1;
-      last = index;
-    }
-    else if ( index == mLastClickedHeaderIndex )
-    {
-      // row was selected and now it is shift-clicked
-      // ignore the shift and deselect the row
-      first = last = index;
+      first = mIndexPressed;
+      mLastClickedHeaderIndex = last = index;
     }
     else
     {
-      first = index;
-      last = mLastClickedHeaderIndex - 1;
+      last = mIndexPressed;
+      mLastClickedHeaderIndex = first = index;
     }
 
-    // for all the rows update the selection, without starting a new selection
-    if ( first <= last )
-      updateRowSelection( first, last, false );
-
-    mLastClickedHeaderIndex = last;
+    updateRowSelection( first, last, 3 );
+    mView->setSelectionMode( QAbstractItemView::NoSelection );
+    return;
   }
-  else if ( mView->ctrlPressed() )
+  else // No drag
   {
-    QgsDebugMsg( "ctrl" );
-    // update the single row selection, without starting a new selection
-    updateRowSelection( index, index, false );
+    if ( key == "Shift" )
+    {
+      QgsDebugMsg( "shift" );
+      // get the first and last index of the rows to be selected/deselected
+      first = last = 0;
 
-    // the next shift would start from here
-    mLastClickedHeaderIndex = index;
-  }
-  else
-  {
-    QgsDebugMsg( "ordinary click" );
-    // update the single row selection, start a new selection if the row was not selected
-    updateRowSelection( index, index, true );
+      if ( index > mLastClickedHeaderIndex )
+      {
+        first = mLastClickedHeaderIndex;
+        last = index;
+      }
+      else if ( index == mLastClickedHeaderIndex )
+      {
+        // row was selected and now it is shift-clicked
+        first = last = index;
+      }
+      else
+      {
+        first = index;
+        last = mLastClickedHeaderIndex;
+      }
 
-    // the next shift would start from here
-    mLastClickedHeaderIndex = index;
+      // for all the rows update the selection, without starting a new selection
+      if ( first <= last )
+        updateRowSelection( first, last, 1 );
+    }
+    else if ( key == "Control" )
+    {
+      QgsDebugMsg( "ctrl" );
+      // update the single row selection, without starting a new selection
+      updateRowSelection( index, index, 2 );
+
+      // the next shift would start from here
+      mLastClickedHeaderIndex = index;
+    }
+    else // Single click
+    {
+      // Start a new selection if the row was not selected
+      updateRowSelection( index, index, 0 );
+
+      // the next shift would start from here
+      mLastClickedHeaderIndex = index;
+    }
   }
+  mView->setSelectionMode( QAbstractItemView::NoSelection );
 }
 
-// fast row deselection needed
-void QgsAttributeTableDialog::updateRowSelection( int first, int last, bool startNewSelection )
+void QgsAttributeTableDialog::updateRowSelection( int first, int last, int clickType )
 {
+  // clickType= 0:Single click, 1:Shift, 2:Ctrl, 3: Dragged click
   disconnect( mLayer, SIGNAL( selectionChanged() ), this, SLOT( updateSelectionFromLayer() ) );
 
-  //index = mFilterModel->mapFromSource(mModel->index(index, 0)).row();
   // Id must be mapped to table/view row
   QModelIndex index = mFilterModel->mapToSource( mFilterModel->index( first, 0 ) );
   int fid = mModel->rowToId( index.row() );
   bool wasSelected = mSelectedFeatures.contains( fid );
 
   // new selection should be created
-  if ( startNewSelection )
+  if ( clickType == 0 ) // Single click
   {
-    mView->clearSelection();
-    mSelectedFeatures.clear();
+    if ( mSelectedFeatures.size() == 1 && wasSelected ) // One item selected
+      return; // Click over a selected item doesn't do anything
 
-    if ( wasSelected )
-    {
-      mLayer->removeSelection();
-      connect( mLayer, SIGNAL( selectionChanged() ), this, SLOT( updateSelectionFromLayer() ) );
-      return;
-    }
-
-    // set clicked row to current
     mView->setCurrentIndex( mFilterModel->index( first, 0 ) );
-    mView->setSelectionMode( QAbstractItemView::SingleSelection );
-
-    //QModelIndex index = mFilterModel->mapFromSource(mModel->index(first, 0));
-
     mView->selectRow( first );
-    mView->setSelectionMode( QAbstractItemView::NoSelection );
 
+    mSelectedFeatures.clear();
     mSelectedFeatures.insert( fid );
-    //mLayer->setSelectedFeatures(mSelectedFeatures);
     mLayer->removeSelection();
     mLayer->select( fid );
-    //mFilterModel->invalidate();
     connect( mLayer, SIGNAL( selectionChanged() ), this, SLOT( updateSelectionFromLayer() ) );
     return;
   }
-
-  // existing selection should be updated
-  for ( int i = first; i <= last; ++i )
+  else if ( clickType == 1 ) // Shift
   {
-    if ( i > first )
+    QgsFeatureIds newSelection;
+
+    for ( int i = first; i <= last; ++i )
     {
-      // Id must be mapped to table/view row
-      index = mFilterModel->mapToSource( mFilterModel->index( i, 0 ) );
-      fid = mModel->rowToId( index.row() );
-      wasSelected = mSelectedFeatures.contains( fid );
+      if ( i >= first )
+      {
+        // Id must be mapped to table/view row
+        index = mFilterModel->mapToSource( mFilterModel->index( i, 0 ) );
+        fid = mModel->rowToId( index.row() );
+      }
+      newSelection.insert( fid );
     }
 
+    // Remove items in mSelectedFeatures if they aren't in mNewSelection
+    QgsFeatureIds::Iterator it = mSelectedFeatures.begin();
+    while ( it != mSelectedFeatures.end() )
+    {
+      if ( !newSelection.contains( *it ) )
+      {
+        it = mSelectedFeatures.erase( it );
+      }
+      else
+      {
+        ++it;
+      }
+    }
+
+    // Append the other fids in range first-last to mSelectedFeatures
+    QgsFeatureIds::Iterator itNew = newSelection.begin();
+    for ( ; itNew != newSelection.end(); ++itNew )
+    {
+      if ( !mSelectedFeatures.contains( *itNew ) )
+      {
+        mSelectedFeatures.insert( *itNew );
+      }
+    }
+  }
+  else if ( clickType == 2 ) // Ctrl
+  {
+    // existing selection should be updated
     if ( wasSelected )
       mSelectedFeatures.remove( fid );
     else
       mSelectedFeatures.insert( fid );
   }
-  //mFilterModel->invalidate();
+  else if ( clickType == 3 ) // Dragged click
+  {
+    QgsFeatureIds newSelection;
+
+    for ( int i = first; i <= last; ++i )
+    {
+      if ( i >= first )
+      {
+        // Id must be mapped to table/view row
+        index = mFilterModel->mapToSource( mFilterModel->index( i, 0 ) );
+        fid = mModel->rowToId( index.row() );
+      }
+      newSelection.insert( fid );
+    }
+
+    // Remove items in mSelectedFeatures if they aren't in mNewSelection
+    QgsFeatureIds::Iterator it = mSelectedFeatures.begin();
+    while ( it != mSelectedFeatures.end() )
+    {
+      if ( !newSelection.contains( *it ) )
+      {
+        it = mSelectedFeatures.erase( it );
+      }
+      else
+      {
+        ++it;
+      }
+    }
+
+    // Append the other fids in range first-last to mSelectedFeatures
+    QgsFeatureIds::Iterator itNew = newSelection.begin();
+    for ( ; itNew != newSelection.end(); ++itNew )
+    {
+      if ( !mSelectedFeatures.contains( *itNew ) )
+      {
+        mSelectedFeatures.insert( *itNew );
+      }
+    }
+  }
 
-  /*
-  QItemSelection selection;
-  QModelIndex leftUpIndex = mFilterModel->index(first, 0);
-  QModelIndex rightBottomIndex = mFilterModel->index(last, mModel->columnCount() - 1);
-  selection.append(QItemSelectionRange(leftUpIndex, rightBottomIndex));
-  mSelectionModel->select(selection, QItemSelectionModel::Select);
-  mView->setSelectionModel(mSelectionModel);
-  */
   updateSelection();
   mLayer->setSelectedFeatures( mSelectedFeatures );
   connect( mLayer, SIGNAL( selectionChanged() ), this, SLOT( updateSelectionFromLayer() ) );
@@ -405,6 +506,10 @@ void QgsAttributeTableDialog::updateSelectionFromLayer()
 {
   QgsDebugMsg( "updateFromLayer" );
   mSelectedFeatures = mLayer->selectedFeaturesIds();
+
+  if ( cbxShowSelectedOnly->isChecked() )
+    mFilterModel->invalidate();
+
   updateSelection();
 }
 
@@ -478,7 +583,7 @@ void QgsAttributeTableDialog::doSearch( QString searchString )
   else
     str = tr( "No matching features found." );
 
-  QMessageBox::information( this, tr( "Search results" ), str );
+  QgisApp::instance()->statusBar()->showMessage( str );
 }
 
 void QgsAttributeTableDialog::search()
@@ -522,6 +627,15 @@ void QgsAttributeTableDialog::editingToggled()
   mToggleEditingButton->setChecked( mLayer->isEditable() );
   mToggleEditingButton->blockSignals( false );
 
+  bool canChangeAttributes = mLayer->dataProvider()->capabilities() & QgsVectorDataProvider::ChangeAttributeValues;
+  bool canDeleteFeatures = mLayer->dataProvider()->capabilities() & QgsVectorDataProvider::DeleteFeatures;
+  bool canAddAttributes = mLayer->dataProvider()->capabilities() & QgsVectorDataProvider::AddAttributes;
+  bool canDeleteAttributes = mLayer->dataProvider()->capabilities() & QgsVectorDataProvider::DeleteAttributes;
+  mOpenFieldCalculator->setEnabled( canChangeAttributes && mLayer->isEditable() );
+  mDeleteSelectedButton->setEnabled( canDeleteFeatures && mLayer->isEditable() );
+  mAddAttribute->setEnabled( canAddAttributes && mLayer->isEditable() );
+  mRemoveAttribute->setEnabled( canDeleteAttributes && mLayer->isEditable() );
+
   // (probably reload data if user stopped editing - possible revert)
   mModel->reload( mModel->index( 0, 0 ), mModel->index( mModel->rowCount(), mModel->columnCount() ) );
 
@@ -548,3 +662,79 @@ void QgsAttributeTableDialog::revert()
   mModel->revert();
   mModel->reload( mModel->index( 0, 0 ), mModel->index( mModel->rowCount(), mModel->columnCount() ) );
 }
+
+void QgsAttributeTableDialog::on_mAddAttribute_clicked()
+{
+  if ( !mLayer )
+  {
+    return;
+  }
+
+  QgsAddAttrDialog dialog( mLayer->dataProvider(), this );
+  if ( dialog.exec() == QDialog::Accepted )
+  {
+    mLayer->beginEditCommand( tr( "Attribute added" ) );
+    if ( mLayer->addAttribute( dialog.field() ) )
+    {
+      mLayer->endEditCommand();
+    }
+    else
+    {
+      QMessageBox::critical( 0, tr( "Attribute Error" ), tr( "The attribute could not be added to the layer" ) );
+      mLayer->destroyEditCommand();
+    }
+    // update model - a field has been added or updated
+    mModel->reload( mModel->index( 0, 0 ), mModel->index( mModel->rowCount(), mModel->columnCount() ) );
+  }
+}
+
+void QgsAttributeTableDialog::on_mRemoveAttribute_clicked()
+{
+  if ( !mLayer )
+  {
+    return;
+  }
+
+  QgsDelAttrDialog dialog( mLayer );
+  if ( dialog.exec() == QDialog::Accepted )
+  {
+    QList<int> attributes = dialog.selectedAttributes();
+    if ( attributes.size() < 1 )
+    {
+      return;
+    }
+
+    mLayer->beginEditCommand( tr( "Deleted attribute" ) );
+    bool deleted = false;
+    QList<int>::const_iterator it = attributes.constBegin();
+    for ( ; it != attributes.constEnd(); ++it )
+    {
+      if ( mLayer->deleteAttribute( *it ) )
+      {
+        deleted = true;
+      }
+    }
+
+    if ( deleted )
+    {
+      mLayer->endEditCommand();
+    }
+    else
+    {
+      QMessageBox::critical( 0, tr( "Attribute Error" ), tr( "The attribute(s) could not be deleted" ) );
+      mLayer->destroyEditCommand();
+    }
+    // update model - a field has been added or updated
+    mModel->reload( mModel->index( 0, 0 ), mModel->index( mModel->rowCount(), mModel->columnCount() ) );
+  }
+}
+
+void QgsAttributeTableDialog::on_mOpenFieldCalculator_clicked()
+{
+  QgsFieldCalculator calc( mLayer );
+  if ( calc.exec() == QDialog::Accepted )
+  {
+    // update model - a field has been added or updated
+    mModel->reload( mModel->index( 0, 0 ), mModel->index( mModel->rowCount(), mModel->columnCount() ) );
+  }
+}
