@@ -21,6 +21,8 @@
 #include "qgscomposerview.h"
 #include "qgscomposition.h"
 #include "qgscompositionwidget.h"
+#include "qgscomposerarrow.h"
+#include "qgscomposerarrowwidget.h"
 #include "qgscomposerlabel.h"
 #include "qgscomposerlabelwidget.h"
 #include "qgscomposerlegend.h"
@@ -31,6 +33,8 @@
 #include "qgscomposerpicturewidget.h"
 #include "qgscomposerscalebar.h"
 #include "qgscomposerscalebarwidget.h"
+#include "qgscomposershape.h"
+#include "qgscomposershapewidget.h"
 #include "qgsexception.h"
 #include "qgsproject.h"
 #include "qgsmapcanvas.h"
@@ -65,10 +69,12 @@
 #include <QSizeGrip>
 #include "qgslogger.h"
 
-QgsComposer::QgsComposer( QgisApp *qgis ): QMainWindow()
+QgsComposer::QgsComposer( QgisApp *qgis, const QString& title ): QMainWindow(), mTitle( title )
 {
   setupUi( this );
+  setWindowTitle( mTitle );
   setupTheme();
+  QObject::connect( mButtonBox, SIGNAL( rejected() ), this, SLOT( close() ) );
 
   QToolButton* orderingToolButton = new QToolButton( this );
   orderingToolButton->setPopupMode( QToolButton::InstantPopup );
@@ -104,10 +110,10 @@ QgsComposer::QgsComposer( QgisApp *qgis ): QMainWindow()
   toggleActionGroup->addAction( mActionAddNewScalebar );
   toggleActionGroup->addAction( mActionAddImage );
   toggleActionGroup->addAction( mActionSelectMoveItem );
+  toggleActionGroup->addAction( mActionAddBasicShape );
+  toggleActionGroup->addAction( mActionAddArrow );
   toggleActionGroup->setExclusive( true );
 
-  setWindowTitle( tr( "QGIS - print composer" ) );
-
   mActionAddNewMap->setCheckable( true );
   mActionAddNewLabel->setCheckable( true );
   mActionAddNewLegend->setCheckable( true );
@@ -115,31 +121,27 @@ QgsComposer::QgsComposer( QgisApp *qgis ): QMainWindow()
   mActionAddNewScalebar->setCheckable( true );
   mActionAddImage->setCheckable( true );
   mActionMoveItemContent->setCheckable( true );
+  mActionAddBasicShape->setCheckable( true );
+  mActionAddArrow->setCheckable( true );
 
 #ifdef Q_WS_MAC
   QMenu *appMenu = menuBar()->addMenu( tr( "QGIS" ) );
   appMenu->addAction( QgisApp::instance()->actionAbout() );
   appMenu->addAction( QgisApp::instance()->actionOptions() );
+#endif
 
   QMenu *fileMenu = menuBar()->addMenu( tr( "File" ) );
-  QAction *closeAction = fileMenu->addAction( tr( "Close" ), this, SLOT( close() ), tr( "Ctrl+W" ) );
+  fileMenu->addAction( mActionLoadFromTemplate );
+  fileMenu->addAction( mActionSaveAsTemplate );
+  fileMenu->addSeparator();
   fileMenu->addAction( mActionExportAsImage );
+  fileMenu->addAction( mActionExportAsPDF );
   fileMenu->addAction( mActionExportAsSVG );
   fileMenu->addSeparator();
   fileMenu->addAction( mActionPrint );
-
-  QMenu *editMenu = menuBar()->addMenu( tr( "Edit" ) );
-  QAction *undoAction = editMenu->addAction( tr( "&Undo" ), this, SLOT( undo() ), tr( "Ctrl+Z" ) );
-  undoAction->setEnabled( false );
-  editMenu->addSeparator();
-  QAction *cutAction = editMenu->addAction( tr( "Cu&t" ), this, SLOT( cut() ), tr( "Ctrl+X" ) );
-  cutAction->setEnabled( false );
-  QAction *copyAction = editMenu->addAction( tr( "&Copy" ), this, SLOT( copy() ), tr( "Ctrl+C" ) );
-  copyAction->setEnabled( false );
-  QAction *pasteAction = editMenu->addAction( tr( "&Paste" ), this, SLOT( paste() ), tr( "Ctrl+V" ) );
-  pasteAction->setEnabled( false );
-  QAction *deleteAction = editMenu->addAction( tr( "Delete" ) );
-  deleteAction->setEnabled( false );
+  fileMenu->addSeparator();
+  fileMenu->addAction( mActionQuit );
+  QObject::connect( mActionQuit, SIGNAL( triggered() ), this, SLOT( close() ) );
 
   QMenu *viewMenu = menuBar()->addMenu( tr( "View" ) );
   viewMenu->addAction( mActionZoomIn );
@@ -156,6 +158,8 @@ QgsComposer::QgsComposer( QgisApp *qgis ): QMainWindow()
   layoutMenu->addAction( mActionAddImage );
   layoutMenu->addAction( mActionSelectMoveItem );
   layoutMenu->addAction( mActionMoveItemContent );
+  layoutMenu->addAction( mActionAddBasicShape );
+  layoutMenu->addAction( mActionAddArrow );
   layoutMenu->addSeparator();
   layoutMenu->addAction( mActionGroupItems );
   layoutMenu->addAction( mActionUngroupItems );
@@ -164,20 +168,21 @@ QgsComposer::QgsComposer( QgisApp *qgis ): QMainWindow()
   layoutMenu->addAction( mActionMoveItemsToTop );
   layoutMenu->addAction( mActionMoveItemsToBottom );
 
+#ifdef Q_WS_MAC
 #ifndef Q_WS_MAC64 /* assertion failure in NSMenuItem setSubmenu (Qt 4.5.0-snapshot-20080830) */
   menuBar()->addMenu( QgisApp::instance()->windowMenu() );
 
   menuBar()->addMenu( QgisApp::instance()->helpMenu() );
 #endif
-
-  // Create action to select this window and add it to Window menu
-  mWindowAction = new QAction( windowTitle(), this );
-  connect( mWindowAction, SIGNAL( triggered() ), this, SLOT( activate() ) );
 #endif
 
   mQgis = qgis;
   mFirstTime = true;
 
+  // Create action to select this window
+  mWindowAction = new QAction( windowTitle(), this );
+  connect( mWindowAction, SIGNAL( triggered() ), this, SLOT( activate() ) );
+
   QgsDebugMsg( "entered." );
 
   setMouseTracking( true );
@@ -189,17 +194,17 @@ QgsComposer::QgsComposer( QgisApp *qgis ): QMainWindow()
   connectSlots();
 
   mComposition  = new QgsComposition( mQgis->mapCanvas()->mapRenderer() );
+  mComposition->setParent( mView );
   mView->setComposition( mComposition );
 
   QgsCompositionWidget* compositionWidget = new QgsCompositionWidget( mCompositionOptionsFrame, mComposition );
+  QObject::connect( mComposition, SIGNAL( paperSizeChanged() ), compositionWidget, SLOT( displayCompositionWidthHeight() ) );
   compositionWidget->show();
 
   mCompositionOptionsLayout = new QGridLayout( mCompositionOptionsFrame );
   mCompositionOptionsLayout->setMargin( 0 );
   mCompositionOptionsLayout->addWidget( compositionWidget );
 
-  mPrinter = 0;
-
   QGridLayout *l = new QGridLayout( mViewFrame );
   l->setMargin( 0 );
   l->addWidget( mView, 0, 0 );
@@ -216,29 +221,23 @@ QgsComposer::QgsComposer( QgisApp *qgis ): QMainWindow()
 
   mView->setFocus();
 
-  //connect with signals from QgsProject to read/write project files
+  //connect with signals from QgsProject to write project files
   if ( QgsProject::instance() )
   {
-    connect( QgsProject::instance(), SIGNAL( readProject( const QDomDocument& ) ), this, SLOT( readXML( const QDomDocument& ) ) );
     connect( QgsProject::instance(), SIGNAL( writeProject( QDomDocument& ) ), this, SLOT( writeXML( QDomDocument& ) ) );
   }
 }
 
 QgsComposer::~QgsComposer()
 {
-  //delete all the items
-  QMap<QgsComposerItem*, QWidget*>::iterator it = mItemWidgetMap.begin();
-  for ( ; it != mItemWidgetMap.end(); ++it )
-  {
-    delete it.key();
-    delete it.value();
-  }
+  deleteItems();
 }
 
 void QgsComposer::setupTheme()
 {
   //now set all the icons - getThemeIcon will fall back to default theme if its
   //missing from active theme
+  mActionQuit->setIcon( QgisApp::getThemeIcon( "/mActionFileExit.png" ) );
   mActionLoadFromTemplate->setIcon( QgisApp::getThemeIcon( "/mActionFileOpen.png" ) );
   mActionSaveAsTemplate->setIcon( QgisApp::getThemeIcon( "/mActionFileSaveAs.png" ) );
   mActionExportAsImage->setIcon( QgisApp::getThemeIcon( "/mActionSaveMapAsImage.png" ) );
@@ -254,6 +253,8 @@ void QgsComposer::setupTheme()
   mActionAddNewLabel->setIcon( QgisApp::getThemeIcon( "/mActionLabel.png" ) );
   mActionAddNewLegend->setIcon( QgisApp::getThemeIcon( "/mActionAddLegend.png" ) );
   mActionAddNewScalebar->setIcon( QgisApp::getThemeIcon( "/mActionScaleBar.png" ) );
+  mActionAddBasicShape->setIcon( QgisApp::getThemeIcon( "/mActionAddBasicShape.png" ) );
+  mActionAddArrow->setIcon( QgisApp::getThemeIcon( "/mActionAddArrow.png" ) );
   mActionSelectMoveItem->setIcon( QgisApp::getThemeIcon( "/mActionSelectPan.png" ) );
   mActionMoveItemContent->setIcon( QgisApp::getThemeIcon( "/mActionMoveItemContent.png" ) );
   mActionGroupItems->setIcon( QgisApp::getThemeIcon( "/mActionGroupItems.png" ) );
@@ -279,6 +280,8 @@ void QgsComposer::connectSlots()
   connect( mView, SIGNAL( composerScaleBarAdded( QgsComposerScaleBar* ) ), this, SLOT( addComposerScaleBar( QgsComposerScaleBar* ) ) );
   connect( mView, SIGNAL( composerLegendAdded( QgsComposerLegend* ) ), this, SLOT( addComposerLegend( QgsComposerLegend* ) ) );
   connect( mView, SIGNAL( composerPictureAdded( QgsComposerPicture* ) ), this, SLOT( addComposerPicture( QgsComposerPicture* ) ) );
+  connect( mView, SIGNAL( composerShapeAdded( QgsComposerShape* ) ), this, SLOT( addComposerShape( QgsComposerShape* ) ) );
+  connect( mView, SIGNAL( composerArrowAdded( QgsComposerArrow* ) ), this, SLOT( addComposerArrow( QgsComposerArrow* ) ) );
   connect( mView, SIGNAL( actionFinished() ), this, SLOT( setSelectionTool() ) );
 }
 
@@ -302,6 +305,7 @@ void QgsComposer::open( void )
 
 void QgsComposer::activate()
 {
+  show();
   raise();
   setWindowState( windowState() & ~Qt::WindowMinimized );
   activateWindow();
@@ -325,15 +329,6 @@ void QgsComposer::changeEvent( QEvent* event )
   }
 }
 
-void QgsComposer::closeEvent( QCloseEvent *event )
-{
-  QMainWindow::closeEvent( event );
-  if ( event->isAccepted() )
-  {
-    QgisApp::instance()->removeWindow( mWindowAction );
-  }
-}
-
 void QgsComposer::showEvent( QShowEvent *event )
 {
   QMainWindow::showEvent( event );
@@ -345,6 +340,16 @@ void QgsComposer::showEvent( QShowEvent *event )
 }
 #endif
 
+void QgsComposer::setTitle( const QString& title )
+{
+  mTitle = title;
+  setWindowTitle( mTitle );
+  if ( mWindowAction )
+  {
+    mWindowAction->setText( title );
+  }
+}
+
 void QgsComposer::showCompositionOptions( QWidget *w )
 {
   QWidget* currentWidget = mItemStackedWidget->currentWidget();
@@ -428,15 +433,29 @@ void QgsComposer::on_mActionZoomOut_triggered()
 
 void QgsComposer::on_mActionRefreshView_triggered()
 {
-  if ( mComposition )
+  if ( !mComposition )
   {
-    mComposition->update();
+    return;
   }
+
+  //refresh preview of all composer maps
+  QMap<QgsComposerItem*, QWidget*>::iterator it = mItemWidgetMap.begin();
+  for ( ; it != mItemWidgetMap.end(); ++it )
+  {
+    QgsComposerMap* map = dynamic_cast<QgsComposerMap*>( it.key() );
+    if ( map && !map->isDrawing() )
+    {
+      map->cache();
+      map->update();
+    }
+  }
+
+  mComposition->update();
 }
 
 void QgsComposer::on_mActionExportAsPDF_triggered()
 {
-  QSettings myQSettings;  // where we keep last used filter in persistant state
+  QSettings myQSettings;  // where we keep last used filter in persistent state
   QString myLastUsedFile = myQSettings.value( "/UI/lastSaveAsPdfFile", "qgis.pdf" ).toString();
   QFileInfo file( myLastUsedFile );
   QFileDialog *myQFileDialog = new QFileDialog( this, tr( "Choose a file name to save the map as" ),
@@ -458,6 +477,7 @@ void QgsComposer::on_mActionExportAsPDF_triggered()
 
   printer.setOutputFormat( QPrinter::PdfFormat );
   printer.setOutputFileName( myOutputFileNameQString );
+  printer.setPaperSize( QSizeF( mComposition->paperWidth(), mComposition->paperHeight() ), QPrinter::Millimeter );
 
   print( printer );
 }
@@ -465,10 +485,23 @@ void QgsComposer::on_mActionExportAsPDF_triggered()
 void QgsComposer::on_mActionPrint_triggered()
 {
   QPrinter printer;
-
-  QPrintDialog printDialog( &printer );
+  if ( mComposition )
+  {
+    if ( mComposition->paperWidth() >= mComposition->paperHeight() )
+    {
+      printer.setOrientation( QPrinter::Landscape );
+    }
+    else
+    {
+      printer.setOrientation( QPrinter::Portrait );
+    }
+  }
+  printer.setPaperSize( QSizeF( mComposition->paperWidth(), mComposition->paperHeight() ), QPrinter::Millimeter );
+  QPrintDialog printDialog( &printer, 0 );
   if ( printDialog.exec() != QDialog::Accepted )
+  {
     return;
+  }
 
   print( printer );
 }
@@ -483,16 +516,6 @@ void QgsComposer::print( QPrinter &printer )
     showWMSPrintingWarning();
   }
 
-  //try to set most of the print dialog settings based on composer properties
-  if ( mComposition->paperHeight() > mComposition->paperWidth() )
-  {
-    printer.setOrientation( QPrinter::Portrait );
-  }
-  else
-  {
-    printer.setOrientation( QPrinter::Landscape );
-  }
-
   //set resolution based on composer setting
   printer.setFullPage( true );
   printer.setColorMode( QPrinter::Color );
@@ -574,7 +597,7 @@ void QgsComposer::on_mActionExportAsImage_triggered()
   FilterMap myFilterMap;
 
   //find out the last used filter
-  QSettings myQSettings;  // where we keep last used filter in persistant state
+  QSettings myQSettings;  // where we keep last used filter in persistent state
   QString myLastUsedFormat = myQSettings.value( "/UI/lastSaveAsImageFormat", "png" ).toString();
   QString myLastUsedFile = myQSettings.value( "/UI/lastSaveAsImageFile", "qgis.png" ).toString();
   QFileInfo file( myLastUsedFile );
@@ -641,6 +664,12 @@ void QgsComposer::on_mActionExportAsImage_triggered()
   QgsDebugMsg( QString( "Selected filter: %1" ).arg( myFilterString ) );
   QgsDebugMsg( QString( "Image type: %1" ).arg( myFilterMap[myFilterString] ) );
 
+  // Add the file type suffix to the fileName if required
+  if ( !myOutputFileNameQString.endsWith( myFilterMap[myFilterString] ) )
+  {
+    myOutputFileNameQString += "." + myFilterMap[myFilterString];
+  }
+
   myQSettings.setValue( "/UI/lastSaveAsImageFormat", myFilterMap[myFilterString] );
   myQSettings.setValue( "/UI/lastSaveAsImageFile", myOutputFileNameQString );
 
@@ -687,16 +716,9 @@ void QgsComposer::on_mActionExportAsSVG_triggered()
     m->setCheckBoxQSettingsLabel( myQSettingsLabel );
     m->setMessageAsHtml( tr( "<p>The SVG export function in Qgis has several "
                              "problems due to bugs and deficiencies in the " )
-#if QT_VERSION < 0x040300
-                         + tr( "Qt4 svg code. Of note, text does not "
-                               "appear in the SVG file and there are problems "
-                               "with the map bounding box clipping other items "
-                               "such as the legend or scale bar.</p>" )
-#else
                          + tr( "Qt4 svg code. In particular, there are problems "
                                "with layers not being clipped to the map "
                                "bounding box.</p>" )
-#endif
                          + tr( "If you require a vector-based output file from "
                                "Qgis it is suggested that you try printing "
                                "to PostScript if the SVG output is not "
@@ -724,30 +746,28 @@ void QgsComposer::on_mActionExportAsSVG_triggered()
   //mView->setScene(0);//don't redraw the scene on the display while we render
   mComposition->setPlotStyle( QgsComposition::Print );
 
-#if QT_VERSION < 0x040300
-  Q3Picture pic;
-  QPainter p( &pic );
-  QRectF renderArea( 0, 0, ( mComposition->paperWidth() * mComposition->scale() ), ( mComposition->paperHeight() * mComposition->scale() ) );
-#else
   QSvgGenerator generator;
+#if QT_VERSION >= 0x040500
+  generator.setTitle( QgsProject::instance()->title() );
+#endif
   generator.setFileName( myOutputFileNameQString );
-  generator.setSize( QSize(( int )mComposition->paperWidth(), ( int )mComposition->paperHeight() ) );
-  generator.setResolution( 25.4 ); //because the rendering is done in mm, convert the dpi
+  //width in pixel
+  int width = ( int )( mComposition->paperWidth() * mComposition->printResolution() / 25.4 );
+  //height in pixel
+  int height = ( int )( mComposition->paperHeight() * mComposition->printResolution() / 25.4 );
+  generator.setSize( QSize( width, height ) );
+  generator.setResolution( mComposition->printResolution() ); //because the rendering is done in mm, convert the dpi
 
   QPainter p( &generator );
-  QRectF renderArea( 0, 0, mComposition->paperWidth(), mComposition->paperHeight() );
-#endif
-  mComposition->render( &p, renderArea, renderArea );
+
+  QRectF sourceArea( 0, 0, mComposition->paperWidth(), mComposition->paperHeight() );
+  QRectF targetArea( 0, 0, width, height );
+  mComposition->render( &p, targetArea, sourceArea );
+
   p.end();
 
   mComposition->setPlotStyle( QgsComposition::Preview );
   //mView->setScene(mComposition->canvas()); //now that we're done, set the view to show the scene again
-
-#if QT_VERSION < 0x040300
-  QRect br = pic.boundingRect();
-
-  pic.save( myOutputFileNameQString, "svg" );
-#endif
 }
 
 void QgsComposer::on_mActionSelectMoveItem_triggered()
@@ -798,6 +818,22 @@ void QgsComposer::on_mActionAddImage_triggered()
   }
 }
 
+void QgsComposer::on_mActionAddBasicShape_triggered()
+{
+  if ( mView )
+  {
+    mView->setCurrentTool( QgsComposerView::AddShape );
+  }
+}
+
+void QgsComposer::on_mActionAddArrow_triggered()
+{
+  if ( mView )
+  {
+    mView->setCurrentTool( QgsComposerView::AddArrow );
+  }
+}
+
 void QgsComposer::on_mActionSaveAsTemplate_triggered()
 {
   //show file dialog
@@ -861,6 +897,7 @@ void QgsComposer::on_mActionLoadFromTemplate_triggered()
     return;
   }
 
+  deleteItems();
   readXML( templateDocument );
 
   //clean up after template read (e.g. legend and map extent)
@@ -1005,16 +1042,6 @@ void QgsComposer::restoreWindowState()
   }
 }
 
-void QgsComposer::on_helpPButton_clicked()
-{
-  QgsContextHelp::run( context_id );
-}
-
-void QgsComposer::on_closePButton_clicked()
-{
-  close();
-}
-
 void  QgsComposer::writeXML( QDomDocument& doc )
 {
 
@@ -1035,6 +1062,17 @@ void  QgsComposer::writeXML( QDomDocument& doc )
 void QgsComposer::writeXML( QDomNode& parentNode, QDomDocument& doc )
 {
   QDomElement composerElem = doc.createElement( "Composer" );
+  composerElem.setAttribute( "title", mTitle );
+
+  //store if composer is open or closed
+  if ( isVisible() )
+  {
+    composerElem.setAttribute( "visible", 1 );
+  }
+  else
+  {
+    composerElem.setAttribute( "visible", 0 );
+  }
   parentNode.appendChild( composerElem );
 
   //store composer items:
@@ -1055,22 +1093,25 @@ void QgsComposer::writeXML( QDomNode& parentNode, QDomDocument& doc )
 
 void QgsComposer::readXML( const QDomDocument& doc )
 {
-  //look for Composer element
-  QDomNodeList nl = doc.elementsByTagName( "Composer" );
-  if ( nl.size() < 1 )
+  QDomNodeList composerNodeList = doc.elementsByTagName( "Composer" );
+  if ( composerNodeList.size() < 1 )
   {
-    return; //nothing to do...
+    return;
   }
-  QDomElement composerElem = nl.at( 0 ).toElement();
+  readXML( composerNodeList.at( 0 ).toElement(), doc, true );
+}
 
-  //look for Composition element
-  QDomNodeList cnl = composerElem.elementsByTagName( "Composition" );
-  if ( cnl.size() < 1 )
+void QgsComposer::readXML( const QDomElement& composerElem, const QDomDocument& doc, bool fromTemplate )
+{
+  // Set title only if reading from project file
+  if ( !fromTemplate )
   {
-    return; //nothing to do
+    if ( composerElem.hasAttribute( "title" ) )
+    {
+      setTitle( composerElem.attribute( "title", tr( "Composer" ) ) );
+    }
   }
 
-
   //delete composer view and composition
   delete mView;
   mView = 0;
@@ -1082,19 +1123,22 @@ void QgsComposer::readXML( const QDomDocument& doc )
     delete( *it );
   }
   //delete composition widget
-  QgsCompositionWidget* oldCompositionWidget = dynamic_cast<QgsCompositionWidget*>( mCompositionOptionsFrame->children().at( 0 ) );
+  QgsCompositionWidget* oldCompositionWidget = qobject_cast<QgsCompositionWidget *>( mCompositionOptionsFrame->children().at( 0 ) );
   delete oldCompositionWidget;
   delete mCompositionOptionsLayout;
   mCompositionOptionsLayout = 0;
 
-  QDomElement compositionElem = cnl.at( 0 ).toElement();
-
-  //todo: move in function because duplicated code with constructor
   mView = new QgsComposerView( mViewFrame );
   connectSlots();
 
+  //read composition settings
   mComposition = new QgsComposition( mQgis->mapCanvas()->mapRenderer() );
-  mComposition->readXML( compositionElem, doc );
+  QDomNodeList compositionNodeList = composerElem.elementsByTagName( "Composition" );
+  if ( compositionNodeList.size() > 0 )
+  {
+    QDomElement compositionElem = compositionNodeList.at( 0 ).toElement();
+    mComposition->readXML( compositionElem, doc );
+  }
 
   QGridLayout *l = new QGridLayout( mViewFrame );
   l->setMargin( 0 );
@@ -1102,6 +1146,7 @@ void QgsComposer::readXML( const QDomDocument& doc )
 
   //create compositionwidget
   QgsCompositionWidget* compositionWidget = new QgsCompositionWidget( mCompositionOptionsFrame, mComposition );
+  QObject::connect( mComposition, SIGNAL( paperSizeChanged() ), compositionWidget, SLOT( displayCompositionWidthHeight() ) );
   compositionWidget->show();
 
   mCompositionOptionsLayout = new QGridLayout( mCompositionOptionsFrame );
@@ -1185,12 +1230,65 @@ void QgsComposer::readXML( const QDomDocument& doc )
     showItemOptions( newPicture );
   }
 
+  //composer shapes
+  QDomNodeList composerShapeList = composerElem.elementsByTagName( "ComposerShape" );
+  for ( int i = 0; i < composerShapeList.size(); ++i )
+  {
+    QDomElement currentShapeElem = composerShapeList.at( i ).toElement();
+    QgsComposerShape* newShape = new QgsComposerShape( mComposition );
+    newShape->readXML( currentShapeElem, doc );
+    addComposerShape( newShape );
+    mComposition->addItem( newShape );
+    mComposition->update();
+    mComposition->clearSelection();
+    newShape->setSelected( true );
+    showItemOptions( newShape );
+  }
+
+  //composer arrows
+  QDomNodeList composerArrowList = composerElem.elementsByTagName( "ComposerArrow" );
+  for ( int i = 0; i < composerArrowList.size(); ++i )
+  {
+    QDomElement currentArrowElem = composerArrowList.at( i ).toElement();
+    QgsComposerArrow* newArrow = new QgsComposerArrow( mComposition );
+    newArrow->readXML( currentArrowElem, doc );
+    addComposerArrow( newArrow );
+    mComposition->addItem( newArrow );
+    mComposition->update();
+    mComposition->clearSelection();
+    newArrow->setSelected( true );
+    showItemOptions( newArrow );
+  }
+
   mComposition->sortZList();
   mView->setComposition( mComposition );
 
   setSelectionTool();
 }
 
+void QgsComposer::deleteItems()
+{
+  //delete all the items
+  QMap<QgsComposerItem*, QWidget*>::iterator it = mItemWidgetMap.begin();
+  for ( ; it != mItemWidgetMap.end(); ++it )
+  {
+    delete it.key();
+    delete it.value();
+  }
+  mItemWidgetMap.clear();
+}
+
+void QgsComposer::addComposerArrow( QgsComposerArrow* arrow )
+{
+  if ( !arrow )
+  {
+    return;
+  }
+
+  QgsComposerArrowWidget* arrowWidget = new QgsComposerArrowWidget( arrow );
+  mItemWidgetMap.insert( arrow, arrowWidget );
+}
+
 void QgsComposer::addComposerMap( QgsComposerMap* map )
 {
   if ( !map )
@@ -1247,6 +1345,16 @@ void QgsComposer::addComposerPicture( QgsComposerPicture* picture )
   mItemWidgetMap.insert( picture, pWidget );
 }
 
+void QgsComposer::addComposerShape( QgsComposerShape* shape )
+{
+  if ( !shape )
+  {
+    return;
+  }
+  QgsComposerShapeWidget* sWidget = new QgsComposerShapeWidget( shape );
+  mItemWidgetMap.insert( shape, sWidget );
+}
+
 void QgsComposer::deleteItem( QgsComposerItem* item )
 {
   QMap<QgsComposerItem*, QWidget*>::iterator it = mItemWidgetMap.find( item );
@@ -1276,7 +1384,7 @@ bool QgsComposer::containsWMSLayer() const
   for ( ; item_it != mItemWidgetMap.constEnd(); ++item_it )
   {
     currentItem = item_it.key();
-    currentMap = dynamic_cast<QgsComposerMap*>( currentItem );
+    currentMap = dynamic_cast<QgsComposerMap *>( currentItem );
     if ( currentMap )
     {
       if ( currentMap->containsWMSLayer() )
@@ -1313,7 +1421,7 @@ void QgsComposer::cleanupAfterTemplateRead()
   for ( ; itemIt != mItemWidgetMap.constEnd(); ++itemIt )
   {
     //update all legends completely
-    QgsComposerLegend* legendItem = dynamic_cast<QgsComposerLegend*>( itemIt.key() );
+    QgsComposerLegend* legendItem = dynamic_cast<QgsComposerLegend *>( itemIt.key() );
     if ( legendItem )
     {
       legendItem->updateLegend();
@@ -1321,7 +1429,7 @@ void QgsComposer::cleanupAfterTemplateRead()
     }
 
     //update composer map extent if it does not intersect the full extent of all layers
-    QgsComposerMap* mapItem = dynamic_cast<QgsComposerMap*>( itemIt.key() );
+    QgsComposerMap* mapItem = dynamic_cast<QgsComposerMap *>( itemIt.key() );
     if ( mapItem )
     {
       //test if composer map extent intersects extent of all layers
@@ -1357,3 +1465,5 @@ void QgsComposer::cleanupAfterTemplateRead()
     }
   }
 }
+
+
