@@ -15,7 +15,7 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-/* $Id: qgslegend.h 11290 2009-08-07 11:58:50Z mhugent $ */
+/* $Id$ */
 
 #ifndef QGSLEGEND_H
 #define QGSLEGEND_H
@@ -25,12 +25,13 @@
 #include <set>
 #include <QTreeWidget>
 
+class QgsLegendGroup;
 class QgsLegendLayer;
-class QgsLegendLayerFile;
 class QgsLegendItem;
 class QgsMapLayer;
 class QgsMapCanvas;
 class QDomDocument;
+class QDomElement;
 class QDomNode;
 class QMouseEvent;
 class QTreeWidgetItem;
@@ -69,12 +70,6 @@ class QTreeWidgetItem;
    <li>QgsLegendPropertyItem - A list of properties related to the layer. Double clicking
                               a property item will invoke a dialog that will let you change
                               the property settings. Can only exist inside a property group</li>
-   <li>QgsLegendLayerFileGroup - each QgsLegendLayer can have one or more files associated
-                              with it. This is the container group for these files. Can
-                              only exist inside of a QgsLegendLayer.</li>
-   <li>QgsLegendLayerFile -  A file node that relates to a file on disk. Assigning multiple
-                             file nodes in a file group allows you treat them as if they are
-                             one entity.</li>
    </ul>
    @note Additional group types may be defined in the future to accommodate WMS, PostGIS etc layers.
    @author Gary E.Sherman, Tim Sutton, Marco Hugentobler and Jens Oberender
@@ -101,14 +96,18 @@ class QgsLegend : public QTreeWidget
     //! Destructor
     ~QgsLegend();
 
-    /** Returns QgsLegendLayerFile associated with current layer */
-    QgsLegendLayerFile* currentLayerFile();
+    /** Returns QgsLegendLayer associated with current layer */
+    QgsLegendLayer* currentLegendLayer();
 
-    /*!Returns the current layer if the current item is a QgsLegendLayerFile.
+    /*!Returns the current layer if the current item is a QgsLegendLayer.
     If the current item is a QgsLegendLayer, its first maplayer is returned.
     Else, 0 is returned.*/
     QgsMapLayer* currentLayer();
 
+    /*!set the current layer
+    returns true if the layer exists, false otherwise*/
+    bool setCurrentLayer( QgsMapLayer *layer );
+
     /**Writes the content of the legend to a project file*/
     bool writeXML( QDomNode & layer_node, QDomDocument & document );
 
@@ -118,6 +117,12 @@ class QgsLegend : public QTreeWidget
     /**Returns true, if the y-coordinate is >= the center of the item*/
     bool yCoordAboveCenter( QgsLegendItem* it, int ycoord );
 
+    /**Returns true, if the item at index is a QgsLegendGroup*/
+    bool isLegendGroup( const QModelIndex &index );
+
+    /**Returns a string list of groups*/
+    QStringList groups();
+
     /**Returns the first item in the hierarchy*/
     QTreeWidgetItem* firstItem();
 
@@ -168,10 +173,6 @@ class QgsLegend : public QTreeWidget
     /**Removes an entry from mPixmapHeightValues*/
     void removePixmapHeightValue( int height );
 
-    /** Sets the name of the QgsLegendLayer that is the parent of
-        the given QgsLegendLayerFile */
-    void setName( QgsLegendLayerFile* w, QString layerName );
-
     /**Sets the toggle editing action. Usually called from QgisApp*/
     void setToggleEditingAction( QAction* editingAction ) {mToggleEditingAction = editingAction;}
 
@@ -179,7 +180,7 @@ class QgsLegend : public QTreeWidget
     QgsLegendPixmaps& pixmaps() { return mPixmaps; }
 
 
-    void updateCheckStates( QTreeWidgetItem* item, Qt::CheckState state ) {mStateOfCheckBoxes[item] = state;}
+    void updateCheckStates( QTreeWidgetItem* item, Qt::CheckState state ) { item->setData( 0, Qt::UserRole, state ); }
 
   public slots:
 
@@ -208,9 +209,19 @@ class QgsLegend : public QTreeWidget
     /*!
      * Slot called when user wishes to add a new empty layer group to the legend.
      * The user will be prompted for the name of the newly added group.
+     * @param name name of the new group
+     * @param expand expand the group
      * @return void
      */
-    void addGroup();
+    int addGroup( QString name = QString(), bool expand = true );
+
+    /*!
+     * Removes all groups with the given name.
+     * @param name name of the groups to remove
+     * @return void
+     */
+    void removeGroup( int groupIndex );
+
     void removeLayer( QString );
 
     /** called to read legend settings from project */
@@ -219,8 +230,8 @@ class QgsLegend : public QTreeWidget
     /** called to write legend settings to project */
     void writeProject( QDomDocument & );
 
-    /**Removes the current LegendLayer and all its LegendLayerFiles*/
-    void legendLayerRemove();
+    /**Removes the current LegendLayer*/
+    void removeCurrentLayer();
 
     /**Removes a layer. If the layer is editable, a dialog is shown where user can select 'save', 'discard' and optionally 'cancel'. Cancel
       is useful if a single layer is removed whereas on closing of the whole project or application, the cancel option may not be possible
@@ -229,6 +240,14 @@ class QgsLegend : public QTreeWidget
       @param return false if canceled or in case of error, true else*/
     bool removeLayer( QgsMapLayer* ml, bool askCancelOnEditable );
 
+    /*!
+     * Moves a layer to a group.
+     * @param ml the maplayer to move
+     * @param groupIndex index of group
+     * @return false if the group does not exist, false otherwise
+     */
+    void moveLayer( QgsMapLayer* ml, int groupIndex );
+
     /**Toggle show in overview for current layer*/
     void legendLayerShowInOverview();
 
@@ -295,9 +314,12 @@ class QgsLegend : public QTreeWidget
     /**Moves an item back to the position where storeInitialPosition has been called*/
     void resetToInitialPosition( QTreeWidgetItem* li );
 
-    /**Returns the legend layer to which a map layer gelongs*/
+    /**Returns the legend layer to which a map layer belongs to*/
     QgsLegendLayer* findLegendLayer( const QString& layerKey );
 
+    /**Returns the legend layer to which a map layer belongs to*/
+    QgsLegendLayer* findLegendLayer( const QgsMapLayer *layer );
+
     /**Checks mPixmapWidthValues and mPixmapHeightValues and sets a new icon size if necessary*/
     void adjustIconSize();
 
@@ -332,12 +354,17 @@ class QgsLegend : public QTreeWidget
     /** Returns the last visible item in the tree widget */
     QTreeWidgetItem *lastVisibleItem();
 
+    /** read layer settings from XML element and add item */
+    QgsLegendLayer* readLayerFromXML( QDomElement& childelem, bool& isOpen );
+
   private slots:
 
     /**Calls 'handleRightClickEvent' on the item*/
     void handleRightClickEvent( QTreeWidgetItem* item, const QPoint& position );
     /**Removes the current legend group*/
     void legendGroupRemove();
+    /**Removes a legend group and its layers*/
+    void removeGroup( QgsLegendGroup * lg );
     /**Sets all listview items to open*/
     void expandAll();
     /**Sets all listview items to closed*/
@@ -350,8 +377,6 @@ class QgsLegend : public QTreeWidget
     void openEditor();
     /**Removes the current item and inserts it as a toplevel item at the end of the legend*/
     void makeToTopLevelItem();
-    /**Show/ Hide the legend layer file groups*/
-    void showLegendLayerFileGroups();
 
   private:
 
@@ -396,8 +421,6 @@ class QgsLegend : public QTreeWidget
     HIERARCHY_POSITION_TYPE mRestoreInformation;
     QTreeWidgetItem* mRestoreItem;
 
-    bool mShowLegendLayerFiles;
-
     /**Stores the layer ordering before a mouse Move. After the move, this is used to
      decide if the mapcanvas really has to be refreshed*/
     std::deque<QString> mLayersPriorToMove;
@@ -406,7 +429,7 @@ class QgsLegend : public QTreeWidget
     QAction* mToggleEditingAction;
 
     /*!
-     * A fuction sed to determin how far down in the list an item is (starting with one for the first Item.
+     * A function to determine how far down in the list an item is (starting with one for the first Item).
      *If the item is not in the legend, -1 is returned
      * @see mItemBeingMovedOrigPos
      */
@@ -415,10 +438,6 @@ class QgsLegend : public QTreeWidget
     /**Pointer to the main canvas. Used for requiring repaints in case of legend changes*/
     QgsMapCanvas* mMapCanvas;
 
-    /**Map that keeps track of which checkboxes are in which check state. This is necessary because QTreeView does not emit
-       a signal for check state changes*/
-    std::map<QTreeWidgetItem*, Qt::CheckState> mStateOfCheckBoxes;
-
     /**Stores the width values of the LegendSymbologyItem pixmaps. The purpose of this is that the legend may automatically change
      the global IconWidth when items are added or removed*/
     std::multiset<int> mPixmapWidthValues;
@@ -452,6 +471,8 @@ class QgsLegend : public QTreeWidget
     QWidget *mInsertionLine;
 
   signals:
+    void itemMoved( QModelIndex oldIndex, QModelIndex newIndex );
+
     void zOrderChanged( QgsLegend * lv );
 
     //! Emited whenever current (selected) layer changes
