@@ -14,7 +14,7 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-/* $Id: main.cpp 11588 2009-09-08 07:23:21Z timlinux $ */
+/* $Id$ */
 
 //qt includes
 #include <QBitmap>
@@ -46,14 +46,12 @@
 #ifdef MSVC
 #undef _fmode
 int _fmode = _O_BINARY;
-#endif
-#ifndef _MSC_VER
+#else
 // Only do this if we are not building on windows with msvc.
 // Recommended method for doing this with msvc is with a call to _set_fmode
 // which is the first thing we do in main().
-#undef _fmode
-int _fmode = _O_BINARY;
-#endif//_MSC_VER
+// Similarly, with MinGW set _fmode in main().
+#endif  //_MSC_VER
 #else
 #include <getopt.h>
 #endif
@@ -75,7 +73,7 @@ typedef SInt32 SRefCon;
 #include "qgsrectangle.h"
 #include "qgslogger.h"
 
-static const char * const ident_ = "$Id: main.cpp 11588 2009-09-08 07:23:21Z timlinux $";
+static const char * const ident_ = "$Id$";
 
 /** print usage text
  */
@@ -238,9 +236,13 @@ void myMessageOutput( QtMsgType type, const char *msg )
 
 int main( int argc, char *argv[] )
 {
+#ifdef WIN32  // Windows
 #ifdef _MSC_VER
   _set_fmode( _O_BINARY );
-#endif
+#else //MinGW
+  _fmode = _O_BINARY;
+#endif  // _MSC_VER
+#endif  // WIN32
 
 #ifndef _MSC_VER
   // Set up the custom qWarning/qDebug custom handler
@@ -488,12 +490,8 @@ int main( int argc, char *argv[] )
     exit( 1 );
   }
 
-
-  // myApp.setFont(QFont("helvetica", 11));
-
   QString i18nPath = QgsApplication::i18nPath();
 
-
   /* Translation file for QGIS.
    */
   QSettings mySettings;
@@ -614,12 +612,9 @@ int main( int argc, char *argv[] )
   }
 #endif
 
-
-
   QgisApp *qgis = new QgisApp( mypSplash ); // "QgisApp" used to find canonical instance
   qgis->setObjectName( "QgisApp" );
 
-
   /////////////////////////////////////////////////////////////////////
   // If no --project was specified, parse the args to look for a     //
   // .qgs file and set myProjectFileName to it. This allows loading  //
@@ -648,15 +643,12 @@ int main( int argc, char *argv[] )
     qgis->openProject( myProjectFileName );
   }
 
-
   /////////////////////////////////////////////////////////////////////
   // autoload any file names that were passed in on the command line
   /////////////////////////////////////////////////////////////////////
   QgsDebugMsg( QString( "Number of files in myFileList: %1" ).arg( myFileList.count() ) );
   for ( QStringList::Iterator myIterator = myFileList.begin(); myIterator != myFileList.end(); ++myIterator )
   {
-
-
     QgsDebugMsg( QString( "Trying to load file : %1" ).arg(( *myIterator ) ) );
     QString myLayerName = *myIterator;
     // don't load anything with a .qgs extension - these are project files
