@@ -14,7 +14,7 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-/*  $Id: qgisapp.h 11573 2009-09-05 19:50:05Z jef $ */
+/*  $Id$ */
 
 #ifndef QGISAPP_H
 #define QGISAPP_H
@@ -41,6 +41,7 @@ class QValidator;
 class QgisAppInterface;
 class QgsClipboard;
 class QgsComposer;
+class QgsComposerView;
 class QgsGeometry;
 class QgsHelpViewer;
 class QgsFeature;
@@ -59,6 +60,8 @@ class QgsRectangle;
 class QgsUndoWidget;
 class QgsVectorLayer;
 
+class QDomDocument;
+
 #include <QMainWindow>
 #include <QToolBar>
 #include <QAbstractSocket>
@@ -143,8 +146,6 @@ class QgisApp : public QMainWindow
     /** Get the mapcanvas object from the app */
     QgsMapCanvas * mapCanvas() { return mMapCanvas; };
 
-    QgsComposer* printComposer() {return mComposer;}
-
     //! Set theme (icons)
     void setTheme( QString themeName = "default" );
     //! Setup the toolbar popup menus for a given theme
@@ -196,6 +197,14 @@ class QgisApp : public QMainWindow
      * windows which are hidden rather than deleted when closed. */
     void removeWindow( QAction *action );
 
+    /**Returns the print composers*/
+    QSet<QgsComposer*> printComposers() const {return mPrintComposers;}
+    /**Creates a new composer and returns a pointer to it*/
+    QgsComposer* createNewComposer();
+    /**Deletes a composer and removes entry from Set*/
+    void deleteComposer( QgsComposer* c );
+
+
     //! Actions to be inserted in menus and toolbars
     QAction *actionNewProject() { return mActionNewProject; }
     QAction *actionOpenProject() { return mActionOpenProject; }
@@ -206,7 +215,7 @@ class QgisApp : public QMainWindow
     QAction *actionFileSeparator2() { return mActionFileSeparator2; }
     QAction *actionProjectProperties() { return mActionProjectProperties; }
     QAction *actionFileSeparator3() { return mActionFileSeparator3; }
-    QAction *actionPrintComposer() { return mActionPrintComposer; }
+    QAction *actionNewPrintComposer() { return mActionNewPrintComposer; }
     QAction *actionFileSeparator4() { return mActionFileSeparator4; }
     QAction *actionExit() { return mActionExit; }
 
@@ -220,9 +229,10 @@ class QgisApp : public QMainWindow
     QAction *actionDeleteSelected() { return mActionDeleteSelected; }
     QAction *actionMoveFeature() { return mActionMoveFeature; }
     QAction *actionSplitFeatures() { return mActionSplitFeatures; }
-    QAction *actionAddVertex() { return mActionAddVertex; }
-    QAction *actionDeleteVertex() { return mActionDeleteVertex; }
-    QAction *actionMoveVertex() { return mActionMoveVertex; }
+    //These three tools are deprecated - use node tool rather...
+    //QAction *actionAddVertex() { return mActionAddVertex; }
+    //QAction *actionDeleteVertex() { return mActionDeleteVertex; }
+    //QAction *actionMoveVertex() { return mActionMoveVertex; }
     QAction *actionAddRing() { return mActionAddRing; }
     QAction *actionAddIsland() { return mActionAddIsland; }
     QAction *actionSimplifyFeature() { return mActionSimplifyFeature; }
@@ -315,6 +325,7 @@ class QgisApp : public QMainWindow
     QMenu *firstRightStandardMenu() { return mHelpMenu; }
     QMenu *windowMenu() { return NULL; }
 #endif
+    QMenu *printComposersMenu() {return mPrintComposersMenu;}
     QMenu *helpMenu() { return mHelpMenu; }
 
     //! Toolbars
@@ -332,6 +343,9 @@ class QgisApp : public QMainWindow
     QToolBar *pluginToolBar() { return mPluginToolBar; }
     QToolBar *helpToolBar() { return mHelpToolBar; }
 
+    //! run python
+    void runPythonString( const QString &expr );
+
   public slots:
     //! Zoom to full extent
     void zoomFull();
@@ -365,6 +379,9 @@ class QgisApp : public QMainWindow
 
     void loadOGRSublayers( QString layertype, QString uri, QStringList list );
 
+    /**Deletes the selected attributes for the currently selected vector layer*/
+    void deleteSelected( QgsMapLayer *layer = 0 );
+
   protected:
 
     //! Handle state changes (WindowTitleChange)
@@ -443,7 +460,8 @@ class QgisApp : public QMainWindow
     //! Create a new empty vector layer
     void newVectorLayer();
     //! Print the current map view frame
-    void filePrint();
+    void newPrintComposer();
+    void showComposerManager();
     //! Add all loaded layers into the overview - overides qgisappbase method
     void addAllToOverview();
     //! Remove all loaded layers from the overview - overides qgisappbase method
@@ -460,6 +478,8 @@ class QgisApp : public QMainWindow
     */
     //! Return pointer to the active layer
     QgsMapLayer *activeLayer();
+    //! set the active layer
+    bool setActiveLayer( QgsMapLayer * );
     //! Open the help contents in a browser
     void helpContents();
     //! Open the QGIS homepage in users browser
@@ -495,8 +515,6 @@ class QgisApp : public QMainWindow
     void captureLine();
     //! activates the capture polygon tool
     void capturePolygon();
-    /**Deletes the selected attributes for the currently selected vector layer*/
-    void deleteSelected();
     //! activates the move feature tool
     void moveFeature();
     //! activates the reshape features tool
@@ -523,9 +541,15 @@ class QgisApp : public QMainWindow
     void mergeSelectedFeatures();
     //! provides operations with nodes
     void nodeTool();
+    //! activates the rotate points tool
+    void rotatePointSymbols();
 
     //! activates the selection tool
     void select();
+
+    //! deselect features from all layers
+    void deselectAll();
+
     //! refresh map canvas
     void refreshMapCanvas();
     //! returns pointer to map legend
@@ -611,6 +635,11 @@ class QgisApp : public QMainWindow
     //! Stops rendering of the main map
     void stopRendering();
 
+    void showStyleManagerV2();
+
+    //! project changed
+    void projectChanged( const QDomDocument & );
+
   signals:
     /** emitted when a key is pressed and we want non widget sublasses to be able
       to pick up on this (e.g. maplayer) */
@@ -642,6 +671,15 @@ class QgisApp : public QMainWindow
      */
     void currentThemeChanged( QString );
 
+    /**This signal is emitted when a new composer instance has been created
+       @note added in version 1.4*/
+    void composerAdded( QgsComposerView* v );
+
+    /**This signal is emitted before a new composer instance is going to be removed
+      @note added in version 1.4*/
+    void composerWillBeRemoved( QgsComposerView* v );
+
+
   private:
     /** This method will open a dialog so the user can select the sublayers
     * to load
@@ -652,10 +690,10 @@ class QgisApp : public QMainWindow
      */
     bool addRasterLayer( QgsRasterLayer * theRasterLayer );
     //@todo We should move these next two into vector layer class
-    /** This helper checks to see whether the file name appears to be a valid vector file name */
-    bool isValidVectorFileName( QString theFileNameQString );
+    /** This helper checks to see whether the file name appears to be a valid shape file name */
+    bool isValidShapeFileName( QString theFileNameQString );
     /** Overloaded version of the above function provided for convenience that takes a qstring pointer */
-    bool isValidVectorFileName( QString * theFileNameQString );
+    bool isValidShapeFileName( QString * theFileNameQString );
     /** add this file to the recently opened/saved projects list
      *  pass settings by reference since creating more than one
      * instance simultaneously results in data loss.
@@ -673,6 +711,11 @@ class QgisApp : public QMainWindow
       @return 0 in case of error*/
     QgsGeometry* unionGeometries( const QgsVectorLayer* vl, QgsFeatureList& featureList );
 
+    /**Deletes all the composer objects and clears mPrintComposers*/
+    void deletePrintComposers();
+    /**Creates the composer instances in a project file and adds them to the menu*/
+    bool loadComposersFromProject( const QString& projectFilePath );
+
     /// QgisApp aren't copyable
     QgisApp( QgisApp const & );
     /// QgisApp aren't copyable
@@ -713,7 +756,8 @@ class QgisApp : public QMainWindow
     QAction *mActionFileSeparator2;
     QAction *mActionProjectProperties;
     QAction *mActionFileSeparator3;
-    QAction *mActionPrintComposer;
+    QAction *mActionNewPrintComposer;
+    QAction *mActionShowComposerManager;
     QAction *mActionFileSeparator4;
     QAction *mActionExit;
 
@@ -731,9 +775,11 @@ class QgisApp : public QMainWindow
     QAction *mActionMoveFeature;
     QAction *mActionReshapeFeatures;
     QAction *mActionSplitFeatures;
+#if 0 // deprecated
     QAction *mActionAddVertex;
     QAction *mActionDeleteVertex;
     QAction *mActionMoveVertex;
+#endif
     QAction *mActionAddRing;
     QAction *mActionAddIsland;
     QAction *mActionEditSeparator2;
@@ -742,12 +788,14 @@ class QgisApp : public QMainWindow
     QAction *mActionDeletePart;
     QAction *mActionMergeFeatures;
     QAction *mActionNodeTool;
+    QAction *mActionRotatePointSymbols;
     QAction *mActionEditSeparator3;
 
     QAction *mActionPan;
     QAction *mActionZoomIn;
     QAction *mActionZoomOut;
     QAction *mActionSelect;
+    QAction *mActionDeselectAll;
     QAction *mActionIdentify;
     QAction *mActionMeasure;
     QAction *mActionMeasureArea;
@@ -813,6 +861,9 @@ class QgisApp : public QMainWindow
     QAction *mActionHelpSeparator2;
     QAction *mActionAbout;
 
+    QAction *mActionUseRendererV2;
+    QAction *mActionStyleManagerV2;
+
     // action groups ----------------------------------
     QActionGroup *mMapToolGroup;
 
@@ -828,6 +879,7 @@ class QgisApp : public QMainWindow
 #ifdef Q_WS_MAC
     QMenu *mWindowMenu;
 #endif
+    QMenu *mPrintComposersMenu;
     QMenu *mHelpMenu;
 
     // docks ------------------------------------------
@@ -864,6 +916,7 @@ class QgisApp : public QMainWindow
         QgsMapTool* mDeleteRing;
         QgsMapTool* mDeletePart;
         QgsMapTool* mNodeTool;
+        QgsMapTool* mRotatePointSymbolsTool;
     } mMapTools;
 
     QgsMapTool *mNonEditMapTool;
@@ -928,8 +981,8 @@ class QgisApp : public QMainWindow
     QgsHelpViewer *mHelpViewer;
     //! list of recently opened/saved project files
     QStringList mRecentProjectPaths;
-    //! Map composer
-    QgsComposer *mComposer;
+    //! Print composers of this project, accessible by id string
+    QSet<QgsComposer*> mPrintComposers;
     //! How to determine the number of decimal places used to
     //! display the mouse position
     bool mMousePrecisionAutomatic;
@@ -958,7 +1011,7 @@ class QgisApp : public QMainWindow
 
     /* Maptip object
      */
-    QgsMapTip *  mpMaptip;
+    QgsMapTip *mpMaptip;
 
     // Flag to indicate if maptips are on or off
     bool mMapTipsVisible;
@@ -976,6 +1029,7 @@ class QgisApp : public QMainWindow
 
     QgsUndoWidget* mUndoWidget;
 
+    int mLastComposerId;
 };
 
 #endif
