@@ -14,7 +14,7 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-/* $Id: qgsgraduatedsymboldialog.cpp 11371 2009-08-14 09:20:23Z jef $ */
+/* $Id$ */
 
 #include <algorithm>
 #include <cmath>
@@ -44,7 +44,7 @@ QgsGraduatedSymbolDialog::QgsGraduatedSymbolDialog( QgsVectorLayer * layer ): QD
   for ( QgsFieldMap::const_iterator it = fields.begin(); it != fields.end(); ++it )
   {
     QVariant::Type type = ( *it ).type();
-    if ( type == QVariant::Int || type == QVariant::Double )
+    if ( type == QVariant::Int || type == QVariant::Double || type == QVariant::LongLong )
     {
       displayName = layer->attributeDisplayName( it.key() );
       classificationComboBox->addItem( displayName );
@@ -53,7 +53,7 @@ QgsGraduatedSymbolDialog::QgsGraduatedSymbolDialog( QgsVectorLayer * layer ): QD
   }
 
   //restore the correct settings
-  const QgsGraduatedSymbolRenderer* renderer = dynamic_cast < const QgsGraduatedSymbolRenderer * >( layer->renderer() );
+  const QgsGraduatedSymbolRenderer* renderer = dynamic_cast<const QgsGraduatedSymbolRenderer *>( layer->renderer() );
 
   //
   // Set up the mode combo
@@ -112,6 +112,7 @@ QgsGraduatedSymbolDialog::QgsGraduatedSymbolDialog( QgsVectorLayer * layer ): QD
       sym->setBrush(( *it )->brush() );
       sym->setNamedPointSymbol(( *it )->pointSymbolName() );
       sym->setPointSize(( *it )->pointSize() );
+      sym->setPointSizeUnits(( *it )->pointSizeUnits() );
       sym->setScaleClassificationField(( *it )->scaleClassificationField() );
       sym->setRotationClassificationField(( *it )->rotationClassificationField() );
       mEntries.insert( std::make_pair( classbreak, sym ) );
@@ -210,6 +211,7 @@ void QgsGraduatedSymbolDialog::apply()
     {
       sy->setNamedPointSymbol( it->second->pointSymbolName() );
       sy->setPointSize( it->second->pointSize() );
+      sy->setPointSizeUnits( it->second->pointSizeUnits() );
       sy->setScaleClassificationField( it->second->scaleClassificationField() );
       sy->setRotationClassificationField( it->second->rotationClassificationField() );
     }
@@ -347,8 +349,8 @@ void QgsGraduatedSymbolDialog::adjustClassification()
     {
       if ( last_it != quantileBorders.end() )
       {
-        lowerString = QVariant( *last_it ).toString();
-        upperString = QVariant( *it ).toString();
+        lowerString = QString::number( QVariant( *last_it ).toDouble(), 'f', 3 );
+        upperString = QString::number( QVariant( *it ).toDouble(), 'f', 3 );
         ( *symbol_it )->setLowerValue( lowerString );
         ( *symbol_it )->setUpperValue( upperString );
 
@@ -371,8 +373,8 @@ void QgsGraduatedSymbolDialog::adjustClassification()
       //switch if attribute is int or double
       double lower = minimum + ( maximum - minimum ) / numberofclassesspinbox->value() * i;
       double upper = minimum + ( maximum - minimum ) / numberofclassesspinbox->value() * ( i + 1 );
-      lowerString = QVariant( lower ).toString();
-      upperString = QVariant( upper ).toString();
+      lowerString = QString::number( lower, 'f', 3 );
+      upperString = QString::number( upper, 'f', 3 );
       ( *symbol_it )->setLowerValue( lowerString );
       ( *symbol_it )->setUpperValue( upperString );
       listBoxText = lowerString + " - " + upperString;
