@@ -15,10 +15,9 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-/* $Id: qgsidentifyresults.cpp 11631 2009-09-12 00:21:04Z jef $ */
+/* $Id$ */
 
 #include "qgsidentifyresults.h"
-#include "qgscontexthelp.h"
 #include "qgsapplication.h"
 #include "qgisapp.h"
 #include "qgsmaplayer.h"
@@ -27,6 +26,7 @@
 #include "qgsgeometry.h"
 #include "qgsattributedialog.h"
 #include "qgsmapcanvas.h"
+#include "qgsattributeaction.h"
 
 #include <QCloseEvent>
 #include <QLabel>
@@ -37,9 +37,24 @@
 #include <QMenu>
 #include <QClipboard>
 #include <QDockWidget>
+#include <QMenuBar>
+#include <QPushButton>
 
 #include "qgslogger.h"
 
+static void _runPythonString( const QString &expr )
+{
+  QgisApp::instance()->runPythonString( expr );
+}
+
+void QgsFeatureAction::execute()
+{
+  int idx;
+  QList< QPair<QString, QString> > attributes;
+  mResults->retrieveAttributes( mFeatItem, attributes, idx );
+  mLayer->actions()->doAction( mAction, attributes, idx, _runPythonString );
+}
+
 class QgsIdentifyResultsDock : public QDockWidget
 {
   public:
@@ -55,24 +70,28 @@ class QgsIdentifyResultsDock : public QDockWidget
     }
 };
 
-// Tree hierachy
+// Tree hierarchy
 //
 // layer [userrole: QgsMapLayer]
 //   feature: displayfield|displayvalue [userrole: fid]
-//     derived attributes
+//     derived attributes (if any) [userrole: "derived"]
 //       name value
+//     actions (if any) [userrole: "actions"]
+//       edit [userrole: "edit"]
+//       action [userrole: "action", idx]
 //     name value
 //     name value
 //     name value
 //   feature
-//     derived attributes
+//     derived attributes (if any)
 //       name value
+//     actions (if any)
+//       action
 //     name value
 
 QgsIdentifyResults::QgsIdentifyResults( QgsMapCanvas *canvas, QWidget *parent, Qt::WFlags f )
     : QDialog( parent, f ),
     mActionPopup( 0 ),
-    mRubberBand( 0 ),
     mCanvas( canvas ),
     mDock( NULL )
 {
@@ -85,34 +104,36 @@ QgsIdentifyResults::QgsIdentifyResults( QgsMapCanvas *canvas, QWidget *parent, Q
     mDock->setAllowedAreas( Qt::LeftDockWidgetArea | Qt::RightDockWidgetArea );
     mDock->setWidget( this );
     QgisApp::instance()->addDockWidget( Qt::LeftDockWidgetArea, mDock );
-    buttonCancel->hide();
   }
   lstResults->setColumnCount( 2 );
   setColumnText( 0, tr( "Feature" ) );
   setColumnText( 1, tr( "Value" ) );
 
-  connect( buttonCancel, SIGNAL( clicked() ),
-           this, SLOT( close() ) );
+  connect( buttonBox, SIGNAL( rejected() ), this, SLOT( close() ) );
 
   connect( lstResults, SIGNAL( itemExpanded( QTreeWidgetItem* ) ),
            this, SLOT( itemExpanded( QTreeWidgetItem* ) ) );
 
   connect( lstResults, SIGNAL( currentItemChanged( QTreeWidgetItem*, QTreeWidgetItem* ) ),
            this, SLOT( handleCurrentItemChanged( QTreeWidgetItem*, QTreeWidgetItem* ) ) );
+
+  connect( lstResults, SIGNAL( itemClicked( QTreeWidgetItem*, int ) ),
+           this, SLOT( itemClicked( QTreeWidgetItem*, int ) ) );
 }
 
 QgsIdentifyResults::~QgsIdentifyResults()
 {
-  delete mActionPopup;
+  clearRubberbands();
+  if ( mActionPopup )
+    delete mActionPopup;
 }
 
 QTreeWidgetItem *QgsIdentifyResults::layerItem( QObject *layer )
 {
-  QTreeWidgetItem *item;
-
   for ( int i = 0; i < lstResults->topLevelItemCount(); i++ )
   {
-    item = lstResults->topLevelItem( i );
+    QTreeWidgetItem *item = lstResults->topLevelItem( i );
+
     if ( item->data( 0, Qt::UserRole ).value<QObject*>() == layer )
       return item;
   }
@@ -126,18 +147,28 @@ void QgsIdentifyResults::addFeature( QgsMapLayer *layer, int fid,
                                      const QMap<QString, QString> &derivedAttributes )
 {
   QTreeWidgetItem *layItem = layerItem( layer );
+  QgsVectorLayer *vlayer = qobject_cast<QgsVectorLayer *>( layer );
 
   if ( layItem == 0 )
   {
     layItem = new QTreeWidgetItem( QStringList() << layer->name() << tr( "Layer" ) );
-    layItem->setData( 0, Qt::UserRole, QVariant::fromValue( dynamic_cast<QObject*>( layer ) ) );
+    layItem->setData( 0, Qt::UserRole, QVariant::fromValue( qobject_cast<QObject *>( layer ) ) );
     lstResults->addTopLevelItem( layItem );
 
-    connect( layer, SIGNAL( destroyed() ), this, SLOT( layerDestroyed() ) );
-
-    QgsVectorLayer *vlayer = dynamic_cast<QgsVectorLayer*>( layer );
+    QgsVectorLayer *vlayer = qobject_cast<QgsVectorLayer *>( layer );
     if ( vlayer )
+    {
+      connect( vlayer, SIGNAL( layerDeleted() ), this, SLOT( layerDestroyed() ) );
+      connect( vlayer, SIGNAL( layerCrsChanged() ), this, SLOT( layerDestroyed() ) );
       connect( vlayer, SIGNAL( featureDeleted( int ) ), this, SLOT( featureDeleted( int ) ) );
+      connect( vlayer, SIGNAL( editingStarted() ), this, SLOT( editingToggled() ) );
+      connect( vlayer, SIGNAL( editingStopped() ), this, SLOT( editingToggled() ) );
+    }
+    else
+    {
+      connect( layer, SIGNAL( destroyed() ), this, SLOT( layerDestroyed() ) );
+      connect( layer, SIGNAL( layerCrsChanged() ), this, SLOT( layerDestroyed() ) );
+    }
   }
 
   QTreeWidgetItem *featItem = new QTreeWidgetItem( QStringList() << displayField << displayValue );
@@ -151,6 +182,7 @@ void QgsIdentifyResults::addFeature( QgsMapLayer *layer, int fid,
   if ( derivedAttributes.size() >= 0 )
   {
     QTreeWidgetItem *derivedItem = new QTreeWidgetItem( QStringList() << tr( "(Derived)" ) );
+    derivedItem->setData( 0, Qt::UserRole, "derived" );
     featItem->addChild( derivedItem );
 
     for ( QMap< QString, QString>::const_iterator it = derivedAttributes.begin(); it != derivedAttributes.end(); it++ )
@@ -159,7 +191,69 @@ void QgsIdentifyResults::addFeature( QgsMapLayer *layer, int fid,
     }
   }
 
+  if ( vlayer )
+  {
+    QTreeWidgetItem *actionItem = new QTreeWidgetItem( QStringList() << tr( "(Actions)" ) );
+    actionItem->setData( 0, Qt::UserRole, "actions" );
+    featItem->addChild( actionItem );
+
+    QTreeWidgetItem *editItem = new QTreeWidgetItem( QStringList() << "" << ( vlayer->isEditable() ? tr( "Edit feature form" ) : tr( "View feature form" ) ) );
+    editItem->setIcon( 0, QgisApp::getThemeIcon( vlayer->isEditable() ? "/mIconEditable.png" : "/mIconEditable.png" ) );
+    editItem->setData( 0, Qt::UserRole, "edit" );
+    actionItem->addChild( editItem );
+
+    for ( int i = 0; i < vlayer->actions()->size(); i++ )
+    {
+      const QgsAction &action = vlayer->actions()->at( i );
+
+      if ( !action.runable() )
+        continue;
+
+      QTreeWidgetItem *twi = new QTreeWidgetItem( QStringList() << "" << action.name() );
+      twi->setIcon( 0, QgisApp::getThemeIcon( "/mAction.png" ) );
+      twi->setData( 0, Qt::UserRole, "action" );
+      twi->setData( 0, Qt::UserRole + 1, QVariant::fromValue( i ) );
+      actionItem->addChild( twi );
+    }
+  }
+
   layItem->addChild( featItem );
+
+  highlightFeature( featItem );
+}
+
+void QgsIdentifyResults::editingToggled()
+{
+  QTreeWidgetItem *layItem = layerItem( sender() );
+  QgsVectorLayer *vlayer = vectorLayer( layItem );
+  if ( !layItem || !vlayer )
+    return;
+
+  // iterate features
+  int i;
+  for ( i = 0; i < layItem->childCount(); i++ )
+  {
+    QTreeWidgetItem *featItem = layItem->child( i );
+
+    int j;
+    for ( j = 0; j < featItem->childCount() && featItem->child( j )->data( 0, Qt::UserRole ).toString() != "actions"; j++ )
+      QgsDebugMsg( QString( "%1: skipped %2" ).arg( featItem->child( j )->data( 0, Qt::UserRole ).toString() ) );
+
+    if ( j == featItem->childCount() || featItem->child( j )->childCount() < 1 )
+      continue;
+
+    QTreeWidgetItem *actions = featItem->child( j );
+
+    for ( j = 0; i < actions->childCount() && actions->child( j )->data( 0, Qt::UserRole ).toString() != "edit"; j++ )
+      ;
+
+    if ( j == actions->childCount() )
+      continue;
+
+    QTreeWidgetItem *editItem = actions->child( j );
+    editItem->setIcon( 0, QgisApp::getThemeIcon( vlayer->isEditable() ? "/mIconEditable.png" : "/mIconEditable.png" ) );
+    editItem->setText( 1, vlayer->isEditable() ? tr( "Edit feature form" ) : tr( "View feature form" ) );
+  }
 }
 
 // Call to show the dialog box.
@@ -174,14 +268,15 @@ void QgsIdentifyResults::show()
     QTreeWidgetItem *layItem = lstResults->topLevelItem( 0 );
     QTreeWidgetItem *featItem = layItem->child( 0 );
 
-    if ( layItem->childCount() == 1 )
+    if ( lstResults->topLevelItemCount() == 1 && layItem->childCount() == 1 && QSettings().value( "/Map/identifyAutoFeatureForm", false ).toBool() )
     {
-      QgsVectorLayer *layer = dynamic_cast<QgsVectorLayer *>( layItem->data( 0, Qt::UserRole ).value<QObject *>() );
-      if ( layer && layer->isEditable() )
+      QgsVectorLayer *layer = qobject_cast<QgsVectorLayer *>( layItem->data( 0, Qt::UserRole ).value<QObject *>() );
+      if ( layer )
       {
-        // if this is the only feature, it's on a vector layer and that layer is editable:
-        // don't show the edit dialog instead of the results window
-        editFeature( featItem );
+        // if this is the only feature and it's on a vector layer
+        // don't show the form dialog instead of the results window
+        lstResults->setCurrentItem( featItem );
+        featureForm();
         return;
       }
     }
@@ -193,13 +288,21 @@ void QgsIdentifyResults::show()
 
   QDialog::show();
 }
+
 // Slot called when user clicks the Close button
 // (saves the current window size/position)
 void QgsIdentifyResults::close()
 {
+  clearRubberbands();
+  delete mActionPopup;
+  mActionPopup = 0;
+
   saveWindowLocation();
   done( 0 );
+  if ( mDock )
+    mDock->close();
 }
+
 // Save the current window size/position before closing
 // from window menu or X in titlebar
 void QgsIdentifyResults::closeEvent( QCloseEvent *e )
@@ -209,6 +312,19 @@ void QgsIdentifyResults::closeEvent( QCloseEvent *e )
   close();
 }
 
+void QgsIdentifyResults::itemClicked( QTreeWidgetItem *item, int column )
+{
+  if ( item->data( 0, Qt::UserRole ).toString() == "edit" )
+  {
+    lstResults->setCurrentItem( item );
+    featureForm();
+  }
+  else if ( item->data( 0, Qt::UserRole ).toString() == "action" )
+  {
+    doAction( item, item->data( 0, Qt::UserRole + 1 ).toInt() );
+  }
+}
+
 // Popup (create if necessary) a context menu that contains a list of
 // actions that can be applied to the data in the identify results
 // dialog box.
@@ -220,38 +336,29 @@ void QgsIdentifyResults::contextMenuEvent( QContextMenuEvent* event )
   if ( !item )
     return;
 
-  if ( mActionPopup )
-    delete mActionPopup;
-
   QgsVectorLayer *vlayer = vectorLayer( item );
   if ( vlayer == 0 )
     return;
 
+  if ( mActionPopup )
+    delete mActionPopup;
+
   mActionPopup = new QMenu();
 
-  QAction *a;
+  mActionPopup->addAction( vlayer->isEditable() ? tr( "Edit feature form" ) : tr( "View feature form" ), this, SLOT( featureForm() ) );
+  mActionPopup->addAction( tr( "Zoom to feature" ), this, SLOT( zoomToFeature() ) );
+  mActionPopup->addAction( tr( "Copy attribute value" ), this, SLOT( copyAttributeValue() ) );
+  mActionPopup->addAction( tr( "Copy feature attributes" ), this, SLOT( copyFeatureAttributes() ) );
+  mActionPopup->addSeparator();
+  mActionPopup->addAction( tr( "Clear results" ), this, SLOT( clear() ) );
+  mActionPopup->addAction( tr( "Clear highlights" ), this, SLOT( clearRubberbands() ) );
+  mActionPopup->addAction( tr( "Highlight all" ), this, SLOT( highlightAll() ) );
+  mActionPopup->addAction( tr( "Highlight layer" ), this, SLOT( highlightLayer() ) );
+  mActionPopup->addSeparator();
+  mActionPopup->addAction( tr( "Expand all" ), this, SLOT( expandAll() ) );
+  mActionPopup->addAction( tr( "Collapse all" ), this, SLOT( collapseAll() ) );
 
-  if ( vlayer->isEditable() )
-  {
-    a = mActionPopup->addAction( tr( "Edit feature" ) );
-    a->setEnabled( true );
-    a->setData( QVariant::fromValue( -4 ) );
-  }
-
-  a = mActionPopup->addAction( tr( "Zoom to feature" ) );
-  a->setEnabled( true );
-  a->setData( QVariant::fromValue( -3 ) );
-
-  a = mActionPopup->addAction( tr( "Copy attribute value" ) );
-  a->setEnabled( true );
-  a->setData( QVariant::fromValue( -2 ) );
-
-  a = mActionPopup->addAction( tr( "Copy feature attributes" ) );
-  a->setEnabled( true );
-  a->setData( QVariant::fromValue( -1 ) );
-
-  QgsAttributeAction *actions = vlayer->actions();
-  if ( actions && actions->size() > 0 )
+  if ( vlayer->actions()->size() > 0 )
   {
     mActionPopup->addSeparator();
 
@@ -259,22 +366,21 @@ void QgsIdentifyResults::contextMenuEvent( QContextMenuEvent* event )
     // created for each new Identify Results dialog box, and that the
     // contents of the popup menu doesn't change during the time that
     // such a dialog box is around.
-    a = mActionPopup->addAction( tr( "Run action" ) );
+    QAction *a = mActionPopup->addAction( tr( "Run action" ) );
     a->setEnabled( false );
 
-    QgsAttributeAction::aIter iter = actions->begin();
-    for ( int i = 0; iter != actions->end(); ++iter, ++i )
+    for ( int i = 0; i < vlayer->actions()->size(); i++ )
     {
-      QAction* a = mActionPopup->addAction( iter->name() );
-      a->setEnabled( true );
-      // The menu action stores an integer that is used later on to
-      // associate an menu action with an actual qgis action.
-      a->setData( QVariant::fromValue( i ) );
+      const QgsAction &action = vlayer->actions()->at( i );
+
+      if ( !action.runable() )
+        continue;
+
+      QgsFeatureAction *a = new QgsFeatureAction( action.name(), this, vlayer, i, featureItem( item ) );
+      mActionPopup->addAction( action.name(), a, SLOT( execute() ) );
     }
   }
 
-  connect( mActionPopup, SIGNAL( triggered( QAction* ) ), this, SLOT( popupItemSelected( QAction* ) ) );
-
   mActionPopup->popup( event->globalPos() );
 }
 
@@ -299,73 +405,41 @@ void QgsIdentifyResults::setColumnText( int column, const QString & label )
   header->setText( column, label );
 }
 
-// Run the action that was selected in the popup menu
-void QgsIdentifyResults::popupItemSelected( QAction* menuAction )
-{
-  QTreeWidgetItem *item = lstResults->currentItem();
-  if ( item == 0 )
-    return;
-
-  int action = menuAction->data().toInt();
-
-  if ( action < 0 )
-  {
-    if ( action == -4 )
-    {
-      editFeature( item );
-    }
-    else if ( action == -3 )
-    {
-      zoomToFeature( item );
-    }
-    else
-    {
-      QClipboard *clipboard = QApplication::clipboard();
-      QString text;
-
-      if ( action == -2 )
-      {
-        text = item->data( 1, Qt::DisplayRole ).toString();
-      }
-      else
-      {
-        std::vector< std::pair<QString, QString> > attributes;
-        retrieveAttributes( item, attributes );
-
-        for ( std::vector< std::pair<QString, QString> >::iterator it = attributes.begin(); it != attributes.end(); it++ )
-        {
-          text += QString( "%1: %2\n" ).arg( it->first ).arg( it->second );
-        }
-      }
-
-      QgsDebugMsg( QString( "set clipboard: %1" ).arg( text ) );
-      clipboard->setText( text );
-    }
-  }
-  else
-  {
-    doAction( item, action );
-  }
-}
-
 void QgsIdentifyResults::expandColumnsToFit()
 {
   lstResults->resizeColumnToContents( 0 );
   lstResults->resizeColumnToContents( 1 );
 }
 
+void QgsIdentifyResults::clearRubberbands()
+{
+  foreach( QgsRubberBand *rb, mRubberBands )
+  {
+    delete rb;
+  }
+
+  mRubberBands.clear();
+}
+
 void QgsIdentifyResults::clear()
 {
+  for ( int i = 0; i < lstResults->topLevelItemCount(); i++ )
+  {
+    disconnectLayer( lstResults->topLevelItem( i )->data( 0, Qt::UserRole ).value<QObject *>() );
+  }
+
   lstResults->clear();
-  clearRubberBand();
+  clearRubberbands();
 }
 
 void QgsIdentifyResults::activate()
 {
-  if ( mRubberBand )
+#if 0
+  foreach( QgsRubberBand *rb, mRubberBands )
   {
-    mRubberBand->show();
+    rb->show();
   }
+#endif
 
   if ( lstResults->topLevelItemCount() > 0 )
   {
@@ -376,35 +450,27 @@ void QgsIdentifyResults::activate()
 
 void QgsIdentifyResults::deactivate()
 {
-  if ( mRubberBand )
+#if 0
+  foreach( QgsRubberBand *rb, mRubberBands )
   {
-    mRubberBand->hide();
+    rb->hide();
   }
-}
-
-void QgsIdentifyResults::clearRubberBand()
-{
-  if ( !mRubberBand )
-    return;
-
-  delete mRubberBand;
-  mRubberBand = 0;
-  mRubberBandLayer = 0;
-  mRubberBandFid = 0;
+#endif
 }
 
 void QgsIdentifyResults::doAction( QTreeWidgetItem *item, int action )
 {
-  std::vector< std::pair<QString, QString> > attributes;
-  QTreeWidgetItem *featItem = retrieveAttributes( item, attributes );
+  int idx;
+  QList< QPair<QString, QString> > attributes;
+  QTreeWidgetItem *featItem = retrieveAttributes( item, attributes, idx );
   if ( !featItem )
     return;
 
-  QgsVectorLayer *layer = dynamic_cast<QgsVectorLayer *>( featItem->parent()->data( 0, Qt::UserRole ).value<QObject *>() );
+  QgsVectorLayer *layer = qobject_cast<QgsVectorLayer *>( featItem->parent()->data( 0, Qt::UserRole ).value<QObject *>() );
   if ( !layer )
     return;
 
-  int idx = -1;
+  idx = -1;
   if ( item->parent() == featItem )
   {
     QString fieldName = item->data( 0, Qt::DisplayRole ).toString();
@@ -431,7 +497,7 @@ QTreeWidgetItem *QgsIdentifyResults::featureItem( QTreeWidgetItem *item )
     {
       if ( item->parent()->parent()->parent() )
       {
-        // derived attribute item
+        // derived or action attribute item
         featItem = item->parent()->parent();
       }
       else
@@ -458,44 +524,52 @@ QTreeWidgetItem *QgsIdentifyResults::featureItem( QTreeWidgetItem *item )
   return featItem;
 }
 
-QgsVectorLayer *QgsIdentifyResults::vectorLayer( QTreeWidgetItem *item )
+QTreeWidgetItem *QgsIdentifyResults::layerItem( QTreeWidgetItem *item )
 {
-  if ( item->parent() )
+  if ( item && item->parent() )
   {
     item = featureItem( item )->parent();
   }
 
-  return dynamic_cast<QgsVectorLayer *>( item->data( 0, Qt::UserRole ).value<QObject *>() );
+  return item;
 }
 
 
-QTreeWidgetItem *QgsIdentifyResults::retrieveAttributes( QTreeWidgetItem *item, std::vector< std::pair<QString, QString> > &attributes )
+QgsVectorLayer *QgsIdentifyResults::vectorLayer( QTreeWidgetItem *item )
+{
+  item = layerItem( item );
+  if ( !item )
+    return NULL;
+  return qobject_cast<QgsVectorLayer *>( item->data( 0, Qt::UserRole ).value<QObject *>() );
+}
+
+
+QTreeWidgetItem *QgsIdentifyResults::retrieveAttributes( QTreeWidgetItem *item, QList< QPair<QString, QString> > &attributes, int &idx )
 {
   QTreeWidgetItem *featItem = featureItem( item );
 
+  idx = -1;
+
   attributes.clear();
   for ( int i = 0; i < featItem->childCount(); i++ )
   {
     QTreeWidgetItem *item = featItem->child( i );
     if ( item->childCount() > 0 )
       continue;
-    attributes.push_back( std::make_pair( item->data( 0, Qt::DisplayRole ).toString(), item->data( 1, Qt::DisplayRole ).toString() ) );
+    if ( item == lstResults->currentItem() )
+      idx = attributes.size();
+    attributes << QPair<QString, QString>( item->data( 0, Qt::DisplayRole ).toString(), item->data( 1, Qt::DisplayRole ).toString() );
   }
 
   return featItem;
 }
 
-void QgsIdentifyResults::on_buttonHelp_clicked()
-{
-  QgsContextHelp::run( context_id );
-}
-
 void QgsIdentifyResults::itemExpanded( QTreeWidgetItem* item )
 {
   expandColumnsToFit();
 }
 
-void QgsIdentifyResults::handleCurrentItemChanged( QTreeWidgetItem* current, QTreeWidgetItem* previous )
+void QgsIdentifyResults::handleCurrentItemChanged( QTreeWidgetItem *current, QTreeWidgetItem *previous )
 {
   if ( current == NULL )
   {
@@ -503,21 +577,61 @@ void QgsIdentifyResults::handleCurrentItemChanged( QTreeWidgetItem* current, QTr
     return;
   }
 
-  highlightFeature( current );
+  QTreeWidgetItem *layItem = layerItem( current );
+
+  if ( current == layItem )
+  {
+    highlightLayer( layItem );
+  }
+  else
+  {
+    clearRubberbands();
+    highlightFeature( current );
+  }
 }
 
 void QgsIdentifyResults::layerDestroyed()
 {
-  if ( mRubberBandLayer == sender() )
+  QObject *theSender = sender();
+
+  for ( int i = 0; i < lstResults->topLevelItemCount(); i++ )
   {
-    clearRubberBand();
+    QTreeWidgetItem *layItem = lstResults->topLevelItem( i );
+
+    if ( layItem->data( 0, Qt::UserRole ).value<QObject *>() == sender() )
+    {
+      for ( int j = 0; j < layItem->childCount(); j++ )
+      {
+        delete mRubberBands.take( layItem->child( i ) );
+      }
+    }
   }
 
-  delete layerItem( sender() );
+  disconnectLayer( theSender );
+  delete layerItem( theSender );
 
   if ( lstResults->topLevelItemCount() == 0 )
   {
-    hide();
+    close();
+  }
+}
+
+void QgsIdentifyResults::disconnectLayer( QObject *layer )
+{
+  if ( !layer )
+    return;
+
+  QgsVectorLayer *vlayer = qobject_cast<QgsVectorLayer *>( layer );
+  if ( vlayer )
+  {
+    disconnect( vlayer, SIGNAL( layerDeleted() ), this, SLOT( layerDestroyed() ) );
+    disconnect( vlayer, SIGNAL( featureDeleted( int ) ), this, SLOT( featureDeleted( int ) ) );
+    disconnect( vlayer, SIGNAL( editingStarted() ), this, SLOT( editingToggled() ) );
+    disconnect( vlayer, SIGNAL( editingStopped() ), this, SLOT( editingToggled() ) );
+  }
+  else
+  {
+    disconnect( layer, SIGNAL( destroyed() ), this, SLOT( layerDestroyed() ) );
   }
 }
 
@@ -534,8 +648,7 @@ void QgsIdentifyResults::featureDeleted( int fid )
 
     if ( featItem && featItem->data( 0, Qt::UserRole ).toInt() == fid )
     {
-      if ( mRubberBandLayer == sender() && mRubberBandFid == fid )
-        clearRubberBand();
+      delete mRubberBands.take( featItem );
       delete featItem;
       break;
     }
@@ -543,14 +656,12 @@ void QgsIdentifyResults::featureDeleted( int fid )
 
   if ( layItem->childCount() == 0 )
   {
-    if ( mRubberBandLayer == sender() )
-      clearRubberBand();
     delete layItem;
   }
 
   if ( lstResults->topLevelItemCount() == 0 )
   {
-    hide();
+    close();
   }
 }
 
@@ -564,9 +675,10 @@ void QgsIdentifyResults::highlightFeature( QTreeWidgetItem *item )
   if ( !featItem )
     return;
 
-  int fid = featItem->data( 0, Qt::UserRole ).toInt();
+  if ( mRubberBands.contains( featItem ) )
+    return;
 
-  clearRubberBand();
+  int fid = featItem->data( 0, Qt::UserRole ).toInt();
 
   QgsFeature feat;
   if ( ! layer->featureAtId( fid, feat, true, false ) )
@@ -579,20 +691,21 @@ void QgsIdentifyResults::highlightFeature( QTreeWidgetItem *item )
     return;
   }
 
-  mRubberBand = new QgsRubberBand( mCanvas, feat.geometry()->type() == QGis::Polygon );
-  if ( mRubberBand )
+  QgsRubberBand *rb = new QgsRubberBand( mCanvas, feat.geometry()->type() == QGis::Polygon );
+  if ( rb )
   {
-    mRubberBandLayer = layer;
-    mRubberBandFid = fid;
-    mRubberBand->setToGeometry( feat.geometry(), layer );
-    mRubberBand->setWidth( 2 );
-    mRubberBand->setColor( Qt::red );
-    mRubberBand->show();
+    rb->setToGeometry( feat.geometry(), layer );
+    rb->setWidth( 2 );
+    rb->setColor( Qt::red );
+    rb->show();
+    mRubberBands.insert( featItem, rb );
   }
 }
 
-void QgsIdentifyResults::zoomToFeature( QTreeWidgetItem *item )
+void QgsIdentifyResults::zoomToFeature()
 {
+  QTreeWidgetItem *item = lstResults->currentItem();
+
   QgsVectorLayer *layer = vectorLayer( item );
   if ( !layer )
     return;
@@ -627,11 +740,12 @@ void QgsIdentifyResults::zoomToFeature( QTreeWidgetItem *item )
   mCanvas->refresh();
 }
 
-
-void QgsIdentifyResults::editFeature( QTreeWidgetItem *item )
+void QgsIdentifyResults::featureForm()
 {
-  QgsVectorLayer *layer = vectorLayer( item );
-  if ( !layer || !layer->isEditable() )
+  QTreeWidgetItem *item = lstResults->currentItem();
+
+  QgsVectorLayer *vlayer = vectorLayer( item );
+  if ( !vlayer )
     return;
 
   QTreeWidgetItem *featItem = featureItem( item );
@@ -641,13 +755,39 @@ void QgsIdentifyResults::editFeature( QTreeWidgetItem *item )
   int fid = featItem->data( 0, Qt::UserRole ).toInt();
 
   QgsFeature f;
-  if ( ! layer->featureAtId( fid, f ) )
+  if ( !vlayer->featureAtId( fid, f ) )
     return;
 
   QgsAttributeMap src = f.attributeMap();
 
-  layer->beginEditCommand( tr( "Attribute changed" ) );
-  QgsAttributeDialog *ad = new QgsAttributeDialog( layer, &f );
+  vlayer->beginEditCommand( tr( "Attribute changed" ) );
+  QgsAttributeDialog *ad = new QgsAttributeDialog( vlayer, &f );
+
+  if ( !vlayer->isEditable() && vlayer->actions()->size() > 0 )
+  {
+    ad->dialog()->setContextMenuPolicy( Qt::ActionsContextMenu );
+
+    QAction *a = new QAction( tr( "Run actions" ), ad->dialog() );
+    a->setEnabled( false );
+    ad->dialog()->addAction( a );
+
+    for ( int i = 0; i < vlayer->actions()->size(); i++ )
+    {
+      const QgsAction &action = vlayer->actions()->at( i );
+
+      if ( !action.runable() )
+        continue;
+
+      QgsFeatureAction *a = new QgsFeatureAction( action.name(), this, vlayer, i, featItem );
+      ad->dialog()->addAction( a );
+      connect( a, SIGNAL( triggered() ), a, SLOT( execute() ) );
+
+      QPushButton *pb = ad->dialog()->findChild<QPushButton *>( action.name() );
+      if ( pb )
+        connect( pb, SIGNAL( clicked() ), a, SLOT( execute() ) );
+    }
+  }
+
   if ( ad->exec() )
   {
     const QgsAttributeMap &dst = f.attributeMap();
@@ -655,16 +795,85 @@ void QgsIdentifyResults::editFeature( QTreeWidgetItem *item )
     {
       if ( !src.contains( it.key() ) || it.value() != src[it.key()] )
       {
-        layer->changeAttributeValue( f.id(), it.key(), it.value() );
+        vlayer->changeAttributeValue( f.id(), it.key(), it.value() );
       }
     }
-    layer->endEditCommand();
+    vlayer->endEditCommand();
   }
   else
   {
-    layer->destroyEditCommand();
+    vlayer->destroyEditCommand();
   }
 
   delete ad;
   mCanvas->refresh();
 }
+
+void QgsIdentifyResults::highlightAll()
+{
+  for ( int i = 0; i < lstResults->topLevelItemCount(); i++ )
+  {
+    QTreeWidgetItem *layItem = lstResults->topLevelItem( i );
+
+    for ( int j = 0; j < layItem->childCount(); j++ )
+    {
+      highlightFeature( layItem->child( j ) );
+    }
+  }
+}
+
+void QgsIdentifyResults::highlightLayer()
+{
+  highlightLayer( lstResults->currentItem() );
+}
+
+void QgsIdentifyResults::highlightLayer( QTreeWidgetItem *item )
+{
+  QTreeWidgetItem *layItem = layerItem( item );
+  if ( !layItem )
+    return;
+
+  clearRubberbands();
+
+  for ( int i = 0; i < layItem->childCount(); i++ )
+  {
+    highlightFeature( layItem->child( i ) );
+  }
+}
+
+
+void QgsIdentifyResults::expandAll()
+{
+  lstResults->expandAll();
+}
+
+void QgsIdentifyResults::collapseAll()
+{
+  lstResults->collapseAll();
+}
+
+void QgsIdentifyResults::copyAttributeValue()
+{
+  QClipboard *clipboard = QApplication::clipboard();
+  QString text = lstResults->currentItem()->data( 1, Qt::DisplayRole ).toString();
+  QgsDebugMsg( QString( "set clipboard: %1" ).arg( text ) );
+  clipboard->setText( text );
+}
+
+void QgsIdentifyResults::copyFeatureAttributes()
+{
+  QClipboard *clipboard = QApplication::clipboard();
+  QString text;
+
+  int idx;
+  QList< QPair<QString, QString> > attributes;
+  retrieveAttributes( lstResults->currentItem(), attributes, idx );
+
+  for ( QList< QPair<QString, QString> >::iterator it = attributes.begin(); it != attributes.end(); it++ )
+  {
+    text += QString( "%1: %2\n" ).arg( it->first ).arg( it->second );
+  }
+
+  QgsDebugMsg( QString( "set clipboard: %1" ).arg( text ) );
+  clipboard->setText( text );
+}
