@@ -13,7 +13,7 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-/* $Id: qgslabeldialog.cpp 11280 2009-08-06 01:47:36Z telwertowski $ */
+/* $Id$ */
 #include "qgslabeldialog.h"
 #include "qgsfield.h"
 #include "qgslabel.h"
@@ -55,7 +55,7 @@ void QgsLabelDialog::init( )
 {
   QgsDebugMsg( "entering." );
 
-  QgsLabelAttributes * myLabelAttributes = mLabel->layerAttributes();
+  QgsLabelAttributes * myLabelAttributes = mLabel->labelAttributes();
   //populate a string list with all the field names which will be used to set up the
   //data bound combos
   QgsFieldMap& myFieldsMap = mLabel->fields();
@@ -188,7 +188,15 @@ void QgsLabelDialog::init( )
   {
     mFont.setItalic( false );
   }
-  mFont.setUnderline( myLabelAttributes->underline() );
+  if ( myLabelAttributes->underlineIsSet() )
+  {
+    mFont.setUnderline( myLabelAttributes->underline() );
+  }
+  else
+  {
+    mFont.setUnderline( false );
+  }
+
   mFontColor = myLabelAttributes->color();
 
   if ( myLabelAttributes->offsetIsSet() )
@@ -264,7 +272,6 @@ void QgsLabelDialog::init( )
   spinBufferSize->setValue( myLabelAttributes->bufferSize() );
   //TODO - transparency attributes for buffers
 
-  listWidget->setItemSelected( listWidget->item( 0 ), true );
 }
 
 
@@ -340,8 +347,8 @@ void QgsLabelDialog::apply()
   QgsDebugMsg( "entering." );
 
   //set the label props that are NOT bound to a field in the attributes tbl
-  //All of these are set in the layerAttributes member of the layer
-  QgsLabelAttributes * myLabelAttributes = mLabel->layerAttributes();
+  //All of these are set in the labelAttributes member of the layer
+  QgsLabelAttributes * myLabelAttributes = mLabel->labelAttributes();
   myLabelAttributes->setText( leDefaultLabel->text() );
   myLabelAttributes->setFamily( mFont.family() );
   int myTypeInt = 0;
