@@ -15,7 +15,7 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-/* $Id: qgsoptions.cpp 11617 2009-09-10 17:06:49Z timlinux $ */
+/* $Id$ */
 #include "qgsapplication.h"
 #include "qgsoptions.h"
 #include "qgis.h"
@@ -46,22 +46,43 @@ QgsOptions::QgsOptions( QWidget *parent, Qt::WFlags fl ) :
   connect( cmbTheme, SIGNAL( activated( const QString& ) ), this, SLOT( themeChanged( const QString& ) ) );
   connect( cmbTheme, SIGNAL( highlighted( const QString& ) ), this, SLOT( themeChanged( const QString& ) ) );
   connect( cmbTheme, SIGNAL( textChanged( const QString& ) ), this, SLOT( themeChanged( const QString& ) ) );
-  connect( buttonBox, SIGNAL( accepted() ), this, SLOT( accept() ) );
-  connect( buttonBox, SIGNAL( rejected() ), this, SLOT( reject() ) );
   connect( this, SIGNAL( accepted() ), this, SLOT( saveOptions() ) );
 
-  cmbIdentifyMode->addItem( tr( "current layer" ), 0 );
-  cmbIdentifyMode->addItem( tr( "top down, stop at first" ), 1 );
-  cmbIdentifyMode->addItem( tr( "top down" ), 2 );
+  cmbIdentifyMode->addItem( tr( "Current layer" ), 0 );
+  cmbIdentifyMode->addItem( tr( "Top down, stop at first" ), 1 );
+  cmbIdentifyMode->addItem( tr( "Top down" ), 2 );
 
   // read the current browser and set it
   QSettings settings;
   int identifyMode = settings.value( "/Map/identifyMode", 0 ).toInt();
   cmbIdentifyMode->setCurrentIndex( cmbIdentifyMode->findData( identifyMode ) );
+  cbxAutoFeatureForm->setChecked( settings.value( "/Map/identifyAutoFeatureForm", false ).toBool() );
   double identifyValue = settings.value( "/Map/identifyRadius", QGis::DEFAULT_IDENTIFY_RADIUS ).toDouble();
   QgsDebugMsg( QString( "Standard Identify radius setting read from settings file: %1" ).arg( identifyValue ) );
+  if ( identifyValue <= 0.0 )
+    identifyValue = QGis::DEFAULT_IDENTIFY_RADIUS;
+  spinBoxIdentifyValue->setMinimum( 0.01 );
   spinBoxIdentifyValue->setValue( identifyValue );
 
+
+  //local directories to search when looking for an SVG with a given basename
+  QString myPaths = settings.value( "svg/searchPathsForSVG", "" ).toString();
+  if ( !myPaths.isEmpty() )
+  {
+    QStringList myPathList = myPaths.split( "|" );
+    QStringList::const_iterator pathIt = myPathList.constBegin();
+    for ( ; pathIt != myPathList.constEnd(); ++pathIt )
+    {
+      QListWidgetItem* newItem = new QListWidgetItem( mListSVGPaths );
+      newItem->setText( *pathIt );
+      newItem->setFlags( Qt::ItemIsEditable | Qt::ItemIsEnabled | Qt::ItemIsSelectable );
+      mListSVGPaths->addItem( newItem );
+    }
+  }
+
+  //Network timeout
+  mNetworkTimeoutSpinBox->setValue( settings.value( "/qgis/networkAndProxy/networkTimeout", "60000" ).toInt() );
+
   //Web proxy settings
   grpProxy->setChecked( settings.value( "proxy/proxyEnabled", "0" ).toBool() );
   leProxyHost->setText( settings.value( "proxy/proxyHost", "" ).toString() );
@@ -132,7 +153,7 @@ QgsOptions::QgsOptions( QWidget *parent, Qt::WFlags fl ) :
 
 
   // add the themes to the combo box on the option dialog
-  QDir myThemeDir( QgsApplication::pkgDataPath() + "/themes/" );
+  QDir myThemeDir( ":/images/themes/" );
   myThemeDir.setFilter( QDir::Dirs );
   QStringList myDirList = myThemeDir.entryList( QStringList( "*" ) );
   cmbTheme->clear();
@@ -149,6 +170,9 @@ QgsOptions::QgsOptions( QWidget *parent, Qt::WFlags fl ) :
 
   //set the state of the checkboxes
   chkAntiAliasing->setChecked( settings.value( "/qgis/enable_anti_aliasing", false ).toBool() );
+  chkUseRenderCaching->setChecked( settings.value( "/qgis/enable_render_caching", false ).toBool() );
+
+  chkUseSymbologyNG->setChecked( settings.value( "/qgis/use_symbology_ng", false ).toBool() );
 
   // Slightly awkard here at the settings value is true to use QImage,
   // but the checkbox is true to use QPixmap
@@ -160,11 +184,11 @@ QgsOptions::QgsOptions( QWidget *parent, Qt::WFlags fl ) :
   cbxIdentifyResultsDocked->setChecked( settings.value( "/qgis/dockIdentifyResults", false ).toBool() );
   cbxAddPostgisDC->setChecked( settings.value( "/qgis/addPostgisDC", false ).toBool() );
 
-  //set the colour for selections
+  //set the color for selections
   int myRed = settings.value( "/qgis/default_selection_color_red", 255 ).toInt();
   int myGreen = settings.value( "/qgis/default_selection_color_green", 255 ).toInt();
   int myBlue = settings.value( "/qgis/default_selection_color_blue", 0 ).toInt();
-  pbnSelectionColour->setColor( QColor( myRed, myGreen, myBlue ) );
+  pbnSelectionColor->setColor( QColor( myRed, myGreen, myBlue ) );
 
   //set the default color for canvas background
   myRed = settings.value( "/qgis/default_canvas_color_red", 255 ).toInt();
@@ -176,7 +200,7 @@ QgsOptions::QgsOptions( QWidget *parent, Qt::WFlags fl ) :
   myRed = settings.value( "/qgis/default_measure_color_red", 180 ).toInt();
   myGreen = settings.value( "/qgis/default_measure_color_green", 180 ).toInt();
   myBlue = settings.value( "/qgis/default_measure_color_blue", 180 ).toInt();
-  pbnMeasureColour->setColor( QColor( myRed, myGreen, myBlue ) );
+  pbnMeasureColor->setColor( QColor( myRed, myGreen, myBlue ) );
 
   capitaliseCheckBox->setChecked( settings.value( "qgis/capitaliseLayerName", QVariant( false ) ).toBool() );
 
@@ -207,12 +231,12 @@ QgsOptions::QgsOptions( QWidget *parent, Qt::WFlags fl ) :
   myRed = settings.value( "/qgis/digitizing/line_color_red", 255 ).toInt();
   myGreen = settings.value( "/qgis/digitizing/line_color_green", 0 ).toInt();
   myBlue = settings.value( "/qgis/digitizing/line_color_blue", 0 ).toInt();
-  mLineColourToolButton->setColor( QColor( myRed, myGreen, myBlue ) );
+  mLineColorToolButton->setColor( QColor( myRed, myGreen, myBlue ) );
 
   //default snap mode
-  mDefaultSnapModeComboBox->insertItem( 0, tr( "to vertex" ), "to vertex" );
-  mDefaultSnapModeComboBox->insertItem( 1, tr( "to segment" ), "to segment" );
-  mDefaultSnapModeComboBox->insertItem( 2, tr( "to vertex and segment" ), "to vertex and segment" );
+  mDefaultSnapModeComboBox->insertItem( 0, tr( "To vertex" ), "to vertex" );
+  mDefaultSnapModeComboBox->insertItem( 1, tr( "To segment" ), "to segment" );
+  mDefaultSnapModeComboBox->insertItem( 2, tr( "To vertex and segment" ), "to vertex and segment" );
   QString defaultSnapString = settings.value( "/qgis/digitizing/default_snap_mode", "to vertex" ).toString();
   mDefaultSnapModeComboBox->setCurrentIndex( mDefaultSnapModeComboBox->findData( defaultSnapString ) );
   mDefaultSnappingToleranceSpinBox->setValue( settings.value( "/qgis/digitizing/default_snapping_tolerance", 0 ).toDouble() );
@@ -227,7 +251,7 @@ QgsOptions::QgsOptions( QWidget *parent, Qt::WFlags fl ) :
     index = mDefaultSnappingToleranceComboBox->findText( tr( "pixels" ) );
   }
   mDefaultSnappingToleranceComboBox->setCurrentIndex( index );
-  if ( settings.value( "/qgis/digitizing/search_radius_vertex_edit_unit", 0 ).toInt() == QgsTolerance::MapUnits )
+  if ( settings.value( "/qgis/digitizing/search_radius_vertex_edit_unit", QgsTolerance::Pixels ).toInt() == QgsTolerance::MapUnits )
   {
     index = mSearchRadiusVertexEditComboBox->findText( tr( "map units" ) );
   }
@@ -235,7 +259,7 @@ QgsOptions::QgsOptions( QWidget *parent, Qt::WFlags fl ) :
   {
     index = mSearchRadiusVertexEditComboBox->findText( tr( "pixels" ) );
   }
-  mSearchRadiusVertexEditComboBox->setCurrentIndex( settings.value( "/qgis/digitizing/search_radius_vertex_edit_unit", 0 ).toInt() );
+  mSearchRadiusVertexEditComboBox->setCurrentIndex( index );
 
   //vertex marker
   mMarkersOnlyForSelectedCheckBox->setChecked( settings.value( "/qgis/digitizing/marker_only_for_selected", false ).toBool() );
@@ -244,7 +268,7 @@ QgsOptions::QgsOptions( QWidget *parent, Qt::WFlags fl ) :
   mMarkerStyleComboBox->addItem( tr( "Cross" ) );
   mMarkerStyleComboBox->addItem( tr( "None" ) );
 
-  QString markerStyle = settings.value( "/qgis/digitizing/marker_style", "SemiTransparentCircle" ).toString();
+  QString markerStyle = settings.value( "/qgis/digitizing/marker_style", "Cross" ).toString();
   if ( markerStyle == "SemiTransparentCircle" )
   {
     mMarkerStyleComboBox->setCurrentIndex( mMarkerStyleComboBox->findText( tr( "Semi transparent circle" ) ) );
@@ -257,7 +281,7 @@ QgsOptions::QgsOptions( QWidget *parent, Qt::WFlags fl ) :
   {
     mMarkerStyleComboBox->setCurrentIndex( mMarkerStyleComboBox->findText( tr( "None" ) ) );
   }
-  mMarkerSizeSpinBox->setValue( settings.value( "/qgis/digitizing/marker_size", 7 ).toInt()*2+1 );
+  mMarkerSizeSpinBox->setValue( settings.value( "/qgis/digitizing/marker_size", 3 ).toInt() );
 
   chkDisableAttributeValuesDlg->setChecked( settings.value( "/qgis/digitizing/disable_enter_attribute_values_dialog", false ).toBool() );
 
@@ -273,22 +297,45 @@ QgsOptions::QgsOptions( QWidget *parent, Qt::WFlags fl ) :
   mOverlayAlgorithmComboBox->insertItem( 4, tr( "Popmusic chain (very slow)" ) );
 
   QString overlayAlgorithmString = settings.value( "qgis/overlayPlacementAlgorithm", "Central point" ).toString();
-  if ( overlayAlgorithmString == "Chain" ) {mOverlayAlgorithmComboBox->setCurrentIndex( 1 );}
-  else if ( overlayAlgorithmString == "Popmusic tabu chain" ) {mOverlayAlgorithmComboBox->setCurrentIndex( 2 );}
-  else if ( overlayAlgorithmString == "Popmusic tabu" ) {mOverlayAlgorithmComboBox->setCurrentIndex( 3 );}
-  else if ( overlayAlgorithmString == "Popmusic chain" ) {mOverlayAlgorithmComboBox->setCurrentIndex( 4 );}
-  else {mOverlayAlgorithmComboBox->setCurrentIndex( 0 );} //default is central point
+  if ( overlayAlgorithmString == "Chain" )
+  {
+    mOverlayAlgorithmComboBox->setCurrentIndex( 1 );
+  }
+  else if ( overlayAlgorithmString == "Popmusic tabu chain" )
+  {
+    mOverlayAlgorithmComboBox->setCurrentIndex( 2 );
+  }
+  else if ( overlayAlgorithmString == "Popmusic tabu" )
+  {
+    mOverlayAlgorithmComboBox->setCurrentIndex( 3 );
+  }
+  else if ( overlayAlgorithmString == "Popmusic chain" )
+  {
+    mOverlayAlgorithmComboBox->setCurrentIndex( 4 );
+  }
+  else
+  {
+    mOverlayAlgorithmComboBox->setCurrentIndex( 0 );
+  } //default is central point
+
+  restoreGeometry( settings.value( "/Windows/Options/geometry" ).toByteArray() );
+  listWidget->setCurrentRow( settings.value( "/Windows/Options/row" ).toInt() );
 }
 
 //! Destructor
-QgsOptions::~QgsOptions() {}
-
-void QgsOptions::on_pbnSelectionColour_clicked()
+QgsOptions::~QgsOptions()
 {
-  QColor color = QColorDialog::getColor( pbnSelectionColour->color(), this );
+  QSettings settings;
+  settings.setValue( "/Windows/Options/geometry", saveGeometry() );
+  settings.setValue( "/Windows/Options/row", listWidget->currentRow() );
+}
+
+void QgsOptions::on_pbnSelectionColor_clicked()
+{
+  QColor color = QColorDialog::getColor( pbnSelectionColor->color(), this );
   if ( color.isValid() )
   {
-    pbnSelectionColour->setColor( color );
+    pbnSelectionColor->setColor( color );
   }
 }
 
@@ -301,21 +348,21 @@ void QgsOptions::on_pbnCanvasColor_clicked()
   }
 }
 
-void QgsOptions::on_pbnMeasureColour_clicked()
+void QgsOptions::on_pbnMeasureColor_clicked()
 {
-  QColor color = QColorDialog::getColor( pbnMeasureColour->color(), this );
+  QColor color = QColorDialog::getColor( pbnMeasureColor->color(), this );
   if ( color.isValid() )
   {
-    pbnMeasureColour->setColor( color );
+    pbnMeasureColor->setColor( color );
   }
 }
 
-void QgsOptions::on_mLineColourToolButton_clicked()
+void QgsOptions::on_mLineColorToolButton_clicked()
 {
-  QColor color = QColorDialog::getColor( mLineColourToolButton->color(), this );
+  QColor color = QColorDialog::getColor( mLineColorToolButton->color(), this );
   if ( color.isValid() )
   {
-    mLineColourToolButton->setColor( color );
+    mLineColorToolButton->setColor( color );
   }
 }
 
@@ -335,6 +382,22 @@ QString QgsOptions::theme()
 void QgsOptions::saveOptions()
 {
   QSettings settings;
+
+  //search directories for svgs
+  QString myPaths;
+  for ( int i = 0; i < mListSVGPaths->count(); ++i )
+  {
+    if ( i != 0 )
+    {
+      myPaths += "|";
+    }
+    myPaths += mListSVGPaths->item( i )->text();
+  }
+  settings.setValue( "svg/searchPathsForSVG", myPaths );
+
+  //Network timeout
+  settings.setValue( "/qgis/networkAndProxy/networkTimeout", mNetworkTimeoutSpinBox->value() );
+
   //Web proxy settings
   settings.setValue( "proxy/proxyEnabled", grpProxy->isChecked() );
   settings.setValue( "proxy/proxyHost", leProxyHost->text() );
@@ -343,6 +406,7 @@ void QgsOptions::saveOptions()
   settings.setValue( "proxy/proxyPassword", leProxyPassword->text() );
   settings.setValue( "proxy/proxyType", mProxyTypeComboBox->currentText() );
 
+
   //url to exclude from proxys
   QString proxyExcludeString;
   for ( int i = 0; i < mExcludeUrlListWidget->count(); ++i )
@@ -357,6 +421,7 @@ void QgsOptions::saveOptions()
 
   //general settings
   settings.setValue( "/Map/identifyMode", cmbIdentifyMode->itemData( cmbIdentifyMode->currentIndex() ).toInt() );
+  settings.setValue( "/Map/identifyAutoFeatureForm", cbxAutoFeatureForm->isChecked() );
   settings.setValue( "/Map/identifyRadius", spinBoxIdentifyValue->value() );
   settings.setValue( "/qgis/showLegendClassifiers", cbxLegendClassifiers->isChecked() );
   settings.setValue( "/qgis/hideSplash", cbxHideSplash->isChecked() );
@@ -365,7 +430,9 @@ void QgsOptions::saveOptions()
   settings.setValue( "/qgis/addPostgisDC", cbxAddPostgisDC->isChecked() );
   settings.setValue( "/qgis/new_layers_visible", chkAddedVisibility->isChecked() );
   settings.setValue( "/qgis/enable_anti_aliasing", chkAntiAliasing->isChecked() );
+  settings.setValue( "/qgis/enable_render_caching", chkUseRenderCaching->isChecked() );
   settings.setValue( "/qgis/use_qimage_to_render", !( chkUseQPixmap->isChecked() ) );
+  settings.setValue( "/qgis/use_symbology_ng", chkUseSymbologyNG->isChecked() );
   settings.setValue( "qgis/capitaliseLayerName", capitaliseCheckBox->isChecked() );
   settings.setValue( "qgis/askToSaveProjectChanges", chbAskToSaveProjectChanges->isChecked() );
   settings.setValue( "qgis/warnOldProjectVersion", chbWarnOldProjectVersion->isChecked() );
@@ -433,8 +500,8 @@ void QgsOptions::saveOptions()
     settings.setValue( "/qgis/measure/displayunits", "meters" );
   }
   settings.setValue( "/qgis/measure/ellipsoid", getEllipsoidAcronym( cmbEllipsoid->currentText() ) );
-  //set the colour for selections
-  QColor myColor = pbnSelectionColour->color();
+  //set the color for selections
+  QColor myColor = pbnSelectionColor->color();
   settings.setValue( "/qgis/default_selection_color_red", myColor.red() );
   settings.setValue( "/qgis/default_selection_color_green", myColor.green() );
   settings.setValue( "/qgis/default_selection_color_blue", myColor.blue() );
@@ -446,7 +513,7 @@ void QgsOptions::saveOptions()
   settings.setValue( "/qgis/default_canvas_color_blue", myColor.blue() );
 
   //set the default color for the measure tool
-  myColor = pbnMeasureColour->color();
+  myColor = pbnMeasureColor->color();
   settings.setValue( "/qgis/default_measure_color_red", myColor.red() );
   settings.setValue( "/qgis/default_measure_color_green", myColor.green() );
   settings.setValue( "/qgis/default_measure_color_blue", myColor.blue() );
@@ -456,7 +523,7 @@ void QgsOptions::saveOptions()
 
   //digitizing
   settings.setValue( "/qgis/digitizing/line_width", mLineWidthSpinBox->value() );
-  QColor digitizingColor = mLineColourToolButton->color();
+  QColor digitizingColor = mLineColorToolButton->color();
   settings.setValue( "/qgis/digitizing/line_color_red", digitizingColor.red() );
   settings.setValue( "/qgis/digitizing/line_color_green", digitizingColor.green() );
   settings.setValue( "/qgis/digitizing/line_color_blue", digitizingColor.blue() );
@@ -486,7 +553,7 @@ void QgsOptions::saveOptions()
   {
     settings.setValue( "/qgis/digitizing/marker_style", "None" );
   }
-  settings.setValue( "/qgis/digitizing/marker_size", (mMarkerSizeSpinBox->value()-1)/2 );
+  settings.setValue( "/qgis/digitizing/marker_size", ( mMarkerSizeSpinBox->value() ) );
 
   settings.setValue( "/qgis/digitizing/disable_enter_attribute_values_dialog", chkDisableAttributeValuesDlg->isChecked() );
 
@@ -530,7 +597,9 @@ void QgsOptions::on_chkAntiAliasing_stateChanged()
   // used (we we can. but it then doesn't do anti-aliasing, and this
   // will confuse people).
   if ( chkAntiAliasing->isChecked() )
+  {
     chkUseQPixmap->setChecked( false );
+  }
 
 }
 
@@ -540,7 +609,9 @@ void QgsOptions::on_chkUseQPixmap_stateChanged()
   // used (we we can. but it then doesn't do anti-aliasing, and this
   // will confuse people).
   if ( chkUseQPixmap->isChecked() )
+  {
     chkAntiAliasing->setChecked( false );
+  }
 
 }
 
@@ -671,6 +742,32 @@ QStringList QgsOptions::i18nList()
   return myList;
 }
 
+void QgsOptions::on_mBtnAddSVGPath_clicked()
+{
+  QString myDir = QFileDialog::getExistingDirectory(
+                    this,
+                    tr( "Choose a directory" ),
+                    QDir::toNativeSeparators( QDir::homePath() ),
+                    QFileDialog::ShowDirsOnly
+                  );
+
+  if ( ! myDir.isEmpty() )
+  {
+    QListWidgetItem* newItem = new QListWidgetItem( mListSVGPaths );
+    newItem->setText( myDir );
+    newItem->setFlags( Qt::ItemIsEditable | Qt::ItemIsEnabled | Qt::ItemIsSelectable );
+    mListSVGPaths->addItem( newItem );
+    mListSVGPaths->setCurrentItem( newItem );
+  }
+}
+
+void QgsOptions::on_mBtnRemoveSVGPath_clicked()
+{
+  int currentRow = mListSVGPaths->currentRow();
+  QListWidgetItem* itemToRemove = mListSVGPaths->takeItem( currentRow );
+  delete itemToRemove;
+}
+
 void QgsOptions::on_mAddUrlPushButton_clicked()
 {
   QListWidgetItem* newItem = new QListWidgetItem( mExcludeUrlListWidget );
