@@ -15,11 +15,12 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-/* $Id: qgsprojectproperties.cpp 11578 2009-09-06 21:32:16Z jef $ */
+/* $Id$ */
 
 #include "qgsprojectproperties.h"
 
 //qgis includes
+#include "qgsavoidintersectionsdialog.h"
 #include "qgscontexthelp.h"
 #include "qgscoordinatetransform.h"
 #include "qgslogger.h"
@@ -34,7 +35,7 @@
 
 //qt includes
 #include <QColorDialog>
-#include <QHeaderView>	// Qt 4.4
+#include <QHeaderView>  // Qt 4.4
 #include "qgslogger.h"
 
 //stdc++ includes
@@ -83,24 +84,24 @@ QgsProjectProperties::QgsProjectProperties( QgsMapCanvas* mapCanvas, QWidget *pa
     radManual->setChecked( true );
   }
 
-  cbxAbsolutePath->setChecked( QgsProject::instance()->readBoolEntry( "Paths", "/Absolute", true ) );
+  cbxAbsolutePath->setCurrentIndex( QgsProject::instance()->readBoolEntry( "Paths", "/Absolute", true ) ? 0 : 1 );
 
   int dp = QgsProject::instance()->readNumEntry( "PositionPrecision", "/DecimalPlaces" );
   spinBoxDP->setValue( dp );
 
-  //get the colour selections and set the button colour accordingly
+  //get the color selections and set the button color accordingly
   int myRedInt = QgsProject::instance()->readNumEntry( "Gui", "/SelectionColorRedPart", 255 );
   int myGreenInt = QgsProject::instance()->readNumEntry( "Gui", "/SelectionColorGreenPart", 255 );
   int myBlueInt = QgsProject::instance()->readNumEntry( "Gui", "/SelectionColorBluePart", 0 );
-  QColor myColour = QColor( myRedInt, myGreenInt, myBlueInt );
-  pbnSelectionColour->setColor( myColour );
+  QColor myColor = QColor( myRedInt, myGreenInt, myBlueInt );
+  pbnSelectionColor->setColor( myColor );
 
-  //get the colour for map canvas background and set button colour accordingly (default white)
+  //get the color for map canvas background and set button color accordingly (default white)
   myRedInt = QgsProject::instance()->readNumEntry( "Gui", "/CanvasColorRedPart", 255 );
   myGreenInt = QgsProject::instance()->readNumEntry( "Gui", "/CanvasColorGreenPart", 255 );
   myBlueInt = QgsProject::instance()->readNumEntry( "Gui", "/CanvasColorBluePart", 255 );
-  myColour = QColor( myRedInt, myGreenInt, myBlueInt );
-  pbnCanvasColor->setColor( myColour );
+  myColor = QColor( myRedInt, myGreenInt, myBlueInt );
+  pbnCanvasColor->setColor( myColor );
 
   //read the digitizing settings
   int topologicalEditing = QgsProject::instance()->readNumEntry( "Digitizing", "/TopologicalEditing", 0 );
@@ -113,14 +114,16 @@ QgsProjectProperties::QgsProjectProperties( QgsMapCanvas* mapCanvas, QWidget *pa
     mEnableTopologicalEditingCheckBox->setCheckState( Qt::Unchecked );
   }
 
-  int avoidPolygonIntersections = QgsProject::instance()->readNumEntry( "Digitizing", "/AvoidPolygonIntersections", 0 );
-  if ( avoidPolygonIntersections != 0 )
+  bool avoidIntersectionListOk;
+  mAvoidIntersectionsSettings.clear();
+  QStringList avoidIntersectionsList = QgsProject::instance()->readListEntry( "Digitizing", "/AvoidIntersectionsList", &avoidIntersectionListOk );
+  if ( avoidIntersectionListOk )
   {
-    mAvoidIntersectionsCheckBox->setCheckState( Qt::Checked );
-  }
-  else
-  {
-    mAvoidIntersectionsCheckBox->setCheckState( Qt::Unchecked );
+    QStringList::const_iterator avoidIt = avoidIntersectionsList.constBegin();
+    for ( ; avoidIt != avoidIntersectionsList.constEnd(); ++avoidIt )
+    {
+      mAvoidIntersectionsSettings.insert( *avoidIt );
+    }
   }
 
   bool layerIdListOk, enabledListOk, toleranceListOk, toleranceUnitListOk, snapToListOk;
@@ -229,7 +232,9 @@ QgsProjectProperties::QgsProjectProperties( QgsMapCanvas* mapCanvas, QWidget *pa
     twi->setData( Qt::UserRole, it.key() );
     twIdentifyLayers->setVerticalHeaderItem( i, twi );
 
-    twIdentifyLayers->setItem( i, 0, new QTableWidgetItem( currentLayer->name() ) );
+    twi = new QTableWidgetItem( currentLayer->name() );
+    twi->setFlags( twi->flags() & ~Qt::ItemIsEditable );
+    twIdentifyLayers->setItem( i, 0, twi );
 
     QString type;
     if ( currentLayer->type() == QgsMapLayer::VectorLayer )
@@ -238,7 +243,7 @@ QgsProjectProperties::QgsProjectProperties( QgsMapCanvas* mapCanvas, QWidget *pa
     }
     else if ( currentLayer->type() == QgsMapLayer::RasterLayer )
     {
-      QgsRasterLayer *rl = dynamic_cast<QgsRasterLayer *>( currentLayer );
+      QgsRasterLayer *rl = qobject_cast<QgsRasterLayer *>( currentLayer );
 
       if ( rl && rl->providerKey() == "wms" )
       {
@@ -250,7 +255,9 @@ QgsProjectProperties::QgsProjectProperties( QgsMapCanvas* mapCanvas, QWidget *pa
       }
     }
 
-    twIdentifyLayers->setItem( i, 1, new QTableWidgetItem( type ) );
+    twi = new QTableWidgetItem( type );
+    twi->setFlags( twi->flags() & ~Qt::ItemIsEditable );
+    twIdentifyLayers->setItem( i, 1, twi );
 
     QCheckBox *cb = new QCheckBox();
     cb->setChecked( !noIdentifyLayerIdList.contains( currentLayer->getLayerID() ) );
@@ -289,6 +296,10 @@ void QgsProjectProperties::setMapUnits( QGis::UnitType unit )
   {
     radFeet->setChecked( true );
   }
+  else if ( unit == QGis::DegreesMinutesSeconds )
+  {
+    radDMS->setChecked( true );
+  }
   else
   {
     radDecimalDegrees->setChecked( true );
@@ -326,6 +337,10 @@ void QgsProjectProperties::apply()
   {
     mapUnit = QGis::Feet;
   }
+  else if ( radDMS->isChecked() )
+  {
+    mapUnit = QGis::DegreesMinutesSeconds;
+  }
   else
   {
     mapUnit = QGis::Degrees;
@@ -373,26 +388,34 @@ void QgsProjectProperties::apply()
   // Announce that we may have a new display precision setting
   emit displayPrecisionChanged();
 
-  QgsProject::instance()->writeEntry( "Paths", "/Absolute", cbxAbsolutePath->isChecked() );
+  QgsProject::instance()->writeEntry( "Paths", "/Absolute", cbxAbsolutePath->currentIndex() == 0 );
 
-  //set the colour for selections
-  QColor myColour = pbnSelectionColour->color();
-  QgsProject::instance()->writeEntry( "Gui", "/SelectionColorRedPart", myColour.red() );
-  QgsProject::instance()->writeEntry( "Gui", "/SelectionColorGreenPart", myColour.green() );
-  QgsProject::instance()->writeEntry( "Gui", "/SelectionColorBluePart", myColour.blue() );
-  QgsRenderer::setSelectionColor( myColour );
+  //set the color for selections
+  QColor myColor = pbnSelectionColor->color();
+  QgsProject::instance()->writeEntry( "Gui", "/SelectionColorRedPart", myColor.red() );
+  QgsProject::instance()->writeEntry( "Gui", "/SelectionColorGreenPart", myColor.green() );
+  QgsProject::instance()->writeEntry( "Gui", "/SelectionColorBluePart", myColor.blue() );
+  QgsRenderer::setSelectionColor( myColor );
 
-  //set the colour for canvas
-  myColour = pbnCanvasColor->color();
-  QgsProject::instance()->writeEntry( "Gui", "/CanvasColorRedPart", myColour.red() );
-  QgsProject::instance()->writeEntry( "Gui", "/CanvasColorGreenPart", myColour.green() );
-  QgsProject::instance()->writeEntry( "Gui", "/CanvasColorBluePart", myColour.blue() );
+  //set the color for canvas
+  myColor = pbnCanvasColor->color();
+  QgsProject::instance()->writeEntry( "Gui", "/CanvasColorRedPart", myColor.red() );
+  QgsProject::instance()->writeEntry( "Gui", "/CanvasColorGreenPart", myColor.green() );
+  QgsProject::instance()->writeEntry( "Gui", "/CanvasColorBluePart", myColor.blue() );
 
   //write the digitizing settings
   int topologicalEditingEnabled = ( mEnableTopologicalEditingCheckBox->checkState() == Qt::Checked ) ? 1 : 0;
   QgsProject::instance()->writeEntry( "Digitizing", "/TopologicalEditing", topologicalEditingEnabled );
-  int avoidPolygonIntersectionsEnabled = ( mAvoidIntersectionsCheckBox->checkState() == Qt::Checked ) ? 1 : 0;
-  QgsProject::instance()->writeEntry( "Digitizing", "/AvoidPolygonIntersections", avoidPolygonIntersectionsEnabled );
+
+  //store avoid intersection layers
+  QStringList avoidIntersectionList;
+  QSet<QString>::const_iterator avoidIt = mAvoidIntersectionsSettings.constBegin();
+  for ( ; avoidIt != mAvoidIntersectionsSettings.constEnd(); ++avoidIt )
+  {
+    avoidIntersectionList.append( *avoidIt );
+  }
+  QgsProject::instance()->writeEntry( "Digitizing", "/AvoidIntersectionsList", avoidIntersectionList );
+
 
   QMap<QString, LayerEntry>::const_iterator layerEntryIt;
 
@@ -442,7 +465,7 @@ void QgsProjectProperties::apply()
   QStringList noIdentifyLayerList;
   for ( int i = 0; i < twIdentifyLayers->rowCount(); i++ )
   {
-    QCheckBox *cb = dynamic_cast<QCheckBox*>( twIdentifyLayers->cellWidget( i, 2 ) );
+    QCheckBox *cb = qobject_cast<QCheckBox *>( twIdentifyLayers->cellWidget( i, 2 ) );
     if ( cb && !cb->isChecked() )
     {
       QString id = twIdentifyLayers->verticalHeaderItem( i )->data( Qt::UserRole ).toString();
@@ -452,7 +475,7 @@ void QgsProjectProperties::apply()
 
   QgsProject::instance()->writeEntry( "Identify", "/disabledLayers", noIdentifyLayerList );
 
-  //todo XXX set canvas colour
+  //todo XXX set canvas color
   emit refresh();
 }
 
@@ -466,12 +489,12 @@ void QgsProjectProperties::showProjectionsTab()
   tabWidget->setCurrentIndex( 1 );
 }
 
-void QgsProjectProperties::on_pbnSelectionColour_clicked()
+void QgsProjectProperties::on_pbnSelectionColor_clicked()
 {
-  QColor color = QColorDialog::getColor( pbnSelectionColour->color(), this );
+  QColor color = QColorDialog::getColor( pbnSelectionColor->color(), this );
   if ( color.isValid() )
   {
-    pbnSelectionColour->setColor( color );
+    pbnSelectionColor->setColor( color );
   }
 }
 
@@ -484,10 +507,13 @@ void QgsProjectProperties::on_pbnCanvasColor_clicked()
   }
 }
 
-void QgsProjectProperties::on_buttonBox_helpRequested()
+void QgsProjectProperties::on_mAvoidIntersectionsPushButton_clicked()
 {
-  QgsDebugMsg( "running help" );
-  QgsContextHelp::run( context_id );
+  QgsAvoidIntersectionsDialog d( mMapCanvas, mAvoidIntersectionsSettings );
+  if ( d.exec() == QDialog::Accepted )
+  {
+    d.enabledLayers( mAvoidIntersectionsSettings );
+  }
 }
 
 void QgsProjectProperties::on_mSnappingOptionsPushButton_clicked()
