@@ -15,7 +15,7 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-/* $Id: qgsvectorlayerproperties.cpp 11371 2009-08-14 09:20:23Z jef $ */
+/* $Id$ */
 
 #include <memory>
 
@@ -26,6 +26,7 @@
 #include "qgscontexthelp.h"
 #include "qgscontinuouscolordialog.h"
 #include "qgscoordinatetransform.h"
+#include "qgsfieldcalculator.h"
 #include "qgsgraduatedsymboldialog.h"
 #include "qgslabeldialog.h"
 #include "qgslabel.h"
@@ -41,11 +42,8 @@
 #include "qgsconfig.h"
 #include "qgsvectordataprovider.h"
 #include "qgsvectoroverlayplugin.h"
-
-#ifdef HAVE_POSTGRESQL
-#include "qgspgquerybuilder.h"
-#include "../providers/postgres/qgspostgresprovider.h"
-#endif
+#include "qgsquerybuilder.h"
+#include "qgsdatasourceuri.h"
 
 #include <QMessageBox>
 #include <QDir>
@@ -56,6 +54,10 @@
 #include <QComboBox>
 #include <QCheckBox>
 
+#include "qgsrendererv2propertiesdialog.h"
+#include "qgsstylev2.h"
+#include "qgssymbologyv2conversion.h"
+
 #if QT_VERSION < 0x040300
 #define toPlainText() text()
 #endif
@@ -92,7 +94,9 @@ QgsVectorLayerProperties::QgsVectorLayerProperties(
   mAddAttributeButton->setIcon( QgisApp::getThemeIcon( "/mActionNewAttribute.png" ) );
   mDeleteAttributeButton->setIcon( QgisApp::getThemeIcon( "/mActionDeleteAttribute.png" ) );
   mToggleEditingButton->setIcon( QgisApp::getThemeIcon( "/mActionToggleEditing.png" ) );
+  mCalculateFieldButton->setIcon( QgisApp::getThemeIcon( "/mActionCalculateField.png" ) );
 
+  connect( btnUseNewSymbology, SIGNAL( clicked() ), this, SLOT( useNewSymbology() ) );
 
   // Create the Label dialog tab
   QVBoxLayout *layout = new QVBoxLayout( labelOptionsFrame );
@@ -126,6 +130,9 @@ QgsVectorLayerProperties::QgsVectorLayerProperties(
   leSpatialRefSys->setText( layer->srs().toProj4() );
   leSpatialRefSys->setCursorPosition( 0 );
 
+  leEditForm->setText( layer->editForm() );
+  leEditFormInit->setText( layer->editFormInit() );
+
   connect( sliderTransparency, SIGNAL( valueChanged( int ) ), this, SLOT( sliderTransparency_valueChanged( int ) ) );
 
   //for each overlay plugin create a new tab
@@ -136,12 +143,19 @@ QgsVectorLayerProperties::QgsVectorLayerProperties(
   for ( ; it != overlayPluginList.constEnd(); ++it )
   {
     QgsApplyDialog* d = ( *it )->dialog( lyr );
-    position = tabWidget->addTab( d, ( *it )->name() );
-    tabWidget->setCurrentIndex( position ); //ugly, but otherwise the properties dialog is a mess
+    position = stackedWidget->insertWidget( stackedWidget->count(), qobject_cast<QDialog*>( d ) );
+    stackedWidget->setCurrentIndex( position ); //ugly, but otherwise the properties dialog is a mess
     mOverlayDialogs.push_back( d );
+    //shamelessly hard coded - what will we do if other types of layer plugins exist? TS
+    QListWidgetItem * mypItem = new QListWidgetItem( QgisApp::getThemeIcon( "propertyicons/diagram.png" ), ( *it )->name() );
+    listWidget->insertItem( stackedWidget->count(), mypItem );
   }
 
-  tabWidget->setCurrentIndex( 0 );
+  stackedWidget->setCurrentIndex( 0 );
+
+  QSettings settings;
+  restoreGeometry( settings.value( "/Windows/VectorLayerProperties/geometry" ).toByteArray() );
+  listWidget->setCurrentRow( settings.value( "/Windows/VectorLayerProperties/row" ).toInt() );
 } // QgsVectorLayerProperties ctor
 
 void QgsVectorLayerProperties::loadRows()
@@ -150,16 +164,16 @@ void QgsVectorLayerProperties::loadRows()
 
   tblAttributes->clear();
 
-  tblAttributes->setColumnCount( 8 );
+  tblAttributes->setColumnCount( attrColCount );
   tblAttributes->setRowCount( fields.size() );
-  tblAttributes->setHorizontalHeaderItem( 0, new QTableWidgetItem( tr( "id" ) ) );
-  tblAttributes->setHorizontalHeaderItem( 1, new QTableWidgetItem( tr( "name" ) ) );
-  tblAttributes->setHorizontalHeaderItem( 2, new QTableWidgetItem( tr( "type" ) ) );
-  tblAttributes->setHorizontalHeaderItem( 3, new QTableWidgetItem( tr( "length" ) ) );
-  tblAttributes->setHorizontalHeaderItem( 4, new QTableWidgetItem( tr( "precision" ) ) );
-  tblAttributes->setHorizontalHeaderItem( 5, new QTableWidgetItem( tr( "comment" ) ) );
-  tblAttributes->setHorizontalHeaderItem( 6, new QTableWidgetItem( tr( "edit widget" ) ) );
-  tblAttributes->setHorizontalHeaderItem( 7, new QTableWidgetItem( tr( "alias" ) ) );
+  tblAttributes->setHorizontalHeaderItem( attrIdCol, new QTableWidgetItem( tr( "id" ) ) );
+  tblAttributes->setHorizontalHeaderItem( attrNameCol, new QTableWidgetItem( tr( "name" ) ) );
+  tblAttributes->setHorizontalHeaderItem( attrTypeCol, new QTableWidgetItem( tr( "type" ) ) );
+  tblAttributes->setHorizontalHeaderItem( attrLengthCol, new QTableWidgetItem( tr( "length" ) ) );
+  tblAttributes->setHorizontalHeaderItem( attrPrecCol, new QTableWidgetItem( tr( "precision" ) ) );
+  tblAttributes->setHorizontalHeaderItem( attrCommentCol, new QTableWidgetItem( tr( "comment" ) ) );
+  tblAttributes->setHorizontalHeaderItem( attrEditTypeCol, new QTableWidgetItem( tr( "edit widget" ) ) );
+  tblAttributes->setHorizontalHeaderItem( attrAliasCol, new QTableWidgetItem( tr( "alias" ) ) );
 
   tblAttributes->setSelectionBehavior( QAbstractItemView::SelectRows );
   tblAttributes->setSelectionMode( QAbstractItemView::MultiSelection );
@@ -173,42 +187,43 @@ void QgsVectorLayerProperties::loadRows()
 
 void QgsVectorLayerProperties::setRow( int row, int idx, const QgsField &field )
 {
-  tblAttributes->setItem( row, 0, new QTableWidgetItem( QString::number( idx ) ) );
-  tblAttributes->setItem( row, 1, new QTableWidgetItem( field.name() ) );
-  tblAttributes->setItem( row, 2, new QTableWidgetItem( field.typeName() ) );
-  tblAttributes->setItem( row, 3, new QTableWidgetItem( QString::number( field.length() ) ) );
-  tblAttributes->setItem( row, 4, new QTableWidgetItem( QString::number( field.precision() ) ) );
-  tblAttributes->setItem( row, 5, new QTableWidgetItem( field.comment() ) );
+  tblAttributes->setItem( row, attrIdCol, new QTableWidgetItem( QString::number( idx ) ) );
+  tblAttributes->setItem( row, attrNameCol, new QTableWidgetItem( field.name() ) );
+  tblAttributes->setItem( row, attrTypeCol, new QTableWidgetItem( field.typeName() ) );
+  tblAttributes->setItem( row, attrLengthCol, new QTableWidgetItem( QString::number( field.length() ) ) );
+  tblAttributes->setItem( row, attrPrecCol, new QTableWidgetItem( QString::number( field.precision() ) ) );
+  tblAttributes->setItem( row, attrCommentCol, new QTableWidgetItem( field.comment() ) );
 
-  for ( int i = 0; i < 6; i++ )
+  for ( int i = 0; i < attrEditTypeCol; i++ )
     tblAttributes->item( row, i )->setFlags( tblAttributes->item( row, i )->flags() & ~Qt::ItemIsEditable );
 
-  QPushButton * pb = new QPushButton( editTypeButtonText( layer->editType( idx ) ) );
-  tblAttributes->setCellWidget( row, 6, pb );
+  QPushButton *pb = new QPushButton( editTypeButtonText( layer->editType( idx ) ) );
+  tblAttributes->setCellWidget( row, attrEditTypeCol, pb );
   connect( pb, SIGNAL( pressed() ), this, SLOT( attributeTypeDialog( ) ) );
   mButtonMap.insert( idx, pb );
 
   //set the alias for the attribute
-  tblAttributes->setItem( row, 7, new QTableWidgetItem( layer->attributeAlias( idx ) ) );
-
+  tblAttributes->setItem( row, attrAliasCol, new QTableWidgetItem( layer->attributeAlias( idx ) ) );
 }
 
 QgsVectorLayerProperties::~QgsVectorLayerProperties()
 {
   disconnect( labelDialog, SIGNAL( labelSourceSet() ), this, SLOT( setLabelCheckBox() ) );
+
+  QSettings settings;
+  settings.setValue( "/Windows/VectorLayerProperties/geometry", saveGeometry() );
+  settings.setValue( "/Windows/VectorLayerProperties/row", listWidget->currentRow() );
 }
 
 void QgsVectorLayerProperties::attributeTypeDialog( )
 {
-  int index = -1;
-  QMap<int, QPushButton*>::iterator it = mButtonMap.begin();
-  for ( ; it != mButtonMap.end() ; it++ )
-  {
-    if ( it.value()->isDown() )
-    {
-      index = it.key();
-    }
-  }
+  QPushButton *pb = qobject_cast<QPushButton *>( sender() );
+  if ( !pb )
+    return;
+
+  int index = mButtonMap.key( pb, -1 );
+  if ( index == -1 )
+    return;
 
   QgsAttributeTypeDialog attributeTypeDialog( layer );
 
@@ -239,10 +254,16 @@ void QgsVectorLayerProperties::attributeTypeDialog( )
     attributeTypeDialog.setIndex( index );
   }
 
+  if ( mCheckedStates.contains( index ) )
+  {
+    attributeTypeDialog.setCheckedState( mCheckedStates[index].first, mCheckedStates[index].second );
+  }
+
   if ( !attributeTypeDialog.exec() )
     return;
 
   QgsVectorLayer::EditType editType = attributeTypeDialog.editType();
+
   mEditTypeMap.insert( index, editType );
 
   QString buttonText;
@@ -255,12 +276,13 @@ void QgsVectorLayerProperties::attributeTypeDialog( )
     case QgsVectorLayer::SliderRange:
       mRanges.insert( index, attributeTypeDialog.rangeData() );
       break;
+    case QgsVectorLayer::CheckBox:
+      mCheckedStates.insert( index, attributeTypeDialog.checkedState() );
     default:
       break;
   }
-  QPushButton *pb = dynamic_cast<QPushButton*>( tblAttributes->cellWidget( index, 6 ) );
-  pb->setText( editTypeButtonText( editType ) );
 
+  pb->setText( editTypeButtonText( editType ) );
 }
 
 
@@ -358,6 +380,7 @@ void QgsVectorLayerProperties::updateButtons()
     int cap = layer->dataProvider()->capabilities();
     mAddAttributeButton->setEnabled( cap & QgsVectorDataProvider::AddAttributes );
     mDeleteAttributeButton->setEnabled( cap & QgsVectorDataProvider::DeleteAttributes );
+    mCalculateFieldButton->setEnabled( cap & QgsVectorDataProvider::ChangeAttributeValues );
     mToggleEditingButton->setChecked( true );
   }
   else
@@ -365,6 +388,7 @@ void QgsVectorLayerProperties::updateButtons()
     mAddAttributeButton->setEnabled( false );
     mDeleteAttributeButton->setEnabled( false );
     mToggleEditingButton->setChecked( false );
+    mCalculateFieldButton->setEnabled( false );
   }
 }
 
@@ -423,7 +447,7 @@ void QgsVectorLayerProperties::reset( void )
 
   // populate the general information
   txtDisplayName->setText( layer->name() );
-  pbnQueryBuilder->setWhatsThis( tr( "This button opens the PostgreSQL query "
+  pbnQueryBuilder->setWhatsThis( tr( "This button opens the query "
                                      "builder and allows you to create a subset of features to display on "
                                      "the map canvas rather than displaying all features in the layer" ) );
   txtSubsetSQL->setWhatsThis( tr( "The query used to limit the features in the "
@@ -431,22 +455,15 @@ void QgsVectorLayerProperties::reset( void )
                                   "layers. To enter or modify the query, click on the Query Builder button" ) );
 
   //see if we are dealing with a pg layer here
-  if ( layer->providerType() == "postgres" )
-  {
-    grpSubset->setEnabled( true );
-    txtSubsetSQL->setText( layer->subsetString() );
-    // if the user is allowed to type an adhoc query, the app will crash if the query
-    // is bad. For this reason, the sql box is disabled and the query must be built
-    // using the query builder, either by typing it in by hand or using the buttons, etc
-    // on the builder. If the ability to enter a query directly into the box is required,
-    // a mechanism to check it must be implemented.
-    txtSubsetSQL->setEnabled( false );
-    pbnQueryBuilder->setEnabled( true );
-  }
-  else
-  {
-    grpSubset->setEnabled( false );
-  }
+  grpSubset->setEnabled( true );
+  txtSubsetSQL->setText( layer->subsetString() );
+  // if the user is allowed to type an adhoc query, the app will crash if the query
+  // is bad. For this reason, the sql box is disabled and the query must be built
+  // using the query builder, either by typing it in by hand or using the buttons, etc
+  // on the builder. If the ability to enter a query directly into the box is required,
+  // a mechanism to check it must be implemented.
+  txtSubsetSQL->setEnabled( false );
+  pbnQueryBuilder->setEnabled( layer && layer->dataProvider() && layer->dataProvider()->supportsSubsetString() );
 
   //get field list for display field combo
   const QgsFieldMap& myFields = layer->pendingFields();
@@ -474,40 +491,8 @@ void QgsVectorLayerProperties::reset( void )
     }
   }
 
-  //find out the type of renderer in the vectorlayer, create a dialog with these settings and add it to the form
-  delete mRendererDialog;
-  mRendererDialog = 0;
-  QString rtype = layer->renderer()->name();
-  if ( rtype == "Single Symbol" )
-  {
-    mRendererDialog = new QgsSingleSymbolDialog( layer );
-    legendtypecombobox->setCurrentIndex( 0 );
-  }
-  else if ( rtype == "Graduated Symbol" )
-  {
-    mRendererDialog = new QgsGraduatedSymbolDialog( layer );
-    legendtypecombobox->setCurrentIndex( 1 );
-  }
-  else if ( rtype == "Continuous Color" )
-  {
-    mRendererDialog = new QgsContinuousColorDialog( layer );
-    legendtypecombobox->setCurrentIndex( 2 );
-  }
-  else if ( rtype == "Unique Value" )
-  {
-    mRendererDialog = new QgsUniqueValueDialog( layer );
-    legendtypecombobox->setCurrentIndex( 3 );
-  }
-
-  if ( mRendererDialog )
-  {
-    widgetStackRenderers->addWidget( mRendererDialog );
-    widgetStackRenderers->setCurrentWidget( mRendererDialog );
-  }
-
-
-  QObject::connect( legendtypecombobox, SIGNAL( activated( const QString & ) ), this,
-                    SLOT( alterLayerDialog( const QString & ) ) );
+  // load appropriate symbology page (V1 or V2)
+  updateSymbologyPage();
 
   // reset fields in label dialog
   layer->label()->setFields( layer->pendingFields() );
@@ -535,11 +520,6 @@ void QgsVectorLayerProperties::reset( void )
 // methods reimplemented from qt designer base class
 //
 
-void QgsVectorLayerProperties::on_buttonBox_helpRequested()
-{
-  QgsContextHelp::run( context_id );
-}
-
 QMap< QgsVectorLayer::EditType, QString > QgsVectorLayerProperties::editTypeMap;
 
 void QgsVectorLayerProperties::setupEditTypes()
@@ -557,6 +537,9 @@ void QgsVectorLayerProperties::setupEditTypes()
   editTypeMap.insert( QgsVectorLayer::FileName, tr( "File name" ) );
   editTypeMap.insert( QgsVectorLayer::Enumeration, tr( "Enumeration" ) );
   editTypeMap.insert( QgsVectorLayer::Immutable, tr( "Immutable" ) );
+  editTypeMap.insert( QgsVectorLayer::Hidden, tr( "Hidden" ) );
+  editTypeMap.insert( QgsVectorLayer::CheckBox, tr( "Checkbox" ) );
+  editTypeMap.insert( QgsVectorLayer::TextEdit, tr( "Text edit" ) );
 }
 
 QString QgsVectorLayerProperties::editTypeButtonText( QgsVectorLayer::EditType type )
@@ -574,22 +557,17 @@ void QgsVectorLayerProperties::apply()
   //
   // Set up sql subset query if applicable
   //
-#ifdef HAVE_POSTGRESQL
-  //see if we are dealing with a pg layer here
-  if ( layer->providerType() == "postgres" )
-  {
-    grpSubset->setEnabled( true );
-    // set the subset sql for the layer
-    layer->setSubsetString( txtSubsetSQL->toPlainText() );
-    // update the metadata with the updated sql subset
-    QString myStyle = QgsApplication::reportStyleSheet();
-    teMetadata->clear();
-    teMetadata->document()->setDefaultStyleSheet( myStyle );
-    teMetadata->setHtml( metadata() );
-    // update the extents of the layer (fetched from the provider)
-    layer->updateExtents();
-  }
-#endif
+  grpSubset->setEnabled( true );
+  // set the subset sql for the layer
+  layer->setSubsetString( txtSubsetSQL->toPlainText() );
+  // update the metadata with the updated sql subset
+  QString myStyle = QgsApplication::reportStyleSheet();
+  teMetadata->clear();
+  teMetadata->document()->setDefaultStyleSheet( myStyle );
+  teMetadata->setHtml( metadata() );
+  // update the extents of the layer (fetched from the provider)
+  layer->updateExtents();
+
   // set up the scale based layer visibility stuff....
   layer->toggleScaleBasedVisibility( chkUseScaleDependentRendering->isChecked() );
   layer->setMinimumScale( spinMinimumScale->value() );
@@ -598,6 +576,9 @@ void QgsVectorLayerProperties::apply()
   // update the display field
   layer->setDisplayField( displayFieldComboBox->currentText() );
 
+  layer->setEditForm( leEditForm->text() );
+  layer->setEditFormInit( leEditFormInit->text() );
+
   actionDialog->apply();
 
   labelDialog->apply();
@@ -606,9 +587,9 @@ void QgsVectorLayerProperties::apply()
 
   for ( int i = 0; i < tblAttributes->rowCount(); i++ )
   {
-    int idx = tblAttributes->item( i, 0 )->text().toInt();
+    int idx = tblAttributes->item( i, attrIdCol )->text().toInt();
 
-    QPushButton *pb = dynamic_cast<QPushButton*>( tblAttributes->cellWidget( i, 6 ) );
+    QPushButton *pb = qobject_cast<QPushButton *>( tblAttributes->cellWidget( i, attrEditTypeCol ) );
     if ( !pb )
       continue;
 
@@ -632,34 +613,52 @@ void QgsVectorLayerProperties::apply()
         layer->range( idx ) = mRanges[idx];
       }
     }
+    else if ( editType == QgsVectorLayer::CheckBox )
+    {
+      if ( mCheckedStates.contains( idx ) )
+      {
+        layer->setCheckedState( idx, mCheckedStates[idx].first, mCheckedStates[idx].second );
+      }
+    }
   }
 
-  QgsSingleSymbolDialog *sdialog =
-    dynamic_cast < QgsSingleSymbolDialog * >( widgetStackRenderers->currentWidget() );
-  QgsGraduatedSymbolDialog *gdialog =
-    dynamic_cast < QgsGraduatedSymbolDialog * >( widgetStackRenderers->currentWidget() );
-  QgsContinuousColorDialog *cdialog =
-    dynamic_cast < QgsContinuousColorDialog * >( widgetStackRenderers->currentWidget() );
-  QgsUniqueValueDialog* udialog =
-    dynamic_cast< QgsUniqueValueDialog * >( widgetStackRenderers->currentWidget() );
+  if ( layer->isUsingRendererV2() )
+  {
+    QgsRendererV2PropertiesDialog* dlg =
+      static_cast<QgsRendererV2PropertiesDialog*>( widgetStackRenderers->currentWidget() );
+    dlg->apply();
+  }
+  else
+  {
+
+    QgsSingleSymbolDialog *sdialog =
+      qobject_cast < QgsSingleSymbolDialog * >( widgetStackRenderers->currentWidget() );
+    QgsGraduatedSymbolDialog *gdialog =
+      qobject_cast < QgsGraduatedSymbolDialog * >( widgetStackRenderers->currentWidget() );
+    QgsContinuousColorDialog *cdialog =
+      qobject_cast < QgsContinuousColorDialog * >( widgetStackRenderers->currentWidget() );
+    QgsUniqueValueDialog* udialog =
+      qobject_cast< QgsUniqueValueDialog * >( widgetStackRenderers->currentWidget() );
+
+    if ( sdialog )
+    {
+      sdialog->apply();
+    }
+    else if ( gdialog )
+    {
+      gdialog->apply();
+    }
+    else if ( cdialog )
+    {
+      cdialog->apply();
+    }
+    else if ( udialog )
+    {
+      udialog->apply();
+    }
+    layer->setTransparency( static_cast < unsigned int >( 255 - sliderTransparency->value() ) );
 
-  if ( sdialog )
-  {
-    sdialog->apply();
   }
-  else if ( gdialog )
-  {
-    gdialog->apply();
-  }
-  else if ( cdialog )
-  {
-    cdialog->apply();
-  }
-  else if ( udialog )
-  {
-    udialog->apply();
-  }
-  layer->setTransparency( static_cast < unsigned int >( 255 - sliderTransparency->value() ) );
 
   //apply overlay dialogs
   for ( QList<QgsApplyDialog*>::iterator it = mOverlayDialogs.begin(); it != mOverlayDialogs.end(); ++it )
@@ -670,6 +669,9 @@ void QgsVectorLayerProperties::apply()
   // update symbology
   emit refreshLegend( layer->getLayerID(), false );
 
+  //no need to delete the old one, maplayer will do it if needed
+  layer->setCacheImage( 0 );
+
   layer->triggerRepaint();
   // notify the project we've made a change
   QgsProject::instance()->dirty( true );
@@ -678,25 +680,19 @@ void QgsVectorLayerProperties::apply()
 
 void QgsVectorLayerProperties::on_pbnQueryBuilder_clicked()
 {
-#ifdef HAVE_POSTGRESQL
   // launch the query builder using the PostgreSQL connection
   // from the provider
 
-  // cast to postgres provider type
-  QgsPostgresProvider * myPGProvider = ( QgsPostgresProvider * ) layer->dataProvider() ; // FIXME use dynamic cast
-  // create the query builder object using the table name
-  // and postgres connection from the provider
-  QgsDataSourceURI uri( myPGProvider->dataSourceUri() );
-  QgsPgQueryBuilder *pqb = new QgsPgQueryBuilder( &uri, this );
+  QgsQueryBuilder *qb = new QgsQueryBuilder( layer, this );
 
   // Set the sql in the query builder to the same in the prop dialog
   // (in case the user has already changed it)
-  pqb->setSql( txtSubsetSQL->toPlainText() );
+  qb->setSql( txtSubsetSQL->toPlainText() );
   // Open the query builder
-  if ( pqb->exec() )
+  if ( qb->exec() )
   {
     // if the sql is changed, update it in the prop subset text box
-    txtSubsetSQL->setText( pqb->sql() );
+    txtSubsetSQL->setText( qb->sql() );
     //TODO If the sql is changed in the prop dialog, the layer extent should be recalculated
 
     // The datasource for the layer needs to be updated with the new sql since this gets
@@ -704,8 +700,7 @@ void QgsVectorLayerProperties::on_pbnQueryBuilder_clicked()
 
   }
   // delete the query builder object
-  delete pqb;
-#endif
+  delete qb;
 }
 
 void QgsVectorLayerProperties::on_pbnIndex_clicked()
@@ -973,150 +968,114 @@ void QgsVectorLayerProperties::on_pbnSaveDefaultStyle_clicked()
 
 void QgsVectorLayerProperties::on_pbnLoadStyle_clicked()
 {
-  QSettings myQSettings;  // where we keep last used filter in persistant state
+  QSettings myQSettings;  // where we keep last used filter in persistent state
   QString myLastUsedDir = myQSettings.value( "style/lastStyleDir", "." ).toString();
-
-  //create a file dialog
-  std::auto_ptr < QFileDialog > myFileDialog
-  (
-    new QFileDialog(
-      this,
-      tr( "Load layer properties from style file (.qml)" ),
-      myLastUsedDir,
-      tr( "QGIS Layer Style File (*.qml)" )
-    )
-  );
-  myFileDialog->setFileMode( QFileDialog::AnyFile );
-  myFileDialog->setAcceptMode( QFileDialog::AcceptOpen );
-
-  //prompt the user for a file name
-  QString myFileName;
-  if ( myFileDialog->exec() == QDialog::Accepted )
+  QString myFileName = QFileDialog::getOpenFileName( this, tr( "Load layer properties from style file (.qml)" ), myLastUsedDir, tr( "QGIS Layer Style File (*.qml)" ) );
+  if ( myFileName.isNull() )
   {
-    QStringList myFiles = myFileDialog->selectedFiles();
-    if ( !myFiles.isEmpty() )
-    {
-      myFileName = myFiles[0];
-    }
+    return;
   }
 
-  if ( !myFileName.isEmpty() )
+  bool defaultLoadedFlag = false;
+  QString myMessage = layer->loadNamedStyle( myFileName, defaultLoadedFlag );
+  //reset if the default style was loaded ok only
+  if ( defaultLoadedFlag )
   {
-    if ( myFileDialog->selectedFilter() == tr( "QGIS Layer Style File (*.qml)" ) )
-    {
-      //ensure the user never omitted the extension from the file name
-      if ( !myFileName.endsWith( ".qml", Qt::CaseInsensitive ) )
-      {
-        myFileName += ".qml";
-      }
-      bool defaultLoadedFlag = false;
-      QString myMessage = layer->loadNamedStyle( myFileName, defaultLoadedFlag );
-      //reset if the default style was loaded ok only
-      if ( defaultLoadedFlag )
-      {
-        reset();
-      }
-      else
-      {
-        //let the user know what went wrong
-        QMessageBox::information( this, tr( "Saved Style" ), myMessage );
-      }
-    }
-    else
-    {
-      QMessageBox::warning( this, tr( "QGIS" ),
-                            tr( "Unknown style format: %1" ).arg( myFileDialog->selectedFilter() ) );
-    }
-    myQSettings.setValue( "style/lastStyleDir", myFileDialog->directory().absolutePath() );
+    reset();
   }
+  else
+  {
+    //let the user know what went wrong
+    QMessageBox::information( this, tr( "Saved Style" ), myMessage );
+  }
+
+  QFileInfo myFI( myFileName );
+  QString myPath = myFI.path();
+  myQSettings.setValue( "style/lastStyleDir", myPath );
 }
 
 
 void QgsVectorLayerProperties::on_pbnSaveStyleAs_clicked()
 {
-  QSettings myQSettings;  // where we keep last used filter in persistant state
+  QSettings myQSettings;  // where we keep last used filter in persistent state
   QString myLastUsedDir = myQSettings.value( "style/lastStyleDir", "." ).toString();
-
-  //create a file dialog
-  std::auto_ptr < QFileDialog > myFileDialog
-  (
-    new QFileDialog(
-      this,
-      tr( "Save layer properties as style file (.qml)" ),
-      myLastUsedDir,
-      tr( "QGIS Layer Style File (*.qml)" )
-    )
-  );
-  myFileDialog->setFileMode( QFileDialog::AnyFile );
-  myFileDialog->setAcceptMode( QFileDialog::AcceptSave );
-
-  //prompt the user for a file name
-  QString myOutputFileName;
-  if ( myFileDialog->exec() == QDialog::Accepted )
+  QString myOutputFileName = QFileDialog::getSaveFileName( this, tr( "Save layer properties as style file (.qml)" ), myLastUsedDir, tr( "QGIS Layer Style File (*.qml)" ) );
+  if ( myOutputFileName.isNull() ) //dialog canceled
   {
-    QStringList myFiles = myFileDialog->selectedFiles();
-    if ( !myFiles.isEmpty() )
-    {
-      myOutputFileName = myFiles[0];
-    }
+    return;
   }
 
-  if ( !myOutputFileName.isEmpty() )
+  apply(); // make sure the qml to save is uptodate
+
+  //ensure the user never omitted the extension from the file name
+  if ( !myOutputFileName.endsWith( ".qml", Qt::CaseInsensitive ) )
   {
-    if ( myFileDialog->selectedFilter() == tr( "QGIS Layer Style File (*.qml)" ) )
-    {
-      apply(); // make sure the qml to save is uptodate
-
-      //ensure the user never ommitted the extension from the file name
-      if ( !myOutputFileName.endsWith( ".qml", Qt::CaseInsensitive ) )
-      {
-        myOutputFileName += ".qml";
-      }
-
-      bool defaultLoadedFlag = false;
-      QString myMessage = layer->saveNamedStyle( myOutputFileName, defaultLoadedFlag );
-      //reset if the default style was loaded ok only
-      if ( defaultLoadedFlag )
-      {
-        reset();
-      }
-      else
-      {
-        //let the user know what went wrong
-        QMessageBox::information( this, tr( "Saved Style" ), myMessage );
-      }
-    }
-    else
-    {
-      QMessageBox::warning( this, tr( "QGIS" ),
-                            tr( "Unknown style format: %1" ).arg( myFileDialog->selectedFilter() ) );
-    }
-
-    myQSettings.setValue( "style/lastStyleDir", myFileDialog->directory().absolutePath() );
+    myOutputFileName += ".qml";
   }
+
+  bool defaultLoadedFlag = false;
+  QString myMessage = layer->saveNamedStyle( myOutputFileName, defaultLoadedFlag );
+  //reset if the default style was loaded ok only
+  if ( defaultLoadedFlag )
+  {
+    reset();
+  }
+  else
+  {
+    //let the user know what went wrong
+    QMessageBox::information( this, tr( "Saved Style" ), myMessage );
+  }
+
+  QFileInfo myFI( myOutputFileName );
+  QString myPath = myFI.path();
+  // Persist last used dir
+  myQSettings.setValue( "style/lastStyleDir", myPath );
 }
 
 void QgsVectorLayerProperties::on_tblAttributes_cellChanged( int row, int column )
 {
-  if ( column == 8 && layer ) //only consider attribute aliases in this function
+  if ( column == attrAliasCol && layer ) //only consider attribute aliases in this function
   {
+    int idx = tblAttributes->item( row, attrIdCol )->text().toInt();
+
     const QgsFieldMap &fields = layer->pendingFields();
-    if ( row >= fields.size() )
+
+    if ( !fields.contains( idx ) )
     {
-      return; //index must be wrong
+      return; // index must be wrong
     }
 
-    QgsFieldMap::const_iterator f_it = fields.constBegin();
-    f_it += row;
-    int index = f_it.key();
-    QTableWidgetItem* aliasItem = tblAttributes->item( row, column );
+    QTableWidgetItem *aliasItem = tblAttributes->item( row, column );
     if ( aliasItem )
     {
-      layer->addAttributeAlias( index, aliasItem->text() );
+      layer->addAttributeAlias( idx, aliasItem->text() );
     }
   }
 }
 
+void QgsVectorLayerProperties::on_mCalculateFieldButton_clicked()
+{
+  if ( !layer )
+  {
+    return;
+  }
+
+  QgsFieldCalculator calc( layer );
+  calc.exec();
+}
+
+void QgsVectorLayerProperties::on_pbnSelectEditForm_clicked()
+{
+  QSettings myQSettings;
+  QString lastUsedDir = myQSettings.value( "style/lastUIDir", "." ).toString();
+  QString uifilename = QFileDialog::getOpenFileName( this, tr( "Select edit form" ), lastUsedDir, tr( "UI file (*.ui)" ) );
+
+  if ( uifilename.isNull() )
+    return;
+
+  leEditForm->setText( uifilename );
+}
+
 QList<QgsVectorOverlayPlugin*> QgsVectorLayerProperties::overlayPlugins() const
 {
   QList<QgsVectorOverlayPlugin*> pluginList;
@@ -1132,7 +1091,7 @@ QList<QgsVectorOverlayPlugin*> QgsVectorLayerProperties::overlayPlugins() const
       thePlugin = ( *it )->plugin();
       if ( thePlugin && thePlugin->type() == QgisPlugin::VECTOR_OVERLAY )
       {
-        theOverlayPlugin = dynamic_cast<QgsVectorOverlayPlugin*>( thePlugin );
+        theOverlayPlugin = dynamic_cast<QgsVectorOverlayPlugin *>( thePlugin );
         if ( theOverlayPlugin )
         {
           pluginList.push_back( theOverlayPlugin );
@@ -1143,3 +1102,92 @@ QList<QgsVectorOverlayPlugin*> QgsVectorLayerProperties::overlayPlugins() const
 
   return pluginList;
 }
+
+void QgsVectorLayerProperties::setUsingNewSymbology( bool useNewSymbology )
+{
+  if ( useNewSymbology )
+  {
+    QgsSymbologyV2Conversion::rendererV1toV2( layer );
+  }
+  else
+  {
+    QgsSymbologyV2Conversion::rendererV2toV1( layer );
+  }
+
+  // update GUI!
+  updateSymbologyPage();
+}
+
+void QgsVectorLayerProperties::useNewSymbology()
+{
+  int res = QMessageBox::question( this, tr( "Symbology" ),
+                                   tr( "Do you wish to use the new symbology implementation for this layer?" ),
+                                   QMessageBox::Yes | QMessageBox::No );
+
+  if ( res != QMessageBox::Yes )
+    return;
+
+  setUsingNewSymbology( true );
+}
+
+void QgsVectorLayerProperties::updateSymbologyPage()
+{
+
+  //find out the type of renderer in the vectorlayer, create a dialog with these settings and add it to the form
+  delete mRendererDialog;
+  mRendererDialog = 0;
+
+  bool v2 = layer->isUsingRendererV2();
+
+  // hide unused widgets
+  legendtypecombobox->setVisible( !v2 );
+  legendtypelabel->setVisible( !v2 );
+  lblTransparencyPercent->setVisible( !v2 );
+  sliderTransparency->setVisible( !v2 );
+  btnUseNewSymbology->setVisible( !v2 );
+
+  if ( v2 )
+  {
+    mRendererDialog = new QgsRendererV2PropertiesDialog( layer, QgsStyleV2::defaultStyle(), true );
+
+    connect( mRendererDialog, SIGNAL( useNewSymbology( bool ) ), this, SLOT( setUsingNewSymbology( bool ) ) );
+
+  }
+  else
+  {
+
+    QString rtype = layer->renderer()->name();
+    if ( rtype == "Single Symbol" )
+    {
+      mRendererDialog = new QgsSingleSymbolDialog( layer );
+      legendtypecombobox->setCurrentIndex( 0 );
+    }
+    else if ( rtype == "Graduated Symbol" )
+    {
+      mRendererDialog = new QgsGraduatedSymbolDialog( layer );
+      legendtypecombobox->setCurrentIndex( 1 );
+    }
+    else if ( rtype == "Continuous Color" )
+    {
+      mRendererDialog = new QgsContinuousColorDialog( layer );
+      legendtypecombobox->setCurrentIndex( 2 );
+    }
+    else if ( rtype == "Unique Value" )
+    {
+      mRendererDialog = new QgsUniqueValueDialog( layer );
+      legendtypecombobox->setCurrentIndex( 3 );
+    }
+
+    QObject::connect( legendtypecombobox, SIGNAL( activated( const QString & ) ), this,
+                      SLOT( alterLayerDialog( const QString & ) ) );
+
+  }
+
+  if ( mRendererDialog )
+  {
+    widgetStackRenderers->addWidget( mRendererDialog );
+    widgetStackRenderers->setCurrentWidget( mRendererDialog );
+  }
+
+
+}
