@@ -12,25 +12,31 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-/* $Id: qgsapplication.cpp 11628 2009-09-11 22:53:47Z timlinux $ */
+/* $Id$ */
 
 #include "qgsapplication.h"
 #include "qgsmaplayerregistry.h"
 #include "qgsproviderregistry.h"
+#include "qgsexception.h"
 
 #include <QDir>
 #include <QMessageBox>
 #include <QPalette>
+#include <QSettings>
+
+// (if Windows/Mac, use icon from resource)
+#if ! defined(Q_WS_WIN) && ! defined(Q_WS_MAC)
+#include "../../images/themes/default/qgis.xpm" // Linux
+#include <QIcon>
+#endif
+#ifndef Q_WS_WIN
+#include <netinet/in.h>
+#else
+#include <winsock.h>
+#endif
 
 #include "qgsconfig.h"
 
-// for htonl
-#ifdef WIN32
-#include <winsock.h>
-#else
-#include <netinet/in.h>
-#endif
-
 #include <ogr_api.h>
 
 QString QgsApplication::mPrefixPath;
@@ -62,6 +68,11 @@ QgsApplication::QgsApplication( int & argc, char ** argv, bool GUIenabled )
   QString myPrefix = myDir.absolutePath();
   setPrefixPath( myPrefix, true );
 #endif
+
+  // set application's icon (if Windows/Mac, use icon from resource)
+#if ! defined(Q_WS_WIN) && ! defined(Q_WS_MAC)
+  setWindowIcon( QPixmap( qgis_xpm ) );        // Linux
+#endif
 }
 
 QgsApplication::~QgsApplication()
@@ -77,10 +88,18 @@ bool QgsApplication::notify( QObject * receiver, QEvent * event )
   {
     done = QApplication::notify( receiver, event );
   }
+  catch ( QgsException & e )
+  {
+    QMessageBox::critical( activeWindow(), tr( "Exception" ), e.what() );
+  }
   catch ( std::exception & e )
   {
     QMessageBox::critical( activeWindow(), tr( "Exception" ), e.what() );
   }
+  catch ( ... )
+  {
+    QMessageBox::critical( activeWindow(), tr( "Exception" ), "unknown exception" );
+  }
   return done;
 }
 
@@ -124,11 +143,11 @@ const QString QgsApplication::pkgDataPath()
 }
 const QString QgsApplication::defaultThemePath()
 {
-  return mPkgDataPath + "/themes/default/";
+  return ":/images/themes/default/";
 }
 const QString QgsApplication::activeThemePath()
 {
-  return mPkgDataPath + "/themes/" + mThemeName + "/";
+  return ":/images/themes/" + mThemeName + "/";
 }
 
 /*!
@@ -136,7 +155,7 @@ const QString QgsApplication::activeThemePath()
 */
 void QgsApplication::setThemeName( const QString theThemeName )
 {
-  QString myPath = mPkgDataPath + "/themes/" + theThemeName + "/";
+  QString myPath = ":/images/themes/" + theThemeName + "/";
   //check it exists and if not roll back to default theme
   if ( QFile::exists( myPath ) )
   {
@@ -206,9 +225,6 @@ const QString QgsApplication::developerPath()
 const QString QgsApplication::helpAppPath()
 {
   QString helpAppPath = applicationDirPath();
-#ifdef Q_OS_MACX
-  helpAppPath += "/bin/qgis_help.app/Contents/MacOS";
-#endif
   helpAppPath += "/qgis_help";
   return helpAppPath;
 }
@@ -281,13 +297,45 @@ const QString QgsApplication::srsDbFilePath()
 }
 
 /*!
-  Returns the path to the svg directory.
+  Returns the paths to the svg directories.
+*/
+const QStringList QgsApplication::svgPaths()
+{
+  //local directories to search when looking for an SVG with a given basename
+  //defined by user in options dialog
+  QSettings settings;
+  QStringList myPathList;
+  QString myPaths = settings.value( "svg/searchPathsForSVG", "" ).toString();
+  if ( !myPaths.isEmpty() )
+  {
+    myPathList = myPaths.split( "|" );
+  }
+  //additional default paths
+  myPathList
+  << mPkgDataPath + QString( "/svg/" )
+  << qgisSettingsDirPath() + QString( "svg/" );
+  return myPathList;
+
+}
+
+/*!
+  Returns the path to the applications svg directories.
 */
 const QString QgsApplication::svgPath()
 {
   return mPkgDataPath + QString( "/svg/" );
 }
 
+const QString QgsApplication::userStyleV2Path()
+{
+  return qgisSettingsDirPath() + QString( "symbology-ng-style.xml" );
+}
+
+const QString QgsApplication::defaultStyleV2Path()
+{
+  return mPkgDataPath + QString( "/resources/symbology-ng-style.xml" );
+}
+
 QgsApplication::endian_t QgsApplication::endian()
 {
   return ( htonl( 1 ) == 1 ) ? XDR : NDR ;
@@ -317,13 +365,15 @@ QString QgsApplication::showSettings()
                              "Active Theme Name   : %4\n"
                              "Active Theme Path   : %5\n"
                              "Default Theme Path  : %6\n"
-                             "User DB Path        : %7\n" )
+                             "SVG Search Paths    : %7\n"
+                             "User DB Path        : %8\n" )
                     .arg( mPrefixPath )
                     .arg( mPluginPath )
                     .arg( mPkgDataPath )
                     .arg( themeName() )
                     .arg( activeThemePath() )
                     .arg( defaultThemePath() )
+                    .arg( svgPaths().join( "\n" ) )
                     .arg( qgisMasterDbFilePath() );
   return myState;
 }
@@ -332,7 +382,7 @@ QString QgsApplication::reportStyleSheet()
 {
   //
   // Make the style sheet desktop preferences aware by using qappliation
-  // palette as a basis for colours where appropriate
+  // palette as a basis for colors where appropriate
   //
   QColor myColor1 = palette().highlight().color();
   QColor myColor2 = myColor1;
@@ -350,9 +400,53 @@ QString QgsApplication::reportStyleSheet()
             "padding-bottom: 8px;"
             "border: 1px solid #6c6c6c;"
             "}"
-            "h1 {font-size : 22pt; }"
-            "h2 {font-size : 18pt; }"
-            "h3 {font-size : 14pt; }";
+            ".overview{ font: 1.82em; font-weight: bold;}"
+            "body{  background: white;"
+            "  color: black;"
+            "  font-family: arial,sans-serif;"
+            "}"
+            "h2{  background-color: #F6F6F6;"
+            "  color: #8FB171; "
+            "  font-size: medium;  "
+            "  font-weight: normal;"
+            "  font-family: luxi serif, georgia, times new roman, times, serif;"
+            "  background: none;"
+            "  padding: 0.75em 0 0;"
+            "  margin: 0;"
+            "  line-height: 1.1em;"
+            "}"
+            "h3{  background-color: #F6F6F6;"
+            "  color: #729FCF;"
+            "  font-family: luxi serif, georgia, times new roman, times, serif;"
+            "  font-weight: bold;"
+            "  font-size: large;"
+            "  text-align: right;"
+            "  border-bottom: 5px solid #DCEB5C;"
+            "}"
+            "h4{  background-color: #F6F6F6;"
+            "  color: #729FCF;"
+            "  font-family: luxi serif, georgia, times new roman, times, serif;"
+            "  font-weight: bold;"
+            "  font-size: medium;"
+            "  text-align: right;"
+            "}"
+            "h5{    background-color: #F6F6F6;"
+            "   color: #729FCF;"
+            "   font-family: luxi serif, georgia, times new roman, times, serif;"
+            "   font-weight: bold;"
+            "   font-size: small;"
+            "   text-align: right;"
+            "}"
+            "a{  color: #729FCF;"
+            "  font-family: arial,sans-serif;"
+            "  font-size: small;"
+            "}"
+            "label{  background-color: #FFFFCC;"
+            "  border: 1px solid black;"
+            "  margin: 1px;"
+            "  padding: 0px 3px; "
+            "  font-size: small;"
+            "}";
   return myStyle;
 }
 
