@@ -14,7 +14,7 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-/* $Id: qgscoordinatetransform.cpp 11404 2009-08-17 08:48:20Z homann $ */
+/* $Id$ */
 #include <cassert>
 #include "qgscoordinatetransform.h"
 #include "qgslogger.h"
@@ -483,7 +483,7 @@ void QgsCoordinateTransform::transformCoords( const int& numPoints, double *x, d
       }
     }
 
-    pjErr << tr( "with error: " ) << pj_strerrno( projResult ) << '\n';
+    pjErr << tr( "with error: " ) << QString::fromUtf8( pj_strerrno( projResult ) ) << '\n';
 
     QgsDebugMsg( "Projection failed emitting invalid transform signal: " + QString( msg.toLocal8Bit().data() ) );
 
@@ -565,7 +565,7 @@ void QgsCoordinateTransform::setFinder()
   // (version 0.7) and the old installation was deleted
 
   // Another problem: PROJ checks if pj_finder was set before
-  // PROJ_LIB enviroment variable. pj_finder is probably set in
+  // PROJ_LIB environment variable. pj_finder is probably set in
   // GRASS gproj library when plugin is loaded, consequently
   // PROJ_LIB is ignored
 
