@@ -45,7 +45,7 @@
 #endif
 
 static const char * const ident_ =
-  "$Id: qgslabel.cpp 11152 2009-07-23 10:59:59Z jef $";
+  "$Id$";
 
 QgsLabel::QgsLabel( const QgsFieldMap & fields )
     : mMinScale( 0 ),
@@ -508,6 +508,11 @@ QString QgsLabel::labelField( int attr ) const
   return mField[fieldIndex].name();
 }
 
+QgsLabelAttributes *QgsLabel::labelAttributes( void )
+{
+  return mLabelAttributes;
+}
+// @note this will be deprecated use attributes rather
 QgsLabelAttributes *QgsLabel::layerAttributes( void )
 {
   return mLabelAttributes;
@@ -580,7 +585,7 @@ unsigned char* QgsLabel::labelPoint( labelpoint& point, unsigned char *geom, siz
 #endif
   Q_ASSERT( geom + 1 + sizeof( wkbType ) <= geomend );
 
-  geom++; // skip endianess
+  geom++; // skip endianness
   memcpy( &wkbType, geom, sizeof( wkbType ) );
   geom += sizeof( wkbType );
 
