@@ -14,7 +14,7 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-/* $Id: qgsmaplayer.cpp 11571 2009-09-05 19:49:10Z jef $ */
+/* $Id$ */
 
 
 #include <QDateTime>
@@ -40,6 +40,7 @@
 #include "qgsproject.h"
 #include "qgslogger.h"
 #include "qgsdatasourceuri.h"
+#include "qgsvectorlayer.h"
 
 QgsMapLayer::QgsMapLayer( QgsMapLayer::LayerType type,
                           QString lyrname,
@@ -74,6 +75,7 @@ QgsMapLayer::QgsMapLayer( QgsMapLayer::LayerType type,
   mMinScale = 0;
   mMaxScale = 100000000;
   mScaleBasedVisibility = false;
+  mpCacheImage = 0;
 }
 
 
@@ -81,6 +83,10 @@ QgsMapLayer::QgsMapLayer( QgsMapLayer::LayerType type,
 QgsMapLayer::~QgsMapLayer()
 {
   delete mCRS;
+  if ( mpCacheImage )
+  {
+    delete mpCacheImage;
+  }
 }
 
 QgsMapLayer::LayerType QgsMapLayer::type() const
@@ -105,7 +111,7 @@ void QgsMapLayer::setLayerName( const QString & _newVal )
 /** Read property of QString layerName. */
 QString const & QgsMapLayer::name() const
 {
-  QgsDebugMsg( "returning name '" + mLayerName + "'" );
+  QgsDebugMsgLevel( "returning name '" + mLayerName + "'", 3 );
   return mLayerName;
 }
 
@@ -145,14 +151,36 @@ bool QgsMapLayer::readXML( QDomNode & layer_node )
 
   QDomElement element = layer_node.toElement();
 
-  // XXX not needed? QString type = element.attribute("type");
+  QDomNode mnl;
+  QDomElement mne;
+
+  // read provider
+  QString provider;
+  mnl = layer_node.namedItem( "provider" );
+  mne = mnl.toElement();
+  provider = mne.text();
 
   // set data source
-  QDomNode mnl = layer_node.namedItem( "datasource" );
-  QDomElement mne = mnl.toElement();
+  mnl = layer_node.namedItem( "datasource" );
+  mne = mnl.toElement();
   mDataSource = mne.text();
 
-  mDataSource = QgsProject::instance()->readPath( mDataSource );
+  if ( provider == "spatialite" )
+  {
+    QgsDataSourceURI uri( mDataSource );
+    uri.setDatabase( QgsProject::instance()->readPath( uri.database() ) );
+    mDataSource = uri.uri();
+  }
+  else if ( provider == "ogr" )
+  {
+    QStringList theURIParts = mDataSource.split( "|" );
+    theURIParts[0] = QgsProject::instance()->readPath( theURIParts[0] );
+    mDataSource = theURIParts.join( "|" );
+  }
+  else
+  {
+    mDataSource = QgsProject::instance()->readPath( mDataSource );
+  }
 
   // Set the CRS from project file, asking the user if necessary.
   // Make it the saved CRS to have WMS layer projected correctly.
@@ -199,15 +227,7 @@ bool QgsMapLayer::readXML( QDomNode & layer_node )
   }
 
   // use scale dependent visibility flag
-  QString hasScaleBasedVisibility = element.attribute( "hasScaleBasedVisibilityFlag" );
-  if ( "1" == hasScaleBasedVisibility )
-  {
-    toggleScaleBasedVisibility( true );
-  }
-  else
-  {
-    toggleScaleBasedVisibility( false );
-  }
+  toggleScaleBasedVisibility( element.attribute( "hasScaleBasedVisibilityFlag" ).toInt() == 1 );
   setMinimumScale( element.attribute( "minimumScale" ).toFloat() );
   setMaximumScale( element.attribute( "maximumScale" ).toFloat() );
 
@@ -226,6 +246,8 @@ bool QgsMapLayer::readXML( QDomNode & layer_node )
     setTransparency( myElement.text().toInt() );
   }
 
+  readCustomProperties( layer_node );
+
   return true;
 } // void QgsMapLayer::readXML
 
@@ -245,14 +267,7 @@ bool QgsMapLayer::writeXML( QDomNode & layer_node, QDomDocument & document )
   QDomElement maplayer = document.createElement( "maplayer" );
 
   // use scale dependent visibility flag
-  if ( hasScaleBasedVisibility() )
-  {
-    maplayer.setAttribute( "hasScaleBasedVisibilityFlag", 1 );
-  }
-  else
-  {
-    maplayer.setAttribute( "hasScaleBasedVisibilityFlag", 0 );
-  }
+  maplayer.setAttribute( "hasScaleBasedVisibilityFlag", hasScaleBasedVisibility() ? 1 : 0 );
   maplayer.setAttribute( "minimumScale", minimumScale() );
   maplayer.setAttribute( "maximumScale", maximumScale() );
 
@@ -268,7 +283,24 @@ bool QgsMapLayer::writeXML( QDomNode & layer_node, QDomDocument & document )
 
   QString src = source();
 
-  src = QgsProject::instance()->writePath( src );
+  QgsVectorLayer *vlayer = qobject_cast<QgsVectorLayer *>( this );
+  if ( vlayer && vlayer->providerType() == "spatialite" )
+  {
+    QgsDataSourceURI uri( src );
+    QString database = QgsProject::instance()->writePath( uri.database() );
+    uri.setConnection( uri.host(), uri.port(), database, uri.username(), uri.password() );
+    src = uri.uri();
+  }
+  else if ( vlayer && vlayer->providerType() == "ogr" )
+  {
+    QStringList theURIParts = src.split( "|" );
+    theURIParts[0] = QgsProject::instance()->writePath( theURIParts[0] );
+    src = theURIParts.join( "|" );
+  }
+  else
+  {
+    src = QgsProject::instance()->writePath( src );
+  }
 
   QDomText dataSourceText = document.createTextNode( src );
   dataSource.appendChild( dataSourceText );
@@ -284,7 +316,7 @@ bool QgsMapLayer::writeXML( QDomNode & layer_node, QDomDocument & document )
   maplayer.appendChild( layerName );
 
   // zorder
-  // This is no longer stored in the project file. It is superflous since the layers
+  // This is no longer stored in the project file. It is superfluous since the layers
   // are written and read in the proper order.
 
   // spatial reference system id
@@ -301,6 +333,8 @@ bool QgsMapLayer::writeXML( QDomNode & layer_node, QDomDocument & document )
 
   layer_node.appendChild( maplayer );
 
+  writeCustomProperties( maplayer, document );
+
   return writeXml( maplayer, document );
 
 } // bool QgsMapLayer::writeXML
@@ -325,7 +359,7 @@ bool QgsMapLayer::isValid()
 
 void QgsMapLayer::invalidTransformInput()
 {
-  QgsLogger::warning( "QgsMapLayer::invalidTransformInput() called" );
+  QgsDebugMsg( "called" );
   // TODO: emit a signal - it will be used to update legend
 }
 
@@ -342,7 +376,7 @@ QString QgsMapLayer::lastError()
 
 void QgsMapLayer::connectNotify( const char * signal )
 {
-  QgsDebugMsg( "QgsMapLayer connected to " + QString( signal ) );
+  QgsDebugMsgLevel( "QgsMapLayer connected to " + QString( signal ), 3 );
 } //  QgsMapLayer::connectNotify
 
 
@@ -394,14 +428,23 @@ void QgsMapLayer::setSubLayerVisibility( QString name, bool vis )
   // NOOP
 }
 
-const QgsCoordinateReferenceSystem& QgsMapLayer::srs()
+const QgsCoordinateReferenceSystem& QgsMapLayer::crs()
 {
   return *mCRS;
 }
 
-void QgsMapLayer::setCrs( const QgsCoordinateReferenceSystem& srs )
+const QgsCoordinateReferenceSystem& QgsMapLayer::srs()
+{
+  // This will be dropped in QGIS 2.0 due to conflicting name
+  // Please use crs() in the future
+  return *mCRS;
+}
+
+void QgsMapLayer::setCrs( const QgsCoordinateReferenceSystem& srs, bool emitSignal )
 {
   *mCRS = srs;
+  if ( emitSignal )
+    emit layerCrsChanged();
 }
 
 unsigned int QgsMapLayer::getTransparency()
@@ -544,6 +587,21 @@ QString QgsMapLayer::loadNamedStyle( const QString theURI, bool &theResultFlag )
     return myErrorMessage;
   }
 
+  // use scale dependent visibility flag
+  toggleScaleBasedVisibility( myRoot.attribute( "hasScaleBasedVisibilityFlag" ).toInt() == 1 );
+  setMinimumScale( myRoot.attribute( "minimumScale" ).toFloat() );
+  setMaximumScale( myRoot.attribute( "maximumScale" ).toFloat() );
+
+  //read transparency level
+  QDomNode transparencyNode = myRoot.namedItem( "transparencyLevelInt" );
+  if ( ! transparencyNode.isNull() )
+  {
+    // set transparency level only if it's in project
+    // (otherwise it sets the layer transparent)
+    QDomElement myElement = transparencyNode.toElement();
+    setTransparency( myElement.text().toInt() );
+  }
+
   QString errorMsg;
   theResultFlag = readSymbology( myRoot, errorMsg );
   if ( !theResultFlag )
@@ -573,6 +631,18 @@ QString QgsMapLayer::saveNamedStyle( const QString theURI, bool & theResultFlag
   myRootNode.setAttribute( "version", QString( "%1" ).arg( QGis::QGIS_VERSION ) );
   myDocument.appendChild( myRootNode );
 
+  // use scale dependent visibility flag
+  myRootNode.setAttribute( "hasScaleBasedVisibilityFlag", hasScaleBasedVisibility() ? 1 : 0 );
+  myRootNode.setAttribute( "minimumScale", minimumScale() );
+  myRootNode.setAttribute( "maximumScale", maximumScale() );
+
+  // <transparencyLevelInt>
+  QDomElement transparencyLevelIntElement = myDocument.createElement( "transparencyLevelInt" );
+  QDomText    transparencyLevelIntText    = myDocument.createTextNode( QString::number( getTransparency() ) );
+  transparencyLevelIntElement.appendChild( transparencyLevelIntText );
+  myRootNode.appendChild( transparencyLevelIntElement );
+  // now append layer node to map layer node
+
   QString errorMsg;
   if ( !writeSymbology( myRootNode, myDocument, errorMsg ) )
   {
@@ -581,9 +651,22 @@ QString QgsMapLayer::saveNamedStyle( const QString theURI, bool & theResultFlag
 
   // check if the uri is a file or ends with .qml,
   // which indicates that it should become one
-  // everything else goes to the database.
-  QFileInfo myFileInfo( theURI );
-  if ( myFileInfo.exists() || theURI.endsWith( ".qml", Qt::CaseInsensitive ) )
+  // everything else goes to the database
+  QString filename;
+
+  QgsVectorLayer *vlayer = qobject_cast<QgsVectorLayer *>( this );
+  if ( vlayer && vlayer->providerType() == "ogr" )
+  {
+    QStringList theURIParts = theURI.split( "|" );
+    filename = theURIParts[0];
+  }
+  else
+  {
+    filename = theURI;
+  }
+
+  QFileInfo myFileInfo( filename );
+  if ( myFileInfo.exists() || filename.endsWith( ".qml", Qt::CaseInsensitive ) )
   {
     QFileInfo myDirInfo( myFileInfo.path() );  //excludes file name
     if ( !myDirInfo.isWritable() )
@@ -701,3 +784,69 @@ QUndoStack* QgsMapLayer::undoStack()
 {
   return &mUndoStack;
 }
+
+
+void QgsMapLayer::setCustomProperty( const QString& key, const QVariant& value )
+{
+  mCustomProperties[key] = value;
+}
+
+QVariant QgsMapLayer::customProperty( const QString& value, const QVariant& defaultValue ) const
+{
+  return mCustomProperties.value( value, defaultValue );
+}
+
+void QgsMapLayer::removeCustomProperty( const QString& key )
+{
+  mCustomProperties.remove( key );
+}
+
+void QgsMapLayer::readCustomProperties( QDomNode & layerNode )
+{
+  QDomNode propsNode = layerNode.namedItem( "customproperties" );
+  if ( propsNode.isNull() ) // no properties stored...
+    return;
+
+  mCustomProperties.clear();
+
+  QDomNodeList nodes = propsNode.childNodes();
+
+  for ( int i = 0; i < nodes.size(); i++ )
+  {
+    QDomNode propNode = nodes.at( i );
+    if ( propNode.isNull() || propNode.nodeName() != "property" )
+      continue;
+    QDomElement propElement = propNode.toElement();
+
+    QString key = propElement.attribute( "key" );
+    QString value = propElement.attribute( "value" );
+    mCustomProperties[key] = QVariant( value );
+  }
+
+}
+
+void QgsMapLayer::writeCustomProperties( QDomNode & layerNode, QDomDocument & doc )
+{
+  QDomElement propsElement = doc.createElement( "customproperties" );
+
+  for ( QMap<QString, QVariant>::const_iterator it = mCustomProperties.begin(); it != mCustomProperties.end(); ++it )
+  {
+    QDomElement propElement = doc.createElement( "property" );
+    propElement.setAttribute( "key", it.key() );
+    propElement.setAttribute( "value", it.value().toString() );
+    propsElement.appendChild( propElement );
+  }
+
+  layerNode.appendChild( propsElement );
+}
+
+void QgsMapLayer::setCacheImage( QImage * thepImage )
+{
+  QgsDebugMsg( "cache Image set!" );
+  if ( mpCacheImage )
+  {
+    delete mpCacheImage;
+  }
+  mpCacheImage = thepImage;
+}
+
