@@ -58,6 +58,8 @@ QgsMapRenderer::QgsMapRenderer()
   mDestCRS = new QgsCoordinateReferenceSystem( GEO_EPSG_CRS_ID, QgsCoordinateReferenceSystem::EpsgCrsId ); //WGS 84
 
   mOutputUnits = QgsMapRenderer::Millimeters;
+
+  mLabelingEngine = NULL;
 }
 
 QgsMapRenderer::~QgsMapRenderer()
@@ -65,6 +67,7 @@ QgsMapRenderer::~QgsMapRenderer()
   delete mScaleCalculator;
   delete mDistArea;
   delete mDestCRS;
+  delete mLabelingEngine;
 }
 
 
@@ -80,6 +83,8 @@ void QgsMapRenderer::updateScale()
 
 bool QgsMapRenderer::setExtent( const QgsRectangle& extent )
 {
+  //remember the previous extent
+  mLastExtent = mExtent;
 
   // Don't allow zooms where the current extent is so small that it
   // can't be accurately represented using a double (which is what
@@ -175,16 +180,10 @@ void QgsMapRenderer::adjustExtentToSize()
     dymax = mExtent.yMaximum() + whitespace;
   }
 
-#ifdef QGISDEBUG
-  QString myMessage = "+-------------------MapRenderer--------------------------------+\n";
-  myMessage += QString( "Map units per pixel (x,y) : %1, %2\n" ).arg( mapUnitsPerPixelX ).arg( mapUnitsPerPixelY );
-  myMessage += QString( "Pixmap dimensions (x,y) : %1, %2\n" ).arg( myWidth ).arg( myHeight );
-  myMessage += QString( "Extent dimensions (x,y) : %1, %2\n" ).arg( mExtent.width() ).arg( mExtent.height() );
-  myMessage += mExtent.toString();
-  std::cout << myMessage.toLocal8Bit().constData() << std::endl; // OK
-
-#endif
-
+  QgsDebugMsg( QString( "Map units per pixel (x,y) : %1, %2\n" ).arg( mapUnitsPerPixelX ).arg( mapUnitsPerPixelY ) );
+  QgsDebugMsg( QString( "Pixmap dimensions (x,y) : %1, %2\n" ).arg( myWidth ).arg( myHeight ) );
+  QgsDebugMsg( QString( "Extent dimensions (x,y) : %1, %2\n" ).arg( mExtent.width() ).arg( mExtent.height() ) );
+  QgsDebugMsg( mExtent.toString() );
 
   // update extent
   mExtent.setXMinimum( dxmin );
@@ -195,9 +194,7 @@ void QgsMapRenderer::adjustExtentToSize()
   // update the scale
   updateScale();
 
-#ifdef QGISDEBUG
-  QgsLogger::debug( "Scale (assuming meters as map units) = 1", mScale, 1, __FILE__, __FUNCTION__, __LINE__ );
-#endif
+  QgsDebugMsg( QString( "Scale (assuming meters as map units) = 1:%1" ).arg( mScale ) );
 
   newCoordXForm.setParameters( mMapUnitsPerPixel, dxmin, dymin, myHeight );
   mRenderContext.setMapToPixel( newCoordXForm );
@@ -207,16 +204,23 @@ void QgsMapRenderer::adjustExtentToSize()
 
 void QgsMapRenderer::render( QPainter* painter )
 {
+  //flag to see if the render context has changed
+  //since the last time we rendered. If it hasnt changed we can
+  //take some shortcuts with rendering
+  bool mySameAsLastFlag = true;
+
   QgsDebugMsg( "========== Rendering ==========" );
 
   if ( mExtent.isEmpty() )
   {
-    QgsLogger::debug( "empty extent... not rendering" );
+    QgsDebugMsg( "empty extent... not rendering" );
     return;
   }
 
   if ( mDrawing )
+  {
     return;
+  }
 
   QPaintDevice* thePaintDevice = painter->device();
   if ( !thePaintDevice )
@@ -251,9 +255,43 @@ void QgsMapRenderer::render( QPainter* painter )
     scaleFactor = sceneDpi / 25.4;
   }
   double rasterScaleFactor = ( thePaintDevice->logicalDpiX() + thePaintDevice->logicalDpiY() ) / 2.0 / sceneDpi;
-  mRenderContext.setScaleFactor( scaleFactor );
-  mRenderContext.setRasterScaleFactor( rasterScaleFactor );
-  mRenderContext.setRendererScale( mScale );
+  if ( mRenderContext.rasterScaleFactor() != rasterScaleFactor )
+  {
+    mRenderContext.setRasterScaleFactor( rasterScaleFactor );
+    mySameAsLastFlag = false;
+  }
+  if ( mRenderContext.scaleFactor() != scaleFactor )
+  {
+    mRenderContext.setScaleFactor( scaleFactor );
+    mySameAsLastFlag = false;
+  }
+  if ( mRenderContext.rendererScale() != mScale )
+  {
+    //add map scale to render context
+    mRenderContext.setRendererScale( mScale );
+    mySameAsLastFlag = false;
+  }
+  if ( mLastExtent != mExtent )
+  {
+    mLastExtent = mExtent;
+    mySameAsLastFlag = false;
+  }
+
+  mRenderContext.setLabelingEngine( mLabelingEngine );
+  if ( mLabelingEngine )
+    mLabelingEngine->init();
+
+  // know we know if this render is just a repeat of the last time, we
+  // can clear caches if it has changed
+  if ( !mySameAsLastFlag )
+  {
+    //clear the cache pixmap if we changed resolution / extent
+    QSettings mySettings;
+    if ( mySettings.value( "/qgis/enable_render_caching", false ).toBool() )
+    {
+      QgsMapLayerRegistry::instance()->clearAllLayerCaches();
+    }
+  }
 
   bool placeOverlays = false;
   QgsOverlayObjectPositionManager* overlayManager = overlayManagerFromSettings();
@@ -263,9 +301,6 @@ void QgsMapRenderer::render( QPainter* painter )
     placeOverlays = true;
   }
 
-  //add map scale to render context
-  mRenderContext.setRendererScale( mScale );
-
   // render all layers in the stack, starting at the base
   QListIterator<QString> li( mLayerSet );
   li.toBack();
@@ -279,6 +314,10 @@ void QgsMapRenderer::render( QPainter* painter )
       break;
     }
 
+    // Store the painter in case we need to swap it out for the
+    // cache painter
+    QPainter * mypContextPainter = mRenderContext.painter();
+
     QString layerId = li.previous();
 
     QgsDebugMsg( "Rendering at layer item " + layerId );
@@ -296,7 +335,7 @@ void QgsMapRenderer::render( QPainter* painter )
 
     if ( !ml )
     {
-      QgsLogger::warning( "Layer not found in registry!" );
+      QgsDebugMsg( "Layer not found in registry!" );
       continue;
     }
 
@@ -343,21 +382,10 @@ void QgsMapRenderer::render( QPainter* painter )
       }
 
 
-      if ( scaleRaster )
-      {
-        bk_mapToPixel = mRenderContext.mapToPixel();
-        rasterMapToPixel = mRenderContext.mapToPixel();
-        rasterMapToPixel.setMapUnitsPerPixel( mRenderContext.mapToPixel().mapUnitsPerPixel() / rasterScaleFactor );
-        rasterMapToPixel.setYMaximum( mSize.height() * rasterScaleFactor );
-        mRenderContext.setMapToPixel( rasterMapToPixel );
-        mRenderContext.painter()->save();
-        mRenderContext.painter()->scale( 1.0 / rasterScaleFactor, 1.0 / rasterScaleFactor );
-      }
-
       //create overlay objects for features within the view extent
       if ( ml->type() == QgsMapLayer::VectorLayer && overlayManager )
       {
-        QgsVectorLayer* vl = dynamic_cast<QgsVectorLayer*>( ml );
+        QgsVectorLayer* vl = qobject_cast<QgsVectorLayer *>( ml );
         if ( vl )
         {
           QList<QgsVectorOverlay*> thisLayerOverlayList;
@@ -377,10 +405,69 @@ void QgsMapRenderer::render( QPainter* painter )
         }
       }
 
+      // Force render of layers that are being edited
+      // or if there's a labeling engine that needs the layer to register features
+      if ( ml->type() == QgsMapLayer::VectorLayer )
+      {
+        QgsVectorLayer* vl = qobject_cast<QgsVectorLayer *>( ml );
+        if ( vl->isEditable() ||
+             ( mRenderContext.labelingEngine() && mRenderContext.labelingEngine()->willUseLayer( vl ) ) )
+        {
+          ml->setCacheImage( 0 );
+        }
+      }
+
+      QSettings mySettings;
+      if ( ! split )//render caching does not yet cater for split extents
+      {
+        if ( mySettings.value( "/qgis/enable_render_caching", false ).toBool() )
+        {
+          if ( !mySameAsLastFlag || ml->cacheImage() == 0 )
+          {
+            QgsDebugMsg( "\n\n\nCaching enabled but layer redraw forced by extent change or empty cache\n\n\n" );
+            QImage * mypImage = new QImage( mRenderContext.painter()->device()->width(),
+                                            mRenderContext.painter()->device()->height(), QImage::Format_ARGB32 );
+            mypImage->fill( 0 );
+            ml->setCacheImage( mypImage ); //no need to delete the old one, maplayer does it for you
+            QPainter * mypPainter = new QPainter( ml->cacheImage() );
+            if ( mySettings.value( "/qgis/enable_anti_aliasing", false ).toBool() )
+            {
+              mypPainter->setRenderHint( QPainter::Antialiasing );
+            }
+            mRenderContext.setPainter( mypPainter );
+          }
+          else if ( mySameAsLastFlag )
+          {
+            //draw from cached image
+            QgsDebugMsg( "\n\n\nCaching enabled --- drawing layer from cached image\n\n\n" );
+            mypContextPainter->drawImage( 0, 0, *( ml->cacheImage() ) );
+            disconnect( ml, SIGNAL( drawingProgress( int, int ) ), this, SLOT( onDrawingProgress( int, int ) ) );
+            //short circuit as there is nothing else to do...
+            continue;
+          }
+        }
+      }
+
+      if ( scaleRaster )
+      {
+        bk_mapToPixel = mRenderContext.mapToPixel();
+        rasterMapToPixel = mRenderContext.mapToPixel();
+        rasterMapToPixel.setMapUnitsPerPixel( mRenderContext.mapToPixel().mapUnitsPerPixel() / rasterScaleFactor );
+        rasterMapToPixel.setYMaximum( mSize.height() * rasterScaleFactor );
+        mRenderContext.setMapToPixel( rasterMapToPixel );
+        mRenderContext.painter()->save();
+        mRenderContext.painter()->scale( 1.0 / rasterScaleFactor, 1.0 / rasterScaleFactor );
+      }
+
+
       if ( !ml->draw( mRenderContext ) )
       {
         emit drawError( ml );
       }
+      else
+      {
+        QgsDebugMsg( "Layer rendered without issues" );
+      }
 
       if ( split )
       {
@@ -397,9 +484,21 @@ void QgsMapRenderer::render( QPainter* painter )
         mRenderContext.painter()->restore();
       }
 
+      if ( mySettings.value( "/qgis/enable_render_caching", false ).toBool() )
+      {
+        if ( !split )
+        {
+          // composite the cached image into our view and then clean up from caching
+          // by reinstating the painter as it was swapped out for caching renders
+          delete mRenderContext.painter();
+          mRenderContext.setPainter( mypContextPainter );
+          //draw from cached image that we created further up
+          mypContextPainter->drawImage( 0, 0, *( ml->cacheImage() ) );
+        }
+      }
       disconnect( ml, SIGNAL( drawingProgress( int, int ) ), this, SLOT( onDrawingProgress( int, int ) ) );
     }
-    else
+    else // layer not visible due to scale
     {
       QgsDebugMsg( "Layer not rendered because it is not within the defined "
                    "visibility scale range" );
@@ -475,6 +574,16 @@ void QgsMapRenderer::render( QPainter* painter )
   // make sure progress bar arrives at 100%!
   emit drawingProgress( 1, 1 );
 
+  if ( mLabelingEngine )
+  {
+    // set correct extent
+    mRenderContext.setExtent( mExtent );
+    mRenderContext.setCoordinateTransform( NULL );
+
+    mLabelingEngine->drawLabeling( mRenderContext );
+    mLabelingEngine->exit();
+  }
+
   QgsDebugMsg( "Rendering completed in (seconds): " + QString( "%1" ).arg( renderTime.elapsed() / 1000.0 ) );
 
   mDrawing = false;
@@ -594,7 +703,7 @@ bool QgsMapRenderer::splitLayersExtent( QgsMapLayer* layer, QgsRectangle& extent
     catch ( QgsCsException &cse )
     {
       Q_UNUSED( cse );
-      QgsLogger::warning( "Transform error caught in " + QString( __FILE__ ) + ", line " + QString::number( __LINE__ ) );
+      QgsDebugMsg( "Transform error caught" );
       extent = QgsRectangle( -DBL_MAX, -DBL_MAX, DBL_MAX, DBL_MAX );
       r2     = QgsRectangle( -DBL_MAX, -DBL_MAX, DBL_MAX, DBL_MAX );
     }
@@ -638,7 +747,7 @@ QgsPoint QgsMapRenderer::layerToMapCoordinates( QgsMapLayer* theLayer, QgsPoint
     catch ( QgsCsException &cse )
     {
       Q_UNUSED( cse );
-      QgsDebugMsg( QString( "Transform error caught:%s" ).arg( cse.what() ) );
+      QgsDebugMsg( QString( "Transform error caught: %1" ).arg( cse.what() ) );
     }
   }
   else
@@ -659,7 +768,7 @@ QgsPoint QgsMapRenderer::mapToLayerCoordinates( QgsMapLayer* theLayer, QgsPoint
     }
     catch ( QgsCsException &cse )
     {
-      QgsDebugMsg( QString( "Transform error caught: %s" ).arg( cse.what() ) );
+      QgsDebugMsg( QString( "Transform error caught: %1" ).arg( cse.what() ) );
       throw cse; //let client classes know there was a transformation error
     }
   }
@@ -706,7 +815,7 @@ void QgsMapRenderer::updateFullExtent()
     QgsMapLayer * lyr = registry->mapLayer( *it );
     if ( lyr == NULL )
     {
-      QgsLogger::warning( "WARNING: layer '" + ( *it ) + "' not found in map layer registry!" );
+      QgsDebugMsg( QString( "WARNING: layer '%1' not found in map layer registry!" ).arg( *it ) );
     }
     else
     {
@@ -944,3 +1053,11 @@ bool QgsMapRenderer::writeXML( QDomNode & theNode, QDomDocument & theDoc )
 
   return true;
 }
+
+void QgsMapRenderer::setLabelingEngine( QgsLabelingEngineInterface* iface )
+{
+  if ( mLabelingEngine )
+    delete mLabelingEngine;
+
+  mLabelingEngine = iface;
+}
