@@ -14,12 +14,13 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-/* $Id: qgspoint.cpp 9218 2008-08-30 14:53:45Z jef $ */
+/* $Id$ */
 
 
 #include "qgspoint.h"
 #include <cmath>
 #include <QTextStream>
+#include <QObject> // for tr()
 
 
 QgsPoint::QgsPoint( const QgsPoint& p )
@@ -44,6 +45,31 @@ QString QgsPoint::toString( int thePrecision ) const
   return rep;
 }
 
+QString QgsPoint::toDegreesMinutesSeconds( int thePrecision ) const
+{
+  int myDegreesX = int( std::abs( m_x ) );
+  float myFloatMinutesX = float(( std::abs( m_x ) - myDegreesX ) * 60 );
+  int myIntMinutesX = int( myFloatMinutesX );
+  float mySecondsX = float( myFloatMinutesX - myIntMinutesX ) * 60;
+
+  int myDegreesY = int( std::abs( m_y ) );
+  float myFloatMinutesY = float(( std::abs( m_y ) - myDegreesY ) * 60 );
+  int myIntMinutesY = int( myFloatMinutesY );
+  float mySecondsY = float( myFloatMinutesY - myIntMinutesY ) * 60;
+
+  QString myXHemisphere = m_x < 0 ? QObject::tr( "W" ) : QObject::tr( "E" );
+  QString myYHemisphere = m_y < 0 ? QObject::tr( "S" ) : QObject::tr( "N" );
+  QString rep = QString::number( myDegreesX ) + QChar( 176 ) +
+                QString::number( myIntMinutesX ) + QString( "'" ) +
+                QString::number( mySecondsX, 'f', thePrecision ) + QString( "\"" ) +
+                myXHemisphere + QString( "," ) +
+                QString::number( myDegreesY ) + QChar( 176 ) +
+                QString::number( myIntMinutesY ) + QString( "'" ) +
+                QString::number( mySecondsY, 'f', thePrecision ) + QString( "\"" ) +
+                myYHemisphere;
+  return rep;
+}
+
 
 QString QgsPoint::wellKnownText() const
 {
@@ -52,7 +78,7 @@ QString QgsPoint::wellKnownText() const
 
 double QgsPoint::sqrDist( double x, double y ) const
 {
-  return ( m_x -x )*( m_x - x ) + ( m_y - y )*( m_y - y );
+  return ( m_x - x )*( m_x - x ) + ( m_y - y )*( m_y - y );
 }
 
 double QgsPoint::sqrDist( const QgsPoint& other ) const
@@ -98,7 +124,7 @@ int QgsPoint::onSegment( const QgsPoint& a, const QgsPoint& b ) const
 {
   //algorithm from 'graphics GEMS', A. Paeth: 'A Fast 2D Point-on-line test'
   if (
-    fabs(( b.y() - a.y() ) * ( m_x - a.x() ) - ( m_y - a.y() ) * ( b.x() - a.x() ) )
+    fabs(( b.y() - a.y() ) *( m_x - a.x() ) - ( m_y - a.y() ) *( b.x() - a.x() ) )
     >= qMax( fabs( b.x() - a.x() ), fabs( b.y() - a.y() ) )
   )
   {
