@@ -15,7 +15,7 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-/* $Id: qgsscalecalculator.cpp 11580 2009-09-07 09:42:28Z jef $ */
+/* $Id$ */
 
 #include <assert.h>
 #include <math.h>
@@ -68,7 +68,17 @@ double QgsScaleCalculator::calculate( const QgsRectangle &mapExtent, int canvasW
       conversionFactor = 12.0;
       delta = mapExtent.xMaximum() - mapExtent.xMinimum();
       break;
-    case QGis::Degrees:
+    case QGis::DecimalDegrees:
+      // degrees require conversion to meters first
+      conversionFactor = 39.3700787;
+      delta = calculateGeographicDistance( mapExtent );
+      break;
+    case QGis::DegreesMinutesSeconds:
+      // degrees require conversion to meters first
+      conversionFactor = 39.3700787;
+      delta = calculateGeographicDistance( mapExtent );
+      break;
+    case QGis::DegreesDecimalMinutes:
       // degrees require conversion to meters first
       conversionFactor = 39.3700787;
       delta = calculateGeographicDistance( mapExtent );
