@@ -15,7 +15,7 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-/* $Id: qgssymbol.cpp 11444 2009-08-19 19:47:41Z mhugent $ */
+/* $Id$ */
 #include <cmath>
 
 #include "qgis.h"
@@ -217,41 +217,43 @@ void QgsSymbol::setNamedPointSymbol( QString name )
       QgsDebugMsg( "\n\n\n *** Svg Symbol not found on fs ***" );
       QgsDebugMsg( "Name: " + name );
       //see if we can resolve the problem...
-      //by using the qgis svg dir from this local machine
-      //one day when user specified svg are allowed we need
-      //to adjust this logic probably...
-      QString svgPath = QgsApplication::svgPath();
-      QgsDebugMsg( "SvgPath: " + svgPath );
-      QFileInfo myInfo( myTempName );
-      QString myFileName = myInfo.fileName(); // foo.svg
-      QString myLowestDir = myInfo.dir().dirName();
-      QString myLocalPath = svgPath + QDir::separator() +
-                            myLowestDir + QDir::separator() +
-                            myFileName;
-      QgsDebugMsg( "Alternative svg path: " + myLocalPath );
-      if ( QFile( myLocalPath ).exists() )
+      //
+
+      QStringList svgPaths = QgsApplication::svgPaths();
+      for ( int i = 0; i < svgPaths.size(); i++ )
       {
-        name = "svg:" + myLocalPath;
-        QgsDebugMsg( "Svg found in alternative path" );
-      }
-      else if ( myInfo.isRelative() )
-      {
-        QFileInfo pfi( QgsProject::instance()->fileName() );
-        if ( pfi.exists() && QFile( pfi.canonicalPath() + QDir::separator() + myTempName ).exists() )
+        QgsDebugMsg( "SvgPath: " + svgPaths[i] );
+        QFileInfo myInfo( myTempName );
+        QString myFileName = myInfo.fileName(); // foo.svg
+        QString myLowestDir = myInfo.dir().dirName();
+        QString myLocalPath = svgPaths[i] + "/" + myLowestDir + "/" + myFileName;
+
+        QgsDebugMsg( "Alternative svg path: " + myLocalPath );
+        if ( QFile( myLocalPath ).exists() )
         {
-          name = "svg:" + pfi.canonicalPath() + QDir::separator() + myTempName;
+          name = "svg:" + myLocalPath;
           QgsDebugMsg( "Svg found in alternative path" );
         }
+        else if ( myInfo.isRelative() )
+        {
+          QFileInfo pfi( QgsProject::instance()->fileName() );
+          if ( pfi.exists() && QFile( pfi.canonicalPath() + QDir::separator() + myTempName ).exists() )
+          {
+            name = "svg:" + pfi.canonicalPath() + QDir::separator() + myTempName;
+            QgsDebugMsg( "Svg found in alternative path" );
+            break;
+          }
+          else
+          {
+            QgsDebugMsg( "Svg not found in project path" );
+          }
+        }
         else
         {
-          QgsDebugMsg( "Svg not found in project path" );
+          //couldnt find the file, no happy ending :-(
+          QgsDebugMsg( "Computed alternate path but no svg there either" );
         }
       }
-      else
-      {
-        //couldnt find the file, no happy ending :-(
-        QgsDebugMsg( "Computed alternate path but no svg there either" );
-      }
     }
   }
   mPointSymbolName = name;
@@ -294,7 +296,7 @@ double QgsSymbol::pointSize() const
 
 QImage QgsSymbol::getLineSymbolAsImage()
 {
-  //Note by Tim: dont use premultiplied - it causes
+  //Note by Tim: don't use premultiplied - it causes
   //artifacts on the output icon!
   QImage img( 15, 15, QImage::Format_ARGB32 );//QImage::Format_ARGB32_Premultiplied);
   //0 = fully transparent
@@ -314,7 +316,7 @@ QImage QgsSymbol::getLineSymbolAsImage()
 
 QImage QgsSymbol::getPolygonSymbolAsImage()
 {
-  //Note by Tim: dont use premultiplied - it causes
+  //Note by Tim: don't use premultiplied - it causes
   //artifacts on the output icon!
   QImage img( 15, 15, QImage::Format_ARGB32 ); //, QImage::Format_ARGB32_Premultiplied);
   //0 = fully transparent
@@ -325,7 +327,7 @@ QImage QgsSymbol::getPolygonSymbolAsImage()
   p.setBrush( mBrush );
   QPolygon myPolygon;
   //leave a little white space around so
-  //dont draw at 0,0,15,15
+  //don't draw at 0,0,15,15
   myPolygon << QPoint( 2, 2 )
   << QPoint( 1, 5 )
   << QPoint( 1, 10 )
@@ -409,9 +411,9 @@ QImage QgsSymbol::getPointSymbolAsImage( double widthScale, bool selected, QColo
   }
   else
   {
-    QgsDebugMsg( QString( "marker:%1 mPointSize:%2 mPointSizeUnits:%3 scale:%4 widthScale:%5 rasterScaleFactor:%6 opacity:%7" )
+    QgsDebugMsgLevel( QString( "marker:%1 mPointSize:%2 mPointSizeUnits:%3 scale:%4 widthScale:%5 rasterScaleFactor:%6 opacity:%7" )
                  .arg( mPointSymbolName ).arg( mSize ).arg( mSizeInMapUnits ? "true" : "false" )
-                 .arg( scale ).arg( widthScale ).arg( rasterScaleFactor ).arg( opacity ) );
+                 .arg( scale ).arg( widthScale ).arg( rasterScaleFactor ).arg( opacity ), 3 );
 
 
     preRotateImage = QgsMarkerCatalogue::instance()->imageMarker(
@@ -432,8 +434,8 @@ void QgsSymbol::cache( QColor selectionColor )
   QPen pen = mPen;
   pen.setColor( selectionColor );
   QBrush brush = mBrush;
-  // For symbols that have a different coloured border, the line
-  // below causes the fill colour to be wrong for the print
+  // For symbols that have a different colored border, the line
+  // below causes the fill color to be wrong for the print
   // composer. Not sure why...
   // brush.setColor ( selectionColor );
 
@@ -511,11 +513,17 @@ bool QgsSymbol::writeXML( QDomNode & item, QDomDocument & document, const QgsVec
     {
       name = fi.canonicalFilePath();
 
-      QString dir = QFileInfo( QgsApplication::svgPath() ).canonicalFilePath();
+      QStringList svgPaths = QgsApplication::svgPaths();
 
-      if ( !dir.isEmpty() && name.startsWith( dir ) )
+      for ( int i = 0; i < svgPaths.size(); i++ )
       {
-        name = name.mid( dir.size() );
+        QString dir = QFileInfo( svgPaths[i] ).canonicalFilePath();
+
+        if ( !dir.isEmpty() && name.startsWith( dir ) )
+        {
+          name = name.mid( dir.size() );
+          break;
+        }
       }
     }
 
@@ -524,6 +532,7 @@ bool QgsSymbol::writeXML( QDomNode & item, QDomDocument & document, const QgsVec
 
   appendText( symbol, document, "pointsymbol", name );
   appendText( symbol, document, "pointsize", QString::number( pointSize() ) );
+  appendText( symbol, document, "pointsizeunits", pointSizeUnits() ? "mapunits" : "pixels" );
 
   if ( vl )
   {
@@ -548,7 +557,7 @@ bool QgsSymbol::writeXML( QDomNode & item, QDomDocument & document, const QgsVec
   symbol.appendChild( fillcolor );
 
   appendText( symbol, document, "fillpattern", QgsSymbologyUtils::brushStyle2QString( mBrush.style() ) );
-  appendText( symbol, document, "texturepath", mTextureFilePath );
+  appendText( symbol, document, "texturepath", QgsProject::instance()->writePath( mTextureFilePath ) );
 
   return returnval;
 }
@@ -668,7 +677,7 @@ bool QgsSymbol::readXML( QDomNode &synode, const QgsVectorLayer *vl )
 
   QDomNode texturepathnode = synode.namedItem( "texturepath" );
   QDomElement texturepathelement = texturepathnode.toElement();
-  setCustomTexture( texturepathelement.text() );
+  setCustomTexture( QgsProject::instance()->readPath( texturepathelement.text() ) );
 
   //run this after setting the custom texture path, so we override the brush if it isn't the custom pattern brush.
   QDomNode fillpnode = synode.namedItem( "fillpattern" );
