@@ -260,7 +260,7 @@ void QgsMapCanvas::setLayerSet( QList<QgsMapCanvasLayer>& layers )
       QgsMapLayer *currentLayer = layer( i );
       disconnect( currentLayer, SIGNAL( repaintRequested() ), this, SLOT( refresh() ) );
       disconnect( currentLayer, SIGNAL( screenUpdateRequested() ), this, SLOT( updateMap() ) );
-      QgsVectorLayer *isVectLyr = dynamic_cast < QgsVectorLayer * >( currentLayer );
+      QgsVectorLayer *isVectLyr = qobject_cast<QgsVectorLayer *>( currentLayer );
       if ( isVectLyr )
       {
         disconnect( currentLayer, SIGNAL( selectionChanged() ), this, SLOT( selectionChangedSlot() ) );
@@ -276,7 +276,7 @@ void QgsMapCanvas::setLayerSet( QList<QgsMapCanvasLayer>& layers )
       QgsMapLayer *currentLayer = layer( i );
       connect( currentLayer, SIGNAL( repaintRequested() ), this, SLOT( refresh() ) );
       connect( currentLayer, SIGNAL( screenUpdateRequested() ), this, SLOT( updateMap() ) );
-      QgsVectorLayer *isVectLyr = dynamic_cast < QgsVectorLayer * >( currentLayer );
+      QgsVectorLayer *isVectLyr = qobject_cast<QgsVectorLayer *>( currentLayer );
       if ( isVectLyr )
       {
         connect( currentLayer, SIGNAL( selectionChanged() ), this, SLOT( selectionChangedSlot() ) );
@@ -504,9 +504,20 @@ void QgsMapCanvas::setExtent( QgsRectangle const & r )
     mLastExtent.removeAt( i );
   }
 
-
   mLastExtent.append( extent() ) ;
+
+  // adjust history to no more than 20
+  if ( mLastExtent.size() > 20 )
+  {
+    mLastExtent.removeAt( 0 );
+  }
+
+  // the last item is the current extent
   mLastExtentIndex = mLastExtent.size() - 1;
+
+  // update controls' enabled state
+  emit zoomLastStatusChanged( mLastExtentIndex > 0 );
+  emit zoomNextStatusChanged( mLastExtentIndex < mLastExtent.size() - 1 );
   // notify canvas items of change
   updateCanvasItemPositions();
 
@@ -558,7 +569,7 @@ void QgsMapCanvas::zoomToPreviousExtent()
     return;
   }
 
-  if ( mLastExtentIndex > 1 )
+  if ( mLastExtentIndex > 0 )
   {
     mLastExtentIndex--;
     mMapRenderer->setExtent( mLastExtent[mLastExtentIndex] );
@@ -566,9 +577,12 @@ void QgsMapCanvas::zoomToPreviousExtent()
     updateScale();
     if ( mMapOverview )
       mMapOverview->drawExtentRect();
+    refresh();
+    // update controls' enabled state
+    emit zoomLastStatusChanged( mLastExtentIndex > 0 );
+    emit zoomNextStatusChanged( mLastExtentIndex < mLastExtent.size() - 1 );
   }
 
-  refresh();
 } // zoomToPreviousExtent
 
 void QgsMapCanvas::zoomToNextExtent()
@@ -585,10 +599,22 @@ void QgsMapCanvas::zoomToNextExtent()
     updateScale();
     if ( mMapOverview )
       mMapOverview->drawExtentRect();
+    refresh();
+    // update controls' enabled state
+    emit zoomLastStatusChanged( mLastExtentIndex > 0 );
+    emit zoomNextStatusChanged( mLastExtentIndex < mLastExtent.size() - 1 );
   }
-  refresh();
 }// zoomToNextExtent
 
+void QgsMapCanvas::clearExtentHistory()
+{
+  mLastExtent.clear(); // clear the zoom history list
+  mLastExtent.append( extent() ) ; // set the current extent in the list
+  mLastExtentIndex = mLastExtent.size() - 1;
+  // update controls' enabled state
+  emit zoomLastStatusChanged( mLastExtentIndex > 0 );
+  emit zoomNextStatusChanged( mLastExtentIndex < mLastExtent.size() - 1 );
+}// clearExtentHistory
 
 
 bool QgsMapCanvas::hasCrsTransformEnabled()
@@ -624,7 +650,7 @@ void QgsMapCanvas::zoomToSelected( QgsVectorLayer* layer )
   if ( layer == NULL )
   {
     // use current layer by default
-    layer = dynamic_cast < QgsVectorLayer * >( mCurrentLayer );
+    layer = qobject_cast<QgsVectorLayer *>( mCurrentLayer );
   }
 
   if ( layer == NULL )
@@ -632,6 +658,11 @@ void QgsMapCanvas::zoomToSelected( QgsVectorLayer* layer )
     return;
   }
 
+  if ( layer->selectedFeatureCount() == 0 )
+  {
+    return;
+  }
+
   QgsRectangle rect = mMapRenderer->layerExtentToOutputExtent( layer, layer->boundingBoxOfSelected() );
 
   // no selected features, only one selected point feature
@@ -934,7 +965,7 @@ void QgsMapCanvas::updateCanvasItemPositions()
   QList<QGraphicsItem*>::iterator it = list.begin();
   while ( it != list.end() )
   {
-    QgsMapCanvasItem* item = dynamic_cast<QgsMapCanvasItem*>( *it );
+    QgsMapCanvasItem* item = dynamic_cast<QgsMapCanvasItem *>( *it );
 
     if ( item )
     {
@@ -1261,8 +1292,6 @@ void QgsMapCanvas::panActionEnd( QPoint releasePoint )
 
 void QgsMapCanvas::panAction( QMouseEvent * e )
 {
-  QgsDebugMsg( "panAction: entering." );
-
   if ( mDrawing )
   {
     return;
@@ -1286,8 +1315,6 @@ void QgsMapCanvas::moveCanvasContents( bool reset )
   if ( !reset )
     pnt += mCanvasProperties->mouseLastXY - mCanvasProperties->rubberStartPoint;
 
-  QgsDebugMsg( "moveCanvasContents: pnt " + QString::number( pnt.x() ) + "," + QString::number( pnt.y() ) );
-
   mMap->setPanningOffset( pnt );
 
   QList<QGraphicsItem*> list = mScene->items();
@@ -1299,7 +1326,7 @@ void QgsMapCanvas::moveCanvasContents( bool reset )
     if ( item != mMap )
     {
       // this tells map canvas item to draw with offset
-      QgsMapCanvasItem* canvasItem = dynamic_cast<QgsMapCanvasItem*>( item );
+      QgsMapCanvasItem* canvasItem = dynamic_cast<QgsMapCanvasItem *>( item );
       if ( canvasItem )
         canvasItem->setPanningOffset( pnt );
     }
@@ -1344,6 +1371,7 @@ void QgsMapCanvas::readProject( const QDomDocument & doc )
   {
     QDomNode node = nodes.item( 0 );
     mMapRenderer->readXML( node );
+    clearExtentHistory(); // clear the extent history on project load
   }
   else
   {
