@@ -97,7 +97,7 @@ void QgsQuickPrint::setOutputPdf( QString theFileName )
 void QgsQuickPrint::setMapCanvas( QgsMapCanvas * thepMapCanvas )
 {
   mpMapRenderer = thepMapCanvas->mapRenderer();
-  mMapBackgroundColour = thepMapCanvas->canvasColor();
+  mMapBackgroundColor = thepMapCanvas->canvasColor();
 }
 void QgsQuickPrint::setMapRenderer( QgsMapRenderer * thepMapRenderer )
 {
@@ -105,7 +105,7 @@ void QgsQuickPrint::setMapRenderer( QgsMapRenderer * thepMapRenderer )
 }
 void QgsQuickPrint::setMapBackgroundColor( QColor theColor )
 {
-  mMapBackgroundColour = theColor;
+  mMapBackgroundColor = theColor;
 }
 void QgsQuickPrint::setPageSize( QPrinter::PageSize theSize )
 {
@@ -247,9 +247,9 @@ void QgsQuickPrint::printMap()
 #endif
   QString myFontFamily = "Arial";
 
-  // Background colour for pixmaps
-  QColor myLegendBackgroundColour = Qt::white;
-  //QColor myMapBackgroundColour = "#98dbf9"; // nice blue colour
+  // Background color for pixmaps
+  QColor myLegendBackgroundColor = Qt::white;
+  //QColor myMapBackgroundColor = "#98dbf9"; // nice blue color
 
 
   //
@@ -402,8 +402,9 @@ void QgsQuickPrint::printMap()
     if ( mypLayer )
     {
       QgsVectorLayer *mypVectorLayer  =
-        dynamic_cast<QgsVectorLayer *>( mypLayer );
-      if ( mypVectorLayer )
+        qobject_cast<QgsVectorLayer *>( mypLayer );
+      // TODO: add support for symbology-ng renderers
+      if ( mypVectorLayer && mypVectorLayer->renderer() )
       {
         QString myLayerName = mypVectorLayer->name();
         QIcon myIcon;
@@ -684,11 +685,11 @@ void QgsQuickPrint::scaleTextLabels( int theScaleFactor, SymbolScalingType theDi
     if ( mypLayer )
     {
       QgsVectorLayer *mypVectorLayer  =
-        dynamic_cast<QgsVectorLayer *>( mypLayer );
+        qobject_cast<QgsVectorLayer *>( mypLayer );
       if ( mypVectorLayer )
       {
         QgsLabel * mypLabel = mypVectorLayer->label();
-        QgsLabelAttributes * mypLabelAttributes = mypLabel->layerAttributes();
+        QgsLabelAttributes * mypLabelAttributes = mypLabel->labelAttributes();
         if ( theDirection == ScaleUp )
         {
           mypLabelAttributes->setSize(
@@ -724,7 +725,7 @@ void QgsQuickPrint::scalePointSymbols( int theScaleFactor, SymbolScalingType the
     if ( mypLayer )
     {
       QgsVectorLayer *mypVectorLayer  =
-        dynamic_cast<QgsVectorLayer *>( mypLayer );
+        qobject_cast<QgsVectorLayer *>( mypLayer );
       if ( mypVectorLayer )
       {
         const QgsRenderer* mypRenderer = mypVectorLayer->renderer();
@@ -779,7 +780,7 @@ void QgsQuickPrint::renderPrintScaleBar( QPainter * thepPainter,
 {
   //hard coding some options for now
   bool mySnappingFlag = true;
-  QColor mColour = Qt::black;
+  QColor mColor = Qt::black;
   // Hard coded sizes
   int myTextOffsetX = 0;
   int myTextOffsetY = 5;
@@ -904,7 +905,7 @@ void QgsQuickPrint::renderPrintScaleBar( QPainter * thepPainter,
   int myOriginY = myYMargin;
 
   //Set pen to draw with
-  QPen myForegroundPen( mColour, 2 );
+  QPen myForegroundPen( mColor, 2 );
   QPen myBackgroundPen( Qt::white, 3 );
 
   //Cast myScaleBarWidth to int for drawing
