@@ -18,7 +18,7 @@ email                : tim@linfiniti.com
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-/*  $Id: plugin.cpp 10967 2009-06-23 12:03:18Z borysiasty $ */
+/*  $Id$ */
 
 // includes
 
@@ -46,7 +46,7 @@ email                : tim@linfiniti.com
 #include "qgslogger.h"
 
 
-static const char * const ident_ = "$Id: plugin.cpp 10967 2009-06-23 12:03:18Z borysiasty $";
+static const char * const ident_ = "$Id$";
 
 static const QString name_ = QObject::tr( "CopyrightLabel" );
 static const QString description_ = QObject::tr( "Draws copyright information" );
@@ -108,8 +108,9 @@ void QgsCopyrightLabelPlugin::projectRead()
   now = QDate::currentDate();
   defString = "&copy; QGIS " + now.toString( "yyyy" );
 
-  mQFont.setFamily( QgsProject::instance()->readEntry( "CopyrightLabel", "/FontName", "Sans Serif" ) );
-  mQFont.setPointSize( QgsProject::instance()->readNumEntry( "CopyrightLabel", "/FontSize", 9 ) );
+  // there is no font setting in the UI, so just use the Qt/QGIS default font (what mQFont gets when created)
+  //  mQFont.setFamily( QgsProject::instance()->readEntry( "CopyrightLabel", "/FontName", "Sans Serif" ) );
+  //  mQFont.setPointSize( QgsProject::instance()->readNumEntry( "CopyrightLabel", "/FontSize", 9 ) );
   mLabelQString = QgsProject::instance()->readEntry( "CopyrightLabel", "/Label", defString );
   mPlacementIndex = QgsProject::instance()->readNumEntry( "CopyrightLabel", "/Placement", 3 );
   mEnable = QgsProject::instance()->readBoolEntry( "CopyrightLabel", "/Enabled", true );
@@ -159,7 +160,7 @@ void QgsCopyrightLabelPlugin::renderLabel( QPainter * theQPainter )
 
     QTextDocument text;
     text.setDefaultFont( mQFont );
-    // To set the text colour in a QTextDocument we use a CSS style
+    // To set the text color in a QTextDocument we use a CSS style
     QString style = "<style type=\"text/css\"> p {color: " +
                     mLabelQColor.name() + "}</style>";
     text.setHtml( style + "<p>" + mLabelQString + "</p>" );
@@ -171,7 +172,7 @@ void QgsCopyrightLabelPlugin::renderLabel( QPainter * theQPainter )
     {
       case 0: // Bottom Left
         //Define bottom left hand corner start point
-        myYOffset = myHeight - size.height() + 5;
+        myYOffset = myHeight - ( size.height() + 5 );
         myXOffset = 5;
         break;
       case 1: // Top left
@@ -186,7 +187,7 @@ void QgsCopyrightLabelPlugin::renderLabel( QPainter * theQPainter )
         break;
       case 3: // Bottom Right
         //Define bottom right hand corner start point
-        myYOffset = myHeight - size.height() + 5;
+        myYOffset = myHeight - ( size.height() + 5 );
         myXOffset = myWidth - ( size.width() + 5 );
         break;
       default:
@@ -233,7 +234,7 @@ void QgsCopyrightLabelPlugin::setLabel( QString theLabelQString )
   QgsProject::instance()->writeEntry( "CopyrightLabel", "/Label", mLabelQString );
   refreshCanvas();
 }
-//! change the copyright text colour
+//! change the copyright text color
 void QgsCopyrightLabelPlugin::setColor( QColor theQColor )
 {
   mLabelQColor = theQColor;
