@@ -11,7 +11,7 @@
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  ***************************************************************************/
-/* $Id: qgsdelimitedtextplugingui.cpp 10430 2009-03-27 19:12:40Z jef $ */
+/* $Id$ */
 #include "qgsdelimitedtextplugingui.h"
 #include "qgscontexthelp.h"
 
@@ -56,29 +56,12 @@ QgsDelimitedTextPluginGui::QgsDelimitedTextPluginGui( QgisInterface * _qI, QWidg
     delimiterRegexp->setChecked( true );
   }
 
-  QString format = QString( "<h2>%1</h2><p>%2</p><p>%3</p>" );
-  QString header = tr( "Description" );
-  QString paragraph1 = tr( "Select a delimited text file containing a header row"
-                           " and one or more rows of x and y coordinates that you"
-                           " would like to use as a point layer and this plugin will do the job for you!" );
-  QString paragraph2 = tr( "Use the layer name box to specify the legend name for the new layer."
-                           " Use the delimiter box to specify what delimeter is used in your file (e.g. space,"
-                           " comma, tab or a regular expression in Perl style). After choosing a delimiter,"
-                           " press the parse button and select the columns containing the x and y values for the layer." );
-  teInstructions->setHtml( format.arg( header )
-                           .arg( paragraph1 )
-                           .arg( paragraph2 ) );
-
   txtSample->setFixedHeight( 120 );
 }
 QgsDelimitedTextPluginGui::~QgsDelimitedTextPluginGui()
 {
 }
 /** Autoconnected slots **/
-void QgsDelimitedTextPluginGui::on_buttonBox_helpRequested()
-{
-  QgsContextHelp::run( context_id );
-}
 void QgsDelimitedTextPluginGui::on_btnBrowseForFile_clicked()
 {
   getOpenFileName();
@@ -252,11 +235,6 @@ void QgsDelimitedTextPluginGui::enableButtons()
   pbnParse->setEnabled( txtDelimiter->text().length() > 0 && txtFilePath->text().length() > 0 );
   pbnOK->setEnabled( txtDelimiter->text().length() > 0 && txtFilePath->text().length() > 0 );
 }
-void QgsDelimitedTextPluginGui::help()
-{
-  qI->openURL( "plugins/delimited_text/index.html", true );
-}
-
 void QgsDelimitedTextPluginGui::on_txtDelimiter_textChanged( const QString & text )
 {
   if ( !text.isEmpty() )
