@@ -540,7 +540,11 @@ void eVisGenericEventBrowserGui::displayImage( )
       //keep the extent the same just center the map canvas in the display so our feature is in the middle
       QgsRectangle myRect( myPoint.x( ) - ( mCanvas->extent( ).width( ) / 2 ), myPoint.y( ) - ( mCanvas->extent( ).height( ) / 2 ), myPoint.x( ) + ( mCanvas->extent( ).width( ) / 2 ), myPoint.y( ) + ( mCanvas->extent( ).height( ) / 2 ) );
 
-      mCanvas->setExtent( myRect );
+      // only change the extents if the point is beyond the current extents to minimise repaints
+      if ( !mCanvas->extent().contains( myPoint ) )
+      {
+        mCanvas->setExtent( myRect );
+      }
       mCanvas->refresh( );
     }
   }
