@@ -18,7 +18,7 @@ email                : sherman at mrcc.com
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-/*  $Id: qgspggeoprocessing.cpp 11241 2009-08-01 19:20:09Z jef $ */
+/*  $Id$ */
 
 // includes
 #include <vector>
@@ -40,7 +40,7 @@ email                : sherman at mrcc.com
 #include "qgslogger.h"
 
 
-static const char * const ident_ = "$Id: qgspggeoprocessing.cpp 11241 2009-08-01 19:20:09Z jef $";
+static const char * const ident_ = "$Id$";
 
 static const QString name_ = QObject::tr( "PostgreSQL Geoprocessing" );
 static const QString description_ = QObject::tr( "Geoprocessing functions for working with PostgreSQL/PostGIS layers" );
@@ -123,7 +123,7 @@ void QgsPgGeoprocessing::buffer()
         QString tableName = uri.quotedTablename();
 
         // set the fields on the dialog box drop-down
-        QgsVectorDataProvider *dp = dynamic_cast<QgsVectorDataProvider *>( lyr->dataProvider() );
+        QgsVectorDataProvider *dp = qobject_cast<QgsVectorDataProvider *>( lyr->dataProvider() );
         QgsFieldMap flds = dp->fields();
         for ( QgsFieldMap::iterator it = flds.begin(); it != flds.end(); ++it )
         {
