@@ -15,7 +15,7 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-/*  $Id: qgsbabelformat.cpp 9138 2008-08-23 21:37:31Z jef $ */
+/*  $Id$ */
 
 #include "qgsbabelformat.h"
 
@@ -147,9 +147,9 @@ QStringList QgsBabelCommand::importCommand( const QString& babel,
     else if ( *iter == "%type" )
       copy.append( featuretype );
     else if ( *iter == "%in" )
-      copy.append( input );
+      copy.append( QString( "\"%1\"" ).arg( input ) );
     else if ( *iter == "%out" )
-      copy.append( output );
+      copy.append( QString( "\"%1\"" ).arg( output ) );
     else
       copy.append( *iter );
   }
@@ -171,9 +171,9 @@ QStringList QgsBabelCommand::exportCommand( const QString& babel,
     else if ( *iter == "%type" )
       copy.append( featuretype );
     else if ( *iter == "%in" )
-      copy.append( input );
+      copy.append( QString( "\"%1\"" ).arg( input ) );
     else if ( *iter == "%out" )
-      copy.append( output );
+      copy.append( QString( "\"%1\"" ).arg( output ) );
     else
       copy.append( *iter );
   }
