@@ -15,7 +15,7 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-/*  $Id: qgsgpsplugin.cpp 10967 2009-06-23 12:03:18Z borysiasty $ */
+/*  $Id$ */
 
 // includes
 
@@ -49,7 +49,7 @@
 #include "qgslogger.h"
 
 
-static const char * const ident_ = "$Id: qgsgpsplugin.cpp 10967 2009-06-23 12:03:18Z borysiasty $";
+static const char * const ident_ = "$Id$";
 static const QString name_ = QObject::tr( "GPS Tools" );
 static const QString description_ = QObject::tr( "Tools for loading and importing GPS data" );
 static const QString version_ = QObject::tr( "Version 0.1" );
@@ -96,7 +96,7 @@ void QgsGPSPlugin::initGui()
   connect( mQActionPointer, SIGNAL( triggered() ), this, SLOT( run() ) );
   connect( mCreateGPXAction, SIGNAL( triggered() ), this, SLOT( createGPX() ) );
 
-  mQGisInterface->fileToolBar()->addAction( mQActionPointer );
+  mQGisInterface->layerToolBar()->addAction( mQActionPointer );
   mQGisInterface->addPluginToMenu( tr( "&Gps" ), mQActionPointer );
   mQGisInterface->addPluginToMenu( tr( "&Gps" ), mCreateGPXAction );
 
@@ -122,7 +122,7 @@ void QgsGPSPlugin::run()
   {
     if ( iter.value()->type() == QgsMapLayer::VectorLayer )
     {
-      QgsVectorLayer* vLayer = dynamic_cast<QgsVectorLayer*>( iter.value() );
+      QgsVectorLayer* vLayer = qobject_cast<QgsVectorLayer *>( iter.value() );
       if ( vLayer->providerType() == "gpx" )
         gpxLayers.push_back( vLayer );
     }
@@ -329,8 +329,8 @@ void QgsGPSPlugin::convertGPSFile( QString inputFileName,
 
   // try to start the gpsbabel process
   QStringList babelArgs;
-  babelArgs << mBabelPath << "-i" << "gpx" << "-f" << inputFileName
-  << convertStrings << "-o" << "gpx" << "-F" << outputFileName;
+  babelArgs << mBabelPath << "-i" << "gpx" << "-f" << QString( "\"%1\"" ).arg( inputFileName )
+  << convertStrings << "-o" << "gpx" << "-F" << QString( "\"%1\"" ).arg( outputFileName );
   QgsDebugMsg( QString( "Conversion command: " ) + babelArgs.join( "|" ) );
 
   QProcess babelProcess;
