@@ -13,7 +13,7 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-/*  $Id: qgsgpsplugingui.h 9138 2008-08-23 21:37:31Z jef $ */
+/*  $Id$ */
 
 #ifndef QGSGPSPLUGINGUI_H
 #define QGSGPSPLUGINGUI_H
@@ -22,6 +22,7 @@
 #include "ui_qgsgpspluginguibase.h"
 #include "qgsbabelformat.h"
 #include "qgsgpsdevice.h"
+#include "qgscontexthelp.h"
 
 #include <vector>
 
@@ -34,6 +35,7 @@
 class QgsGPSPluginGui : public QDialog, private Ui::QgsGPSPluginGuiBase
 {
     Q_OBJECT
+
   public:
     QgsGPSPluginGui( const BabelMap& importers,
                      std::map<QString, QgsGPSDevice*>& devices,
@@ -58,23 +60,24 @@ class QgsGPSPluginGui : public QDialog, private Ui::QgsGPSPluginGuiBase
     void on_pbnDLOutput_clicked();
 
   private:
-
     void populateDeviceComboBox();
     void populateULLayerComboBox();
     void populateIMPBabelFormats();
     void populatePortComboBoxes();
+#if 0
     void populateLoadDialog();
     void populateDLDialog();
     void populateULDialog();
     void populateIMPDialog();
     void populateCONVDialog();
+#endif
 
   private slots:
 
     void on_pbnRefresh_clicked();
     void on_buttonBox_accepted();
     void on_buttonBox_rejected();
-    void on_buttonBox_helpRequested();
+    void on_buttonBox_helpRequested() { QgsContextHelp::run( metaObject()->className() ); }
 
   signals:
     void drawRasterLayer( QString );
@@ -102,7 +105,6 @@ class QgsGPSPluginGui : public QDialog, private Ui::QgsGPSPluginGuiBase
     QString mBabelFilter;
     QString mImpFormat;
     QAbstractButton *pbnOK;
-    static const int context_id = 995980174;
 };
 
 #endif
