@@ -18,7 +18,7 @@ email                : tim@linfiniti.com
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-/*  $Id: plugin.cpp 10967 2009-06-23 12:03:18Z borysiasty $ */
+/*  $Id$ */
 
 // includes
 
@@ -54,7 +54,7 @@ email                : tim@linfiniti.com
 #endif
 
 //
-static const char * const ident_ = "$Id: plugin.cpp 10967 2009-06-23 12:03:18Z borysiasty $";
+static const char * const ident_ = "$Id$";
 
 static const QString name_ = QObject::tr( "NorthArrow" );
 static const QString description_ = QObject::tr( "Displays a north arrow overlayed onto the map" );
@@ -325,7 +325,7 @@ bool QgsNorthArrowPlugin::calculateNorthDirection()
         p1 = transform.transform( p1 );
         p2 = transform.transform( p2 );
       }
-      catch ( QgsException &e )
+      catch ( QgsCsException &e )
       {
         Q_UNUSED( e );
         // just give up
