@@ -14,7 +14,7 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-/* $Id: qgsspit.cpp 11514 2009-08-26 15:56:43Z jef $ */
+/* $Id$ */
 
 #include <QMessageBox>
 #include <QComboBox>
@@ -115,26 +115,30 @@ void QgsSpit::populateConnectionList()
     ++it;
   }
   settings.endGroup();
+
+  btnConnect->setDisabled( cmbConnections->count() == 0 );
+  btnEdit->setDisabled( cmbConnections->count() == 0 );
+  btnRemove->setDisabled( cmbConnections->count() == 0 );
+
+  cmbConnections->setDisabled( cmbConnections->count() == 0 );
 }
 
 void QgsSpit::newConnection()
 {
   QgsNewConnection *nc = new QgsNewConnection( this );
+  nc->exec();
+  delete nc;
 
-  if ( nc->exec() )
-  {
-    populateConnectionList();
-  }
+  populateConnectionList();
 }
 
 void QgsSpit::editConnection()
 {
   QgsNewConnection *nc = new QgsNewConnection( this, cmbConnections->currentText() );
+  nc->exec();
+  delete nc;
 
-  if ( nc->exec() )
-  {
-    nc->saveConnection();
-  }
+  populateConnectionList();
 }
 
 void QgsSpit::removeConnection()
@@ -143,17 +147,21 @@ void QgsSpit::removeConnection()
   QString key = "/PostgreSQL/connections/" + cmbConnections->currentText();
   QString msg = tr( "Are you sure you want to remove the [%1] connection and all associated settings?" ).arg( cmbConnections->currentText() );
   QMessageBox::StandardButton result = QMessageBox::information( this, tr( "Confirm Delete" ), msg, QMessageBox::Ok | QMessageBox::Cancel );
-  if ( result == QMessageBox::Ok )
-  {
-    settings.remove( key + "/host" );
-    settings.remove( key + "/database" );
-    settings.remove( key + "/port" );
-    settings.remove( key + "/username" );
-    settings.remove( key + "/password" );
-    settings.remove( key + "/save" );
+  if ( result != QMessageBox::Ok )
+    return;
 
-    cmbConnections->removeItem( cmbConnections->currentIndex() );
-  }
+  settings.remove( key + "/host" );
+  settings.remove( key + "/database" );
+  settings.remove( key + "/port" );
+  settings.remove( key + "/username" );
+  settings.remove( key + "/password" );
+  settings.remove( key + "/sslmode" );
+  settings.remove( key + "/publicOnly" );
+  settings.remove( key + "/geometryColumnsOnly" );
+  settings.remove( key + "/save" );
+  settings.remove( key );
+
+  populateConnectionList();
 }
 
 void QgsSpit::addFile()
@@ -467,8 +475,7 @@ void QgsSpit::dbConnect()
       }
     }
 
-    QString schemaSql = QString( "select nspname from pg_namespace where has_schema_privilege(nspname, 'CREATE')" )
-                        .arg( QgsPgUtil::quotedValue( username ) );
+    QString schemaSql = "select nspname from pg_namespace where has_schema_privilege(nspname, 'CREATE')";
     PGresult *schemas = PQexec( conn, schemaSql.toUtf8() );
     // get the schema names
     if ( PQresultStatus( schemas ) == PGRES_TUPLES_OK )
