@@ -51,7 +51,7 @@ void QgsWFSPlugin::initGui()
     mWfsDialogAction = new QAction( QIcon(), tr( "&Add WFS layer" ), 0 );
     setCurrentTheme( "" );
     QObject::connect( mWfsDialogAction, SIGNAL( triggered() ), this, SLOT( showSourceDialog() ) );
-    mIface->fileToolBar()->addAction( mWfsDialogAction );
+    mIface->layerToolBar()->addAction( mWfsDialogAction );
     mIface->addPluginToMenu( tr( "&Add WFS layer" ), mWfsDialogAction );
     // this is called when the icon theme is changed
     connect( mIface, SIGNAL( currentThemeChanged( QString ) ), this, SLOT( setCurrentTheme( QString ) ) );
