@@ -39,7 +39,6 @@ QgsWFSSourceSelect::QgsWFSSourceSelect( QWidget* parent, QgisInterface* iface ):
 
   connect( buttonBox, SIGNAL( accepted() ), this, SLOT( addLayer() ) );
   connect( buttonBox, SIGNAL( rejected() ), this, SLOT( reject() ) );
-  connect( buttonBox, SIGNAL( helpRequested() ), this, SLOT( showHelp() ) );
   connect( btnNew, SIGNAL( clicked() ), this, SLOT( addEntryToServerList() ) );
   connect( btnEdit, SIGNAL( clicked() ), this, SLOT( modifyEntryOfServerList() ) );
   connect( btnDelete, SIGNAL( clicked() ), this, SLOT( deleteEntryOfServerList() ) );
@@ -143,7 +142,10 @@ int QgsWFSSourceSelect::getCapabilitiesGET( QString uri, std::list<QString>& typ
 
   QByteArray result;
   QgsHttpTransaction http( request );
-  http.getSynchronously( result );
+  if ( !http.getSynchronously( result ) )
+  {
+    QMessageBox::critical( 0, tr( "Error" ), tr( "The capabilities document could not be retrieved from the server" ) );
+  }
 
   QDomDocument capabilitiesDocument;
   if ( !capabilitiesDocument.setContent( result, true ) )
@@ -415,8 +417,3 @@ void QgsWFSSourceSelect::changeCRSFilter()
     }
   }
 }
-
-void QgsWFSSourceSelect::showHelp()
-{
-  QgsContextHelp::run( context_id );
-}
