@@ -13,7 +13,7 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-/* $Id: qgsgrass.cpp 11570 2009-09-05 19:45:32Z jef $ */
+/* $Id$ */
 
 #include "qgsgrass.h"
 
@@ -335,9 +335,9 @@ void QgsGrass::setLocation( QString gisdbase, QString location )
 // #if defined(WIN32)
 //  G__setenv(( char * ) "GISDBASE", shortPath( gisdbase ).toLocal8Bit().data() );
 //#else
-  G__setenv(( char * ) "GISDBASE", gisdbase.toAscii().constData() );
+  G__setenv(( char * ) "GISDBASE", gisdbase.toUtf8().constData() );
 //#endif
-  G__setenv(( char * ) "LOCATION_NAME", location.toAscii().constData() );
+  G__setenv(( char * ) "LOCATION_NAME", location.toUtf8().constData() );
   G__setenv(( char * ) "MAPSET", ( char * ) "PERMANENT" ); // PERMANENT must always exist
 
   // Add all available mapsets to search path
@@ -542,7 +542,7 @@ QString GRASS_EXPORT QgsGrass::openMapset( QString gisdbase, QString location, Q
 
   out.close();
 
-  // Set GISRC enviroment variable
+  // Set GISRC environment variable
 
   /* _Correct_ putenv() implementation is not making copy! */
   QString gisrcEnv = "GISRC=" + mGisrc;
@@ -604,7 +604,7 @@ QString QgsGrass::closeMapset( )
 
     // Delete temporary dir
 
-    // To be sure that we dont delete '/' for example
+    // To be sure that we don't delete '/' for example
     if ( mTmp.left( 4 ) == "/tmp" )
     {
       QDir dir( mTmp );
