@@ -14,19 +14,17 @@ email                : sherman at mrcc.com
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-/* $Id: qgsogrprovider.cpp 11539 2009-09-02 19:52:16Z jef $ */
+/* $Id$ */
 
 #include "qgsogrprovider.h"
 #include "qgslogger.h"
 
-#include <iostream>
-#include <cassert>
-
 #define CPL_SUPRESS_CPLUSPLUS
 #include <gdal.h>         // to collect version information
 #include <ogr_api.h>
 #include <ogr_srs_api.h>
 #include <cpl_error.h>
+#include <cpl_conv.h>
 
 #include <QtDebug>
 #include <QFile>
@@ -58,7 +56,9 @@ QgsOgrProvider::QgsOgrProvider( QString const & uri )
     ogrDataSource( 0 ),
     extent_( 0 ),
     ogrLayer( 0 ),
-    ogrDriver( 0 )
+    ogrOrigLayer( 0 ),
+    ogrDriver( 0 ),
+    featuresCounted( -1 )
 {
   QgsApplication::registerOgrDrivers();
 
@@ -75,50 +75,53 @@ QgsOgrProvider::QgsOgrProvider( QString const & uri )
   // This part of the code parses the uri transmitted to the ogr provider to
   // get the options the client wants us to apply
 
-  QString mFilePath;
-  QString theLayerName;
-  int theLayerIndex = 0;
-
   // If there is no & in the uri, then the uri is just the filename. The loaded
   // layer will be layer 0.
   //this is not true for geojson
-  if ( ! uri.contains( '|', Qt::CaseSensitive ) )
+  if ( !uri.contains( '|', Qt::CaseSensitive ) )
   {
     mFilePath = uri;
+    mLayerIndex = 0;
+    mLayerName = QString::null;
   }
   else
   {
-    // If we get here, there are some options added to the filename. We must parse
-    // the different parts separated by &, and among each option, the name and the
-    // value around the =.
-    // A valid uri is of the form: filename&option1=value1&option2=value2,...
-
     QStringList theURIParts = uri.split( "|" );
     mFilePath = theURIParts.at( 0 );
 
     for ( int i = 1 ; i < theURIParts.size(); i++ )
     {
-      QStringList theInstruction = theURIParts.at( i ).split( "=" );
-      if ( theInstruction.at( 0 ) == QString( "layerid" ) )
+      QString part = theURIParts.at( i );
+      int pos = part.indexOf( "=" );
+      QString field = part.left( pos );
+      QString value = part.mid( pos + 1 );
+
+      if ( field == "layerid" )
       {
         bool ok;
-        theLayerIndex = theInstruction.at( 1 ).toInt( &ok );
+        mLayerIndex = value.toInt( &ok );
         if ( ! ok )
         {
-          theLayerIndex = 0;
+          mLayerIndex = -1;
         }
       }
-      if ( theInstruction.at( 0 ) == QString( "layername" ) )
+      else if ( field == "layername" )
       {
-        theLayerName = theInstruction.at( 1 );
+        mLayerName = value;
+      }
+
+      if ( field == "subset" )
+      {
+        mSubsetString = value;
       }
     }
   }
 
   QgsDebugMsg( "mFilePath: " + mFilePath );
-  QgsDebugMsg( "theLayerIndex: " + theLayerIndex );
-  QgsDebugMsg( "theLayerName: " + theLayerName );
-
+  QgsDebugMsg( "mLayerIndex: " + QString::number( mLayerIndex ) );
+  QgsDebugMsg( "mLayerName: " + mLayerName );
+  QgsDebugMsg( "mSubsetString: " + mSubsetString );
+  CPLSetConfigOption( "OGR_ORGANIZE_POLYGONS", "ONLY_CCW" );  // "SKIP" returns MULTIPOLYGONs for multiringed POLYGONs
   CPLPushErrorHandler( CPLQuietErrorHandler );
   ogrDataSource = OGROpen( QFile::encodeName( mFilePath ).constData(), TRUE, &ogrDriver );
   CPLPopErrorHandler();
@@ -144,53 +147,42 @@ QgsOgrProvider::QgsOgrProvider( QString const & uri )
 
     // We get the layer which was requested by the uri. The layername
     // has precedence over the layerid if both are given.
-    if ( theLayerName.isNull() )
+    if ( mLayerName.isNull() )
     {
-      ogrLayer = OGR_DS_GetLayer( ogrDataSource, theLayerIndex );
+      ogrOrigLayer = OGR_DS_GetLayer( ogrDataSource, mLayerIndex );
     }
     else
     {
-      ogrLayer = OGR_DS_GetLayerByName( ogrDataSource, ( char* )( theLayerName.toLocal8Bit().data() ) );
+      ogrOrigLayer = OGR_DS_GetLayerByName( ogrDataSource, mLayerName.toLocal8Bit().data() );
     }
 
-    // get the extent_ (envelope) of the layer
-
-    QgsDebugMsg( "Starting get extent" );
-
-    // TODO: This can be expensive, do we really need it!
-
     extent_ = calloc( sizeof( OGREnvelope ), 1 );
-    OGR_L_GetExtent( ogrLayer, ( OGREnvelope * ) extent_, TRUE );
 
-    QgsDebugMsg( "Finished get extent" );
-
-    // getting the total number of features in the layer
-    // TODO: This can be expensive, do we really need it!
-    featuresCounted = OGR_L_GetFeatureCount( ogrLayer, TRUE );
-
-    // check the validity of the layer
-
-    QgsDebugMsg( "checking validity" );
-    loadFields();
-    QgsDebugMsg( "Done checking validity" );
+    ogrLayer = ogrOrigLayer;
+    setSubsetString( mSubsetString );
   }
   else
   {
     QgsLogger::critical( "Data source is invalid" );
-    const char *er = CPLGetLastErrorMsg();
-    QgsLogger::critical( er );
+    QgsLogger::critical( QString::fromUtf8( CPLGetLastErrorMsg() ) );
     valid = false;
   }
 
+  // FIXME: sync with app/qgsnewvectorlayerdialog.cpp
   mNativeTypes
-  << QgsVectorDataProvider::NativeType( tr( "Integer" ), "integer", QVariant::Int, 1, 10 )
-  << QgsVectorDataProvider::NativeType( tr( "Real" ), "double", QVariant::Double, 1, 20, 0, 5 )
-  << QgsVectorDataProvider::NativeType( tr( "String" ), "string", QVariant::String, 1, 255, 0, 0 )
+  << QgsVectorDataProvider::NativeType( tr( "Whole number (integer)" ), "integer", QVariant::Int, 1, 10 )
+  << QgsVectorDataProvider::NativeType( tr( "Decimal number (real)" ), "double", QVariant::Double, 1, 20, 0, 5 )
+  << QgsVectorDataProvider::NativeType( tr( "Text (string)" ), "string", QVariant::String, 1, 255 )
   ;
 }
 
 QgsOgrProvider::~QgsOgrProvider()
 {
+  if ( ogrLayer != ogrOrigLayer )
+  {
+    OGR_DS_ReleaseResultSet( ogrDataSource, ogrLayer );
+  }
+
   OGR_DS_Destroy( ogrDataSource );
   ogrDataSource = 0;
   free( extent_ );
@@ -202,6 +194,83 @@ QgsOgrProvider::~QgsOgrProvider()
   }
 }
 
+bool QgsOgrProvider::setSubsetString( QString theSQL )
+{
+  if ( theSQL == mSubsetString && featuresCounted >= 0 )
+    return true;
+
+  OGRLayerH prevLayer = ogrLayer;
+  QString prevSubsetString = mSubsetString;
+  mSubsetString = theSQL;
+
+  if ( !mSubsetString.isEmpty() )
+  {
+    QString sql = QString( "SELECT * FROM %1 WHERE %2" )
+                  .arg( quotedIdentifier( OGR_FD_GetName( OGR_L_GetLayerDefn( ogrOrigLayer ) ) ) )
+                  .arg( mSubsetString );
+    ogrLayer = OGR_DS_ExecuteSQL( ogrDataSource, mEncoding->fromUnicode( sql ).constData(), NULL, NULL );
+
+    if ( !ogrLayer )
+    {
+      ogrLayer = prevLayer;
+      mSubsetString = prevSubsetString;
+      return false;
+    }
+  }
+  else
+  {
+    ogrLayer = ogrOrigLayer;
+  }
+
+  if ( prevLayer != ogrOrigLayer )
+  {
+    OGR_DS_ReleaseResultSet( ogrDataSource, prevLayer );
+  }
+
+  QString uri = mFilePath;
+  if ( !mLayerName.isNull() )
+  {
+    uri += QString( "|layername=%1" ).arg( mLayerName );
+  }
+  else if ( mLayerIndex >= 0 )
+  {
+    uri += QString( "|layerid=%1" ).arg( mLayerIndex );
+  }
+
+  if ( !mSubsetString.isEmpty() )
+  {
+    uri += QString( "|subset=%1" ).arg( mSubsetString );
+  }
+
+  setDataSourceUri( uri );
+
+  OGR_L_ResetReading( ogrLayer );
+
+  // getting the total number of features in the layer
+  // TODO: This can be expensive, do we really need it!
+  featuresCounted = OGR_L_GetFeatureCount( ogrLayer, TRUE );
+
+  // get the extent_ (envelope) of the layer
+  QgsDebugMsg( "Starting get extent" );
+
+  // TODO: This can be expensive, do we really need it!
+  OGR_L_GetExtent( ogrLayer, ( OGREnvelope * ) extent_, TRUE );
+
+  QgsDebugMsg( "Finished get extent" );
+
+  // check the validity of the layer
+  QgsDebugMsg( "checking validity" );
+  loadFields();
+  QgsDebugMsg( "Done checking validity" );
+
+  return true;
+}
+
+QString QgsOgrProvider::subsetString()
+{
+  return mSubsetString;
+}
+
 QStringList QgsOgrProvider::subLayers() const
 {
   QStringList theList = QStringList();
@@ -866,19 +935,15 @@ bool QgsOgrProvider::changeGeometryValues( QgsGeometryMap & geometry_map )
 
 bool QgsOgrProvider::createSpatialIndex()
 {
-  QFileInfo fi( dataSourceUri() );     // to get the base name
-  QString sql = QString( "CREATE SPATIAL INDEX ON %1" ).arg( quotedIdentifier( fi.completeBaseName() ) );  // quote the layer name so spaces are handled
-  OGR_DS_ExecuteSQL( ogrDataSource, mEncoding->fromUnicode( sql ).data(), OGR_L_GetSpatialFilter( ogrLayer ), "" );
+  QString layerName = OGR_FD_GetName( OGR_L_GetLayerDefn( ogrOrigLayer ) );
+
+  QString sql = QString( "CREATE SPATIAL INDEX ON %1" ).arg( quotedIdentifier( layerName ) );  // quote the layer name so spaces are handled
+  OGR_DS_ExecuteSQL( ogrDataSource, mEncoding->fromUnicode( sql ).data(), OGR_L_GetSpatialFilter( ogrOrigLayer ), "" );
+
+  QFileInfo fi( mFilePath );     // to get the base name
   //find out, if the .qix file is there
   QFile indexfile( fi.path().append( "/" ).append( fi.completeBaseName() ).append( ".qix" ) );
-  if ( indexfile.exists() )
-  {
-    return true;
-  }
-  else
-  {
-    return false;
-  }
+  return indexfile.exists();
 }
 
 bool QgsOgrProvider::deleteFeatures( const QgsFeatureIds & id )
@@ -897,8 +962,10 @@ bool QgsOgrProvider::deleteFeatures( const QgsFeatureIds & id )
     returnvalue = false;
   }
 
+  QString layerName = OGR_FD_GetName( OGR_L_GetLayerDefn( ogrOrigLayer ) );
+
   QFileInfo fi( dataSourceUri() );     // to get the base name
-  QString sql = QString( "REPACK %1" ).arg( fi.completeBaseName() );   // don't quote the layer name as it works with spaces in the name and won't work if the name is quoted
+  QString sql = QString( "REPACK %1" ).arg( layerName );   // don't quote the layer name as it works with spaces in the name and won't work if the name is quoted
   OGR_DS_ExecuteSQL( ogrDataSource, mEncoding->fromUnicode( sql ).data(), NULL, NULL );
   featuresCounted = OGR_L_GetFeatureCount( ogrLayer, TRUE ); //new feature count
   return returnvalue;
@@ -957,6 +1024,7 @@ int QgsOgrProvider::capabilities() const
       ability |= ChangeGeometries;
     }
 
+#if 0
     if ( OGR_L_TestCapability( ogrLayer, "FastSpatialFilter" ) )
       // TRUE if this layer implements spatial filtering efficiently.
       // Layers that effectively read all features, and test them with the
@@ -990,37 +1058,35 @@ int QgsOgrProvider::capabilities() const
     {
       // No use required for this QGIS release.
     }
-
-    if ( 1 )
-    {
-      // Ideally this should test for Shapefile type and GDAL >= 1.2.6
-      // In reality, createSpatialIndex() looks after itself.
-      ability |= CreateSpatialIndex;
-    }
-
-#if defined(GDAL_VERSION_NUM) && GDAL_VERSION_NUM >= 1600
-    // adding attributes was added in GDAL 1.6
-    if ( ogrDriverName.startsWith( "ESRI" ) )
-    {
-      ability |= AddAttributes;
-    }
 #endif
 
     // OGR doesn't handle shapefiles without attributes, ie. missing DBFs well, fixes #803
-    if ( ogrDriverName.startsWith( "ESRI" ) && mAttributeFields.size() == 0 )
+    if ( ogrDriverName == "ESRI Shapefile" )
     {
-      QgsDebugMsg( "OGR doesn't handle shapefile without attributes well, ie. missing DBFs" );
-      ability &= ~( AddFeatures | DeleteFeatures | ChangeAttributeValues | AddAttributes | DeleteAttributes );
+#if defined(GDAL_VERSION_NUM) && GDAL_VERSION_NUM >= 1260
+      // test for Shapefile type and GDAL >= 1.2.6
+      ability |= CreateSpatialIndex;
+#endif
+#if defined(GDAL_VERSION_NUM) && GDAL_VERSION_NUM >= 1600
+      // adding attributes was added in GDAL 1.6
+      ability |= AddAttributes;
+#endif
+
+      if ( mAttributeFields.size() == 0 )
+      {
+        QgsDebugMsg( "OGR doesn't handle shapefile without attributes well, ie. missing DBFs" );
+        ability &= ~( AddFeatures | DeleteFeatures | ChangeAttributeValues | AddAttributes | DeleteAttributes );
+      }
+
+      if (( ability & ChangeAttributeValues ) == 0 )
+      {
+        // on readonly shapes OGR reports that it can delete features although it can't RandomWrite
+        ability &= ~( AddAttributes | DeleteFeatures );
+      }
     }
   }
 
   return ability;
-
-  /*
-  return (QgsVectorDataProvider::AddFeatures
-  | QgsVectorDataProvider::ChangeAttributeValues
-  | QgsVectorDataProvider::CreateSpatialIndex);
-  */
 }
 
 
@@ -1260,7 +1326,7 @@ QString createFilters( QString type )
       {
         // NOP, we don't know anything about the current driver
         // with regards to a proper file filter string
-        QgsLogger::debug( "fileVectorFilters, unknown driver: " + driverName );
+        QgsDebugMsg( "fileVectorFilters, unknown driver: " + driverName );
       }
 
     }                           // each loaded GDAL driver
@@ -1377,12 +1443,15 @@ QGISEXTERN bool isProvider()
 @param vectortype point/line/polygon or multitypes
 @param attributes a list of name/type pairs for the initial attributes
 @return true in case of success*/
-QGISEXTERN bool createEmptyDataSource( const QString& uri,
-                                       const QString& format,
-                                       const QString& encoding,
+QGISEXTERN bool createEmptyDataSource( const QString &uri,
+                                       const QString &format,
+                                       const QString &encoding,
                                        QGis::WkbType vectortype,
-                                       const std::list<std::pair<QString, QString> >& attributes )
+                                       const std::list<std::pair<QString, QString> > &attributes,
+                                       const QgsCoordinateReferenceSystem *srs = NULL )
 {
+  QgsDebugMsg( QString( "Creating empty vector layer with format: %1" ).arg( format ) );
+
   OGRSFDriverH driver;
   QgsApplication::registerOgrDrivers();
   driver = OGRGetDriverByName( format.toAscii() );
@@ -1391,6 +1460,30 @@ QGISEXTERN bool createEmptyDataSource( const QString& uri,
     return false;
   }
 
+  QString driverName = OGR_Dr_GetName( driver );
+
+  if ( driverName == "ESRI Shapefile" )
+  {
+    if ( !uri.endsWith( ".shp", Qt::CaseInsensitive ) )
+    {
+      return false;
+    }
+
+    // check for duplicate fieldnames
+    QSet<QString> fieldNames;
+    std::list<std::pair<QString, QString> >::const_iterator fldIt;
+    for ( fldIt = attributes.begin(); fldIt != attributes.end(); ++fldIt )
+    {
+      QString name = fldIt->first.left( 10 );
+      if ( fieldNames.contains( name ) )
+      {
+        QgsDebugMsg( QString( "duplicate field (10 significant characters): %1" ).arg( name ) );
+        return false;
+      }
+      fieldNames << name;
+    }
+  }
+
   OGRDataSourceH dataSource;
   dataSource = OGR_Dr_CreateDataSource( driver, QFile::encodeName( uri ).constData(), NULL );
   if ( dataSource == NULL )
@@ -1400,8 +1493,17 @@ QGISEXTERN bool createEmptyDataSource( const QString& uri,
 
   //consider spatial reference system
   OGRSpatialReferenceH reference = NULL;
+
   QgsCoordinateReferenceSystem mySpatialRefSys;
-  mySpatialRefSys.validate();
+  if ( srs )
+  {
+    mySpatialRefSys = *srs;
+  }
+  else
+  {
+    mySpatialRefSys.validate();
+  }
+
   QString myWkt = mySpatialRefSys.toWkt();
 
   if ( !myWkt.isNull()  &&  myWkt.length() != 0 )
@@ -1433,8 +1535,7 @@ QGISEXTERN bool createEmptyDataSource( const QString& uri,
       break;
     default:
     {
-      QgsLogger::debug( "Unknown vector type of: ", ( int )( vectortype ), 1,
-                        __FILE__, __FUNCTION__, __LINE__ );
+      QgsDebugMsg( QString( "Unknown vector type of: %1" ).arg(( int )( vectortype ) ) );
       return false;
       break;
     }
@@ -1503,6 +1604,22 @@ QGISEXTERN bool createEmptyDataSource( const QString& uri,
 
   OGR_DS_Destroy( dataSource );
 
+  if ( driverName == "ESRI Shapefile" )
+  {
+    QString layerName = uri.left( uri.indexOf( ".shp", Qt::CaseInsensitive ) );
+    QFile prjFile( layerName + ".qpj" );
+    if ( prjFile.open( QIODevice::WriteOnly ) )
+    {
+      QTextStream prjStream( &prjFile );
+      prjStream << myWkt.toLocal8Bit().data() << endl;
+      prjFile.close();
+    }
+    else
+    {
+      QgsDebugMsg( "Couldn't open file " + layerName + ".qpj" );
+    }
+  }
+
   QgsDebugMsg( QString( "GDAL Version number %1" ).arg( GDAL_VERSION_NUM ) );
 #if defined(GDAL_VERSION_NUM) && GDAL_VERSION_NUM >= 1310
   if ( reference )
@@ -1519,6 +1636,29 @@ QgsCoordinateReferenceSystem QgsOgrProvider::crs()
 
   QgsCoordinateReferenceSystem srs;
 
+  if ( ogrDriver )
+  {
+    QString driverName = OGR_Dr_GetName( ogrDriver );
+
+    if ( driverName == "ESRI Shapefile" )
+    {
+      QString layerName = mFilePath.left( mFilePath.indexOf( ".shp", Qt::CaseInsensitive ) );
+      QFile prjFile( layerName + ".qpj" );
+      if ( prjFile.open( QIODevice::ReadOnly ) )
+      {
+        QTextStream prjStream( &prjFile );
+        QString myWktString = prjStream.readLine();
+        prjFile.close();
+
+        // create CRS from Wkt
+        srs.createFromWkt( myWktString );
+
+        if ( srs.isValid() )
+          return srs;
+      }
+    }
+  }
+
   OGRSpatialReferenceH mySpatialRefSys = OGR_L_GetSpatialRef( ogrLayer );
   if ( mySpatialRefSys == NULL )
   {
@@ -1527,10 +1667,10 @@ QgsCoordinateReferenceSystem QgsOgrProvider::crs()
   else
   {
     // get the proj4 text
-    char * ppszProj4;
+    char *ppszProj4;
     OSRExportToProj4( mySpatialRefSys, &ppszProj4 );
     QgsDebugMsg( ppszProj4 );
-    char    *pszWkt = NULL;
+    char *pszWkt = NULL;
     OSRExportToWkt( mySpatialRefSys, &pszWkt );
     QString myWktString = QString( pszWkt );
     OGRFree( pszWkt );
@@ -1542,27 +1682,37 @@ QgsCoordinateReferenceSystem QgsOgrProvider::crs()
   return srs;
 }
 
-void QgsOgrProvider::uniqueValues( int index, QList<QVariant> &uniqueValues )
+void QgsOgrProvider::uniqueValues( int index, QList<QVariant> &uniqueValues, int limit )
 {
   QgsField fld = mAttributeFields[index];
   QString theLayerName = OGR_FD_GetName( OGR_L_GetLayerDefn( ogrLayer ) );
 
-  QString sql = QString( "SELECT DISTINCT %1 FROM %2 ORDER BY %1" )
+  QString sql = QString( "SELECT DISTINCT %1 FROM %2" )
                 .arg( quotedIdentifier( fld.name() ) )
                 .arg( quotedIdentifier( theLayerName ) );
 
+  if ( !mSubsetString.isEmpty() )
+  {
+    sql += QString( " WHERE %1" ).arg( mSubsetString );
+  }
+
+  sql += QString( " ORDER BY %1" ).arg( fld.name() );
+
   uniqueValues.clear();
 
   QgsDebugMsg( QString( "SQL: %1" ).arg( sql ) );
   OGRLayerH l = OGR_DS_ExecuteSQL( ogrDataSource, mEncoding->fromUnicode( sql ).data(), NULL, "SQL" );
   if ( l == 0 )
-    return QgsVectorDataProvider::uniqueValues( index, uniqueValues );
+    return QgsVectorDataProvider::uniqueValues( index, uniqueValues, limit );
 
   OGRFeatureH f;
   while ( 0 != ( f = OGR_L_GetNextFeature( l ) ) )
   {
     uniqueValues << convertValue( fld.type(), mEncoding->toUnicode( OGR_F_GetFieldAsString( f, 0 ) ) );
     OGR_F_Destroy( f );
+
+    if ( limit >= 0 && uniqueValues.size() >= limit )
+      break;
   }
 
   OGR_DS_ReleaseResultSet( ogrDataSource, l );
@@ -1577,6 +1727,11 @@ QVariant QgsOgrProvider::minimumValue( int index )
                 .arg( quotedIdentifier( fld.name() ) )
                 .arg( quotedIdentifier( theLayerName ) );
 
+  if ( !mSubsetString.isEmpty() )
+  {
+    sql += QString( " WHERE %1" ).arg( mSubsetString );
+  }
+
   OGRLayerH l = OGR_DS_ExecuteSQL( ogrDataSource, mEncoding->fromUnicode( sql ).data(), NULL, "SQL" );
 
   if ( l == 0 )
@@ -1606,6 +1761,11 @@ QVariant QgsOgrProvider::maximumValue( int index )
                 .arg( quotedIdentifier( fld.name() ) )
                 .arg( quotedIdentifier( theLayerName ) );
 
+  if ( !mSubsetString.isEmpty() )
+  {
+    sql += QString( " WHERE %1" ).arg( mSubsetString );
+  }
+
   OGRLayerH l = OGR_DS_ExecuteSQL( ogrDataSource, mEncoding->fromUnicode( sql ).data(), NULL, "SQL" );
   if ( l == 0 )
     return QgsVectorDataProvider::maximumValue( index );
@@ -1638,22 +1798,25 @@ bool QgsOgrProvider::syncToDisc()
   OGR_L_SyncToDisk( ogrLayer );
 
   //for shapefiles: is there already a spatial index?
-  QFileInfo fi( dataSourceUri() );
-  QString filePath = fi.filePath();
-
-  //remove the suffix and add .qix
-  int suffixLength = fi.suffix().length();
-  if ( suffixLength > 0 )
+  if ( !mFilePath.isEmpty() )
   {
-    QString indexFilePath = filePath;
-    indexFilePath.chop( suffixLength );
-    indexFilePath.append( "qix" );
-    QFile indexFile( indexFilePath );
-    if ( indexFile.exists() ) //there is already a spatial index file
+    QFileInfo fi( mFilePath );
+
+    //remove the suffix and add .qix
+    int suffixLength = fi.suffix().length();
+    if ( suffixLength > 0 )
     {
-      //the already existing spatial index is removed automatically by OGR
-      return createSpatialIndex();
+      QString indexFilePath = mFilePath;
+      indexFilePath.chop( suffixLength );
+      indexFilePath.append( "qix" );
+      QFile indexFile( indexFilePath );
+      if ( indexFile.exists() ) //there is already a spatial index file
+      {
+        //the already existing spatial index is removed automatically by OGR
+        return createSpatialIndex();
+      }
     }
   }
+
   return true;
 }
