@@ -14,7 +14,7 @@ email                : sherman at mrcc.com
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-/* $Id: qgsogrprovider.h 11371 2009-08-14 09:20:23Z jef $ */
+/* $Id$ */
 
 #include "qgsrectangle.h"
 #include "qgsvectordataprovider.h"
@@ -92,6 +92,14 @@ class QgsOgrProvider : public QgsVectorDataProvider
                               bool fetchGeometry = true,
                               QgsAttributeList fetchAttributes = QgsAttributeList() );
 
+    /** Accessor for sql where clause used to limit dataset */
+    virtual QString subsetString();
+
+    virtual bool supportsSubsetString() { return true; }
+
+    /** mutator for sql where clause used to limit dataset size */
+    virtual bool setSubsetString( QString theSQL );
+
     /**
      * Get feature type.
      * @return int representing the feature type
@@ -193,7 +201,7 @@ class QgsOgrProvider : public QgsVectorDataProvider
     /** Return the unique values of an attribute
      *  @param index the index of the attribute
      *  @param values reference to the list of unique values */
-    virtual void uniqueValues( int index, QList<QVariant> &uniqueValues );
+    virtual void uniqueValues( int index, QList<QVariant> &uniqueValues, int limit = -1 );
 
     /** return a provider name
 
@@ -225,6 +233,11 @@ class QgsOgrProvider : public QgsVectorDataProvider
      */
     QString description() const;
 
+    /** Returns true if the provider is strict about the type of inserted features
+          (e.g. no multipolygon in a polygon layer)
+          @note: added in version 1.4*/
+    virtual bool doesStrictFeatureTypeCheck() const { return false;}
+
   protected:
     /** loads fields from input file to member attributeFields */
     void loadFields();
@@ -244,6 +257,19 @@ class QgsOgrProvider : public QgsVectorDataProvider
      in the method QgsOgrProvider::extent(). The purpose is to prevent a memory leak*/
     QgsRectangle mExtentRect;
     OGRLayerH ogrLayer;
+    OGRLayerH ogrOrigLayer;
+
+    //! path to filename
+    QString mFilePath;
+
+    //! layer name
+    QString mLayerName;
+
+    //! layer index
+    int mLayerIndex;
+
+    //! String used to define a subset of the layer
+    QString mSubsetString;
 
     // OGR Driver that was actually used to open the layer
     OGRSFDriverH ogrDriver;
