@@ -15,7 +15,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: qgspostgresprovider.cpp 11464 2009-08-21 13:32:42Z jef $ */
+/* $Id$ */
 
 // for htonl
 #ifdef WIN32
@@ -24,8 +24,6 @@
 #include <netinet/in.h>
 #endif
 
-#include <cassert>
-
 #include <qgis.h>
 #include <qgsapplication.h>
 #include <qgsfeature.h>
@@ -57,7 +55,8 @@ QgsPostgresProvider::QgsPostgresProvider( QString const & uri )
     : QgsVectorDataProvider( uri ),
     mFetching( false ),
     geomType( QGis::WKBUnknown ),
-    mFeatureQueueSize( 200 )
+    mFeatureQueueSize( 200 ),
+    mPrimaryKeyDefault( QString::null )
 {
   // assume this is a valid layer until we determine otherwise
   valid = true;
@@ -104,9 +103,10 @@ QgsPostgresProvider::QgsPostgresProvider( QString const & uri )
   if ( PQresultStatus( testAccess ) != PGRES_TUPLES_OK )
   {
     showMessageBox( tr( "Unable to access relation" ),
-                    tr( "Unable to access the %1 relation.\n" ).arg( mSchemaTableName )
-                    + tr( "The error message from the database was:\n%1.\nSQL: %2" )
-                    .arg( QString::fromUtf8( PQresultErrorMessage( testAccess ) ).arg( sql ) ) );
+                    tr( "Unable to access the %1 relation.\nThe error message from the database was:\n%2.\nSQL: %3" )
+                    .arg( mSchemaTableName )
+                    .arg( QString::fromUtf8( PQresultErrorMessage( testAccess ) ) )
+                    .arg( sql ) );
     valid = false;
     disconnectDb();
     return;
@@ -123,9 +123,10 @@ QgsPostgresProvider::QgsPostgresProvider( QString const & uri )
   if ( PQresultStatus( testAccess ) != PGRES_TUPLES_OK )
   {
     showMessageBox( tr( "Unable to access relation" ),
-                    tr( "Unable to determine table access privileges for the %1 relation.\n" ).arg( mSchemaTableName )
-                    + tr( "The error message from the database was:\n%1.\nSQL: %2" )
-                    .arg( QString::fromUtf8( PQresultErrorMessage( testAccess ) ) ).arg( sql ) );
+                    tr( "Unable to determine table access privileges for the %1 relation.\nThe error message from the database was:\n%2.\nSQL: %3" )
+                    .arg( mSchemaTableName )
+                    .arg( QString::fromUtf8( PQresultErrorMessage( testAccess ) ) )
+                    .arg( sql ) );
     valid = false;
     disconnectDb();
     return;
@@ -157,8 +158,8 @@ QgsPostgresProvider::QgsPostgresProvider( QString const & uri )
   if ( mCurrentSchema == mSchemaName )
   {
     mUri.clearSchema();
-    setDataSourceUri( mUri.uri() );
   }
+
   if ( mSchemaName == "" )
     mSchemaName = mCurrentSchema;
 
@@ -189,12 +190,12 @@ QgsPostgresProvider::QgsPostgresProvider( QString const & uri )
   calculateExtents();
   getFeatureCount();
 
-  // load the field list
-  loadFields();
-
   // set the primary key
   getPrimaryKey();
 
+  // load the field list
+  loadFields();
+
   // Set the postgresql message level so that we don't get the
   // 'there is no transaction in progress' warning.
 #ifndef QGISDEBUG
@@ -228,26 +229,31 @@ QgsPostgresProvider::QgsPostgresProvider( QString const & uri )
   //fill type names into sets
   mNativeTypes
   // integer types
-  << QgsVectorDataProvider::NativeType( tr( "smallint (16bit)" ), "int2", QVariant::Int )
-  << QgsVectorDataProvider::NativeType( tr( "integer (32bit)" ), "int4", QVariant::Int )
-  << QgsVectorDataProvider::NativeType( tr( "integer (64bit)" ), "int8", QVariant::LongLong )
-  << QgsVectorDataProvider::NativeType( tr( "numeric" ), "numeric", QVariant::LongLong, 1, 20, 0, 20 )
-  << QgsVectorDataProvider::NativeType( tr( "decimal" ), "decimal", QVariant::LongLong, 1, 20, 0, 20 )
+  << QgsVectorDataProvider::NativeType( tr( "Whole number (smallint - 16bit)" ), "int2", QVariant::Int )
+  << QgsVectorDataProvider::NativeType( tr( "Whole number (integer - 32bit)" ), "int4", QVariant::Int )
+  << QgsVectorDataProvider::NativeType( tr( "Whole number (integer - 64bit)" ), "int8", QVariant::LongLong )
+  << QgsVectorDataProvider::NativeType( tr( "Decimal number (numeric)" ), "numeric", QVariant::LongLong, 1, 20, 0, 20 )
+  << QgsVectorDataProvider::NativeType( tr( "Decimal number (decimal)" ), "decimal", QVariant::LongLong, 1, 20, 0, 20 )
 
   // floating point
-  << QgsVectorDataProvider::NativeType( tr( "real" ), "real", QVariant::Double )
-  << QgsVectorDataProvider::NativeType( tr( "double" ), "double precision", QVariant::Double )
+  << QgsVectorDataProvider::NativeType( tr( "Decimal number (real)" ), "real", QVariant::Double )
+  << QgsVectorDataProvider::NativeType( tr( "Decimal number (double)" ), "double precision", QVariant::Double )
 
   // string types
-  << QgsVectorDataProvider::NativeType( tr( "char" ), "char", QVariant::String, 1, 255 )
-  << QgsVectorDataProvider::NativeType( tr( "varchar" ), "varchar", QVariant::String, 1, 255 )
-  << QgsVectorDataProvider::NativeType( tr( "text" ), "text", QVariant::String )
+  << QgsVectorDataProvider::NativeType( tr( "Text, fixed length (char)" ), "char", QVariant::String, 1, 255 )
+  << QgsVectorDataProvider::NativeType( tr( "Text, limited variable length (varchar)" ), "varchar", QVariant::String, 1, 255 )
+  << QgsVectorDataProvider::NativeType( tr( "Text, unlimited length (text)" ), "text", QVariant::String )
   ;
 
   if ( primaryKey.isEmpty() )
   {
     valid = false;
   }
+  else
+  {
+    mUri.setKeyColumn( primaryKey );
+    setDataSourceUri( mUri.uri() );
+  }
 
   // Close the database connection if the layer isn't going to be loaded.
   if ( !valid )
@@ -409,6 +415,31 @@ QString QgsPostgresProvider::storageType() const
   return "PostgreSQL database with PostGIS extension";
 }
 
+QString QgsPostgresProvider::fieldExpression( const QgsField &fld ) const
+{
+  const QString &type = fld.typeName();
+  if ( type == "money" )
+  {
+    return QString( "cash_out(%1)" ).arg( quotedIdentifier( fld.name() ) );
+  }
+  else if ( type.startsWith( "_" ) )
+  {
+    return QString( "array_out(%1)" ).arg( quotedIdentifier( fld.name() ) );
+  }
+  else if ( type == "bool" )
+  {
+    return QString( "boolout(%1)" ).arg( quotedIdentifier( fld.name() ) );
+  }
+  else if ( type == "geometry" )
+  {
+    return QString( "asewkt(%1)" ).arg( quotedIdentifier( fld.name() ) );
+  }
+  else
+  {
+    return quotedIdentifier( fld.name() ) + "::text";
+  }
+}
+
 bool QgsPostgresProvider::declareCursor(
   const QString &cursorName,
   const QgsAttributeList &fetchAttributes,
@@ -430,31 +461,10 @@ bool QgsPostgresProvider::declareCursor(
     {
       const QgsField &fld = field( *it );
 
-      const QString &fieldname = fld.name();
-      if ( fieldname == primaryKey )
+      if ( fld.name() == primaryKey )
         continue;
 
-      const QString &type = fld.typeName();
-      if ( type == "money" )
-      {
-        query += QString( ",cash_out(%1)" ).arg( quotedIdentifier( fieldname ) );
-      }
-      else if ( type.startsWith( "_" ) )
-      {
-        query += QString( ",array_out(%1)" ).arg( quotedIdentifier( fieldname ) );
-      }
-      else if ( type == "bool" )
-      {
-        query += QString( ",boolout(%1)" ).arg( quotedIdentifier( fieldname ) );
-      }
-      else if ( type == "geometry" )
-      {
-        query += QString( ",asewkt(%1)" ).arg( quotedIdentifier( fieldname ) );
-      }
-      else
-      {
-        query += "," + quotedIdentifier( fieldname ) + "::text";
-      }
+      query += "," + fieldExpression( fld );
     }
 
     query += " from " + mSchemaTableName;
@@ -685,14 +695,26 @@ bool QgsPostgresProvider::nextFeature( QgsFeature& feature )
 
 QString QgsPostgresProvider::whereClause( int featureId ) const
 {
+  QString whereClause;
+
   if ( primaryKeyType != "tid" )
   {
-    return QString( "%1=%2" ).arg( quotedIdentifier( primaryKey ) ).arg( featureId );
+    whereClause = QString( "%1=%2" ).arg( quotedIdentifier( primaryKey ) ).arg( featureId );
   }
   else
   {
-    return QString( "%1='(%2,%3)'" ).arg( quotedIdentifier( primaryKey ) ).arg( featureId >> 16 ).arg( featureId & 0xffff );
+    whereClause = QString( "%1='(%2,%3)'" ).arg( quotedIdentifier( primaryKey ) ).arg( featureId >> 16 ).arg( featureId & 0xffff );
   }
+
+  if ( !sqlWhereClause.isEmpty() )
+  {
+    if ( !whereClause.isEmpty() )
+      whereClause += " and ";
+
+    whereClause += "(" + sqlWhereClause + ")";
+  }
+
+  return whereClause;
 }
 
 bool QgsPostgresProvider::featureAtId( int featureId, QgsFeature& feature, bool fetchGeometry, QgsAttributeList fetchAttributes )
@@ -847,6 +869,9 @@ void QgsPostgresProvider::loadFields()
   for ( int i = 0; i < PQnfields( result ); i++ )
   {
     QString fieldName = QString::fromUtf8( PQfname( result, i ) );
+    if ( fieldName == geometryColumn )
+      continue;
+
     int fldtyp = PQftype( result, i );
     QString typOid = QString().setNum( fldtyp );
     int fieldModifier = PQfmod( result, i );
@@ -876,75 +901,76 @@ void QgsPostgresProvider::loadFields()
     if ( PQntuples( tresult ) > 0 )
       fieldComment = QString::fromUtf8( PQgetvalue( tresult, 0, 0 ) );
 
-    if ( fieldName != geometryColumn )
+    QVariant::Type fieldType;
+
+    if ( fieldTType == "b" )
     {
-      QVariant::Type fieldType;
+      bool isArray = fieldTypeName.startsWith( "_" );
 
-      if ( fieldTType == "b" )
+      if ( isArray )
+        fieldTypeName = fieldTypeName.mid( 1 );
+
+      if ( fieldTypeName == "int8" )
       {
-        bool isArray = fieldTypeName.startsWith( "_" );
-
-        if ( isArray )
-          fieldTypeName = fieldTypeName.mid( 1 );
-
-        if ( fieldTypeName == "int8" )
-        {
-          fieldType = QVariant::LongLong;
-          fieldSize = -1;
-        }
-        else if ( fieldTypeName.startsWith( "int" ) ||
-                  fieldTypeName == "serial" )
-        {
-          fieldType = QVariant::Int;
-          fieldSize = -1;
-        }
-        else if ( fieldTypeName == "real" ||
-                  fieldTypeName == "double precision" ||
-                  fieldTypeName.startsWith( "float" ) ||
-                  fieldTypeName == "numeric" )
-        {
-          fieldType = QVariant::Double;
-          fieldSize = -1;
-        }
-        else if ( fieldTypeName == "text" ||
-                  fieldTypeName == "char" ||
-                  fieldTypeName == "bpchar" ||
-                  fieldTypeName == "varchar" ||
-                  fieldTypeName == "bool" ||
-                  fieldTypeName == "geometry" ||
-                  fieldTypeName == "money" ||
-                  fieldTypeName.startsWith( "time" ) ||
-                  fieldTypeName.startsWith( "date" ) )
-        {
-          fieldType = QVariant::String;
-        }
-        else
-        {
-          QgsDebugMsg( "Field " + fieldName + " ignored, because of unsupported type " + fieldTypeName );
-          continue;
-        }
-
-        if ( isArray )
-        {
-          fieldTypeName = "_" + fieldTypeName;
-          fieldType = QVariant::String;
-          fieldSize = -1;
-        }
+        fieldType = QVariant::LongLong;
+        fieldSize = -1;
       }
-      else if ( fieldTType == "e" )
+      else if ( fieldTypeName.startsWith( "int" ) ||
+                fieldTypeName == "serial" )
+      {
+        fieldType = QVariant::Int;
+        fieldSize = -1;
+      }
+      else if ( fieldTypeName == "real" ||
+                fieldTypeName == "double precision" ||
+                fieldTypeName.startsWith( "float" ) ||
+                fieldTypeName == "numeric" )
+      {
+        fieldType = QVariant::Double;
+        fieldSize = -1;
+      }
+      else if ( fieldTypeName == "text" ||
+                fieldTypeName == "bpchar" ||
+                fieldTypeName == "varchar" ||
+                fieldTypeName == "bool" ||
+                fieldTypeName == "geometry" ||
+                fieldTypeName == "money" ||
+                fieldTypeName.startsWith( "time" ) ||
+                fieldTypeName.startsWith( "date" ) )
       {
-        // enum
         fieldType = QVariant::String;
         fieldSize = -1;
       }
+      else if ( fieldTypeName == "char" )
+      {
+        fieldType = QVariant::String;
+      }
       else
       {
-        QgsDebugMsg( "Field " + fieldName + " ignored, because of unsupported type type " + fieldTType );
+        QgsDebugMsg( "Field " + fieldName + " ignored, because of unsupported type " + fieldTypeName );
         continue;
       }
 
-      attributeFields.insert( i, QgsField( fieldName, fieldType, fieldTypeName, fieldSize, fieldModifier, fieldComment ) );
+      if ( isArray )
+      {
+        fieldTypeName = "_" + fieldTypeName;
+        fieldType = QVariant::String;
+        fieldSize = -1;
+      }
     }
+    else if ( fieldTType == "e" )
+    {
+      // enum
+      fieldType = QVariant::String;
+      fieldSize = -1;
+    }
+    else
+    {
+      QgsDebugMsg( "Field " + fieldName + " ignored, because of unsupported type type " + fieldTType );
+      continue;
+    }
+
+    attributeFields.insert( i, QgsField( fieldName, fieldType, fieldTypeName, fieldSize, fieldModifier, fieldComment ) );
   }
 }
 
@@ -954,7 +980,7 @@ QString QgsPostgresProvider::getPrimaryKey()
   // can be used as a key into the table. Primary keys are always
   // unique indices, so we catch them as well.
 
-  QString sql = QString( "select indkey from pg_index where indisunique='t' and indrelid=regclass(%1)::oid" )
+  QString sql = QString( "select indkey from pg_index where indisunique and indrelid=regclass(%1)::oid and indpred is null" )
                 .arg( quotedValue( mSchemaTableName ) );
 
   QgsDebugMsg( "Getting unique index using '" + sql + "'" );
@@ -1037,6 +1063,17 @@ QString QgsPostgresProvider::getPrimaryKey()
                             "primary key), has a PostgreSQL oid column or has a ctid\n"
                             "column with a 16bit block number.\n" ) );
       }
+      else
+      {
+        mPrimaryKeyDefault = defaultValue( primaryKey ).toString();
+        if ( mPrimaryKeyDefault.isNull() )
+        {
+          mPrimaryKeyDefault = QString( "max(%1)+1 from %2.%3" )
+                               .arg( quotedIdentifier( primaryKey ) )
+                               .arg( quotedIdentifier( mSchemaName ) )
+                               .arg( quotedIdentifier( mTableName ) );
+        }
+      }
     }
     else if ( type == "v" ) // the relation is a view
     {
@@ -1050,10 +1087,17 @@ QString QgsPostgresProvider::getPrimaryKey()
 
         Result result = connectionRO->PQexec( sql );
 
-        if ( PQresultStatus( result ) != PGRES_TUPLES_OK ||
-             PQntuples( result ) != 1 ||
-             QString( PQgetvalue( result, 0, 0 ) ) != "int4" ||
-             !uniqueData( mSchemaName, mTableName, primaryKey ) )
+        QString type;
+        if ( PQresultStatus( result ) == PGRES_TUPLES_OK &&
+             PQntuples( result ) == 1 )
+        {
+          type = PQgetvalue( result, 0, 0 );
+        }
+
+        // mPrimaryKeyDefault stays null and is retrieved later on demand
+
+        if (( type != "int4" && type != "oid" ) ||
+            !uniqueData( mSchemaName, mTableName, primaryKey ) )
         {
           primaryKey = "";
         }
@@ -1061,18 +1105,7 @@ QString QgsPostgresProvider::getPrimaryKey()
 
       if ( primaryKey.isEmpty() )
       {
-        // Have a poke around the view to see if any of the columns
-        // could be used as the primary key.
-        tableCols cols;
-        // Given a schema.view, populate the cols variable with the
-        // schema.table.column's that underly the view columns.
-        findColumns( cols );
-        // From the view columns, choose one for which the underlying
-        // column is suitable for use as a key into the view.
-        primaryKey = chooseViewColumn( cols );
-
-        mUri.setKeyColumn( primaryKey );
-        setDataSourceUri( mUri.uri() );
+        parseView();
       }
     }
     else
@@ -1172,9 +1205,20 @@ QString QgsPostgresProvider::getPrimaryKey()
       valid = false;
       showMessageBox( tr( "Unable to find a key column" ), log );
     }
+    else
+    {
+      mPrimaryKeyDefault = defaultValue( primaryKey ).toString();
+      if ( mPrimaryKeyDefault.isNull() )
+      {
+        mPrimaryKeyDefault = QString( "max(%1)+1 from %2.%3" )
+                             .arg( quotedIdentifier( primaryKey ) )
+                             .arg( quotedIdentifier( mSchemaName ) )
+                             .arg( quotedIdentifier( mTableName ) );
+      }
+    }
   }
 
-  if ( primaryKey.length() > 0 )
+  if ( !primaryKey.isNull() )
   {
     QgsDebugMsg( "Qgis row key is " + primaryKey );
   }
@@ -1186,11 +1230,58 @@ QString QgsPostgresProvider::getPrimaryKey()
   return primaryKey;
 }
 
+void QgsPostgresProvider::parseView()
+{
+  // Have a poke around the view to see if any of the columns
+  // could be used as the primary key.
+  tableCols cols;
+
+  // Given a schema.view, populate the cols variable with the
+  // schema.table.column's that underly the view columns.
+  findColumns( cols );
+
+  // pick the primary key, if we don't have one yet
+  if ( primaryKey.isEmpty() )
+  {
+    // From the view columns, choose one for which the underlying
+    // column is suitable for use as a key into the view.
+    primaryKey = chooseViewColumn( cols );
+  }
+
+  tableCols::const_iterator it = cols.find( primaryKey );
+  if ( it != cols.end() )
+  {
+    mPrimaryKeyDefault = defaultValue( it->second.column, it->second.relation, it->second.schema ).toString();
+    if ( mPrimaryKeyDefault.isNull() )
+    {
+      mPrimaryKeyDefault = QString( "max(%1)+1 from %2.%3" )
+                           .arg( quotedIdentifier( it->second.column ) )
+                           .arg( quotedIdentifier( it->second.schema ) )
+                           .arg( quotedIdentifier( it->second.relation ) );
+    }
+  }
+  else
+  {
+    mPrimaryKeyDefault = QString( "max(%1)+1 from %2.%3" )
+                         .arg( quotedIdentifier( primaryKey ) )
+                         .arg( quotedIdentifier( mSchemaName ) )
+                         .arg( quotedIdentifier( mTableName ) );
+  }
+}
+
+QString QgsPostgresProvider::primaryKeyDefault()
+{
+  if ( mPrimaryKeyDefault.isNull() )
+    parseView();
+
+  return mPrimaryKeyDefault;
+}
+
 // Given the table and column that each column in the view refers to,
 // choose one. Prefers column with an index on them, but will
 // otherwise choose something suitable.
 
-QString QgsPostgresProvider::chooseViewColumn( const tableCols& cols )
+QString QgsPostgresProvider::chooseViewColumn( const tableCols &cols )
 {
   // For each relation name and column name need to see if it
   // has unique constraints on it, or is a primary key (if not,
@@ -1382,6 +1473,11 @@ bool QgsPostgresProvider::uniqueData( QString schemaName,
                 .arg( quotedIdentifier( schemaName ) )
                 .arg( quotedIdentifier( tableName ) );
 
+  if ( !sqlWhereClause.isEmpty() )
+  {
+    sql += " where " + sqlWhereClause;
+  }
+
   Result unique = connectionRO->PQexec( sql );
 
   if ( PQntuples( unique ) == 1 && QString::fromUtf8( PQgetvalue( unique, 0, 0 ) ).startsWith( "t" ) )
@@ -1576,6 +1672,12 @@ void QgsPostgresProvider::findColumns( tableCols& cols )
              "view_schema=%1 AND view_name=%2" )
     .arg( quotedValue( mSchemaName ) )
     .arg( quotedValue( mTableName ) );
+
+  if ( !primaryKey.isEmpty() )
+  {
+    viewColumnSql += QString( " AND column_name=%1" ).arg( quotedValue( primaryKey ) );
+  }
+
   Result viewColumnResult = connectionRO->PQexec( viewColumnSql );
 
   //find out view definition
@@ -1660,7 +1762,7 @@ QVariant QgsPostgresProvider::minimumValue( int index )
 }
 
 // Returns the list of unique values of an attribute
-void QgsPostgresProvider::uniqueValues( int index, QList<QVariant> &uniqueValues )
+void QgsPostgresProvider::uniqueValues( int index, QList<QVariant> &uniqueValues, int limit )
 {
   uniqueValues.clear();
 
@@ -1683,11 +1785,16 @@ void QgsPostgresProvider::uniqueValues( int index, QList<QVariant> &uniqueValues
             .arg( sqlWhereClause );
     }
 
+    if ( limit >= 0 )
+    {
+      sql += QString( " LIMIT %1" ).arg( limit );
+    }
+
     Result res = connectionRO->PQexec( sql );
     if ( PQresultStatus( res ) == PGRES_TUPLES_OK )
     {
       for ( int i = 0; i < PQntuples( res ); i++ )
-        uniqueValues.append( QString::fromUtf8( PQgetvalue( res, i, 0 ) ) );
+        uniqueValues.append( convertValue( fld.type(), QString::fromUtf8( PQgetvalue( res, i, 0 ) ) ) );
     }
   }
   catch ( PGFieldNotFound )
@@ -1742,7 +1849,7 @@ void QgsPostgresProvider::enumValues( int index, QStringList& enumList )
 bool QgsPostgresProvider::parseEnumRange( QStringList& enumValues, const QString& attributeName ) const
 {
   enumValues.clear();
-  QString enumRangeSql = QString( "SELECT enum_range(%1) from %2 limit1" ).arg( quotedIdentifier( attributeName ) ).arg( mSchemaTableName );
+  QString enumRangeSql = QString( "SELECT enum_range(%1) from %2 limit 1" ).arg( quotedIdentifier( attributeName ) ).arg( mSchemaTableName );
   Result enumRangeRes = connectionRO->PQexec( enumRangeSql );
   if ( PQresultStatus( enumRangeRes ) == PGRES_TUPLES_OK && PQntuples( enumRangeRes ) > 0 )
   {
@@ -1854,28 +1961,18 @@ QVariant QgsPostgresProvider::maximumValue( int index )
 }
 
 
-int QgsPostgresProvider::maxPrimaryKeyValue()
-{
-  QString sql;
-
-  sql = QString( "select max(%1) from %2" )
-        .arg( quotedIdentifier( primaryKey ) )
-        .arg( mSchemaTableName );
-
-  Result rmax = connectionRO->PQexec( sql );
-  QString maxValue = QString::fromUtf8( PQgetvalue( rmax, 0, 0 ) );
-
-  return maxValue.toInt();
-}
-
-
 bool QgsPostgresProvider::isValid()
 {
   return valid;
 }
 
-QVariant QgsPostgresProvider::defaultValue( QString fieldName )
+QVariant QgsPostgresProvider::defaultValue( QString fieldName, QString tableName, QString schemaName )
 {
+  if ( schemaName.isNull() )
+    schemaName = mSchemaName;
+  if ( tableName.isNull() )
+    tableName = mTableName;
+
   // Get the default column value from the Postgres information
   // schema. If there is no default we return an empty string.
 
@@ -1885,8 +1982,8 @@ QVariant QgsPostgresProvider::defaultValue( QString fieldName )
   QString sql( "SELECT column_default FROM"
                " information_schema.columns WHERE"
                " column_default IS NOT NULL"
-               " AND table_schema = " + quotedValue( mSchemaName ) +
-               " AND table_name = " + quotedValue( mTableName ) +
+               " AND table_schema = " + quotedValue( schemaName ) +
+               " AND table_name = " + quotedValue( tableName ) +
                " AND column_name = " + quotedValue( fieldName ) );
 
   QVariant defaultValue( QString::null );
@@ -1896,8 +1993,6 @@ QVariant QgsPostgresProvider::defaultValue( QString fieldName )
   if ( PQntuples( result ) == 1 && !PQgetisnull( result, 0, 0 ) )
     defaultValue = QString::fromUtf8( PQgetvalue( result, 0, 0 ) );
 
-  // QgsDebugMsg( QString("defaultValue for %1 is NULL: %2").arg(fieldId).arg( defaultValue.isNull() ) );
-
   return defaultValue;
 }
 
@@ -2007,7 +2102,7 @@ QByteArray QgsPostgresProvider::paramValue( QString fieldValue, const QString &d
   return fieldValue.toUtf8();
 }
 
-bool QgsPostgresProvider::addFeatures( QgsFeatureList & flist )
+bool QgsPostgresProvider::addFeatures( QgsFeatureList &flist )
 {
   if ( flist.size() == 0 )
     return true;
@@ -2124,10 +2219,6 @@ bool QgsPostgresProvider::addFeatures( QgsFeatureList & flist )
       throw PGException( stmt );
     PQclear( stmt );
 
-    QString keyDefault = defaultValue( primaryKey ).toString();
-    int primaryKeyHighWater = -1;
-    if ( keyDefault.isNull() )
-      primaryKeyHighWater = maxPrimaryKeyValue();
     QList<int> newIds;
 
     for ( QgsFeatureList::iterator features = flist.begin(); features != flist.end(); features++ )
@@ -2142,18 +2233,9 @@ bool QgsPostgresProvider::addFeatures( QgsFeatureList & flist )
 
       if ( primaryKeyType != "tid" )
       {
-        if ( keyDefault.isNull() )
-        {
-          ++primaryKeyHighWater;
-          params << QString::number( primaryKeyHighWater );
-          newIds << primaryKeyHighWater;
-        }
-        else
-        {
-          QByteArray key = paramValue( keyDefault, keyDefault );
-          params << key;
-          newIds << key.toInt();
-        }
+        int id = paramValue( primaryKeyDefault(), primaryKeyDefault() ).toInt();
+        params << QString::number( id );
+        newIds << id;
       }
 
       for ( int i = 0; i < fieldId.size(); i++ )
@@ -2504,9 +2586,18 @@ int QgsPostgresProvider::capabilities() const
   return enabledCapabilities;
 }
 
-void QgsPostgresProvider::setSubsetString( QString theSQL )
+bool QgsPostgresProvider::setSubsetString( QString theSQL )
 {
+  QString prevWhere = sqlWhereClause;
+
   sqlWhereClause = theSQL;
+
+  if ( !uniqueData( mSchemaName, mTableName, primaryKey ) )
+  {
+    sqlWhereClause = prevWhere;
+    return false;
+  }
+
   // Update datasource uri too
   mUri.setSql( theSQL );
   // Update yet another copy of the uri. Why are there 3 copies of the
@@ -2516,6 +2607,8 @@ void QgsPostgresProvider::setSubsetString( QString theSQL )
   // need to recalculate the number of features...
   getFeatureCount();
   calculateExtents();
+
+  return true;
 }
 
 long QgsPostgresProvider::getFeatureCount()
@@ -2876,8 +2969,10 @@ bool QgsPostgresProvider::getGeometryDetails()
   }
   else // something went wrong...
   {
-    log.prepend( tr( "Qgis was unable to determine the type and srid of column %1 in %2. The database communication log was:\n" )
-                 .arg( geometryColumn ).arg( mSchemaTableName ) );
+    log.prepend( tr( "Qgis was unable to determine the type and srid of column %1 in %2. The database communication log was:\n%3" )
+                 .arg( geometryColumn )
+                 .arg( mSchemaTableName )
+                 .arg( QString::fromUtf8( PQresultErrorMessage( result ) ) ) );
     showMessageBox( tr( "Unable to get feature type and srid" ), log );
   }
 
@@ -3051,8 +3146,7 @@ void QgsPostgresProvider::showMessageBox( const QString& title, const QString& t
   message->showMessage();
 }
 
-void QgsPostgresProvider::showMessageBox( const QString& title,
-    const QStringList& text )
+void QgsPostgresProvider::showMessageBox( const QString& title, const QStringList& text )
 {
   showMessageBox( title, text.join( "\n" ) );
 }
@@ -3101,9 +3195,6 @@ QString  QgsPostgresProvider::description() const
   return POSTGRES_DESCRIPTION;
 } //  QgsPostgresProvider::description()
 
-
-
-
 /**
  * Class factory to return a pointer to a newly created
  * QgsPostgresProvider object
