@@ -14,7 +14,7 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-/* $Id: qgspostgresprovider.h 10981 2009-06-26 15:29:46Z jef $ */
+/* $Id$ */
 
 #ifndef QGSPOSTGRESPROVIDER_H
 #define QGSPOSTGRESPROVIDER_H
@@ -189,7 +189,7 @@ class QgsPostgresProvider : public QgsVectorDataProvider
     /** Return the unique values of an attribute
      *  @param index the index of the attribute
      *  @param values reference to the list of unique values */
-    virtual void uniqueValues( int index, QList<QVariant> &uniqueValues );
+    virtual void uniqueValues( int index, QList<QVariant> &uniqueValues, int limit = -1 );
 
     /**Returns the possible enum values of an attribute. Returns an empty stringlist if a provider does not support enum types
       or if the given attribute is not an enum type.
@@ -205,7 +205,7 @@ class QgsPostgresProvider : public QgsVectorDataProvider
     QgsAttributeList attributeIndexes();
 
     /**Returns the default value for field specified by @c fieldName */
-    QVariant defaultValue( QString fieldName );
+    QVariant defaultValue( QString fieldName, QString tableName = QString::null, QString schemaName = QString::null );
 
     /**Returns the default value for field specified by @c fieldId */
     QVariant defaultValue( int fieldId );
@@ -253,7 +253,9 @@ class QgsPostgresProvider : public QgsVectorDataProvider
     QString subsetString();
 
     /** mutator for sql where clause used to limit dataset size */
-    void setSubsetString( QString theSQL );
+    bool setSubsetString( QString theSQL );
+
+    virtual bool supportsSubsetString() { return true; }
 
     /**Returns a bitmask containing the supported capabilities*/
     int capabilities() const;
@@ -341,6 +343,10 @@ class QgsPostgresProvider : public QgsVectorDataProvider
      */
     QString quotedValue( QString value ) const;
 
+    /** expression to retrieve value
+     */
+    QString fieldExpression( const QgsField &fld ) const;
+
     /** Load the field list
     */
     void loadFields();
@@ -554,13 +560,6 @@ class QgsPostgresProvider : public QgsVectorDataProvider
 
     int enabledCapabilities;
 
-    /**Returns the maximum value of the primary key attribute
-       @note  You should run this inside of a PostgreSQL transaction
-              so that you can safely increment the value returned for
-              use in newly added features.
-      */
-    int maxPrimaryKeyValue();
-
     //! Get the feature count based on the where clause
     long getFeatureCount();
 
@@ -696,6 +695,14 @@ class QgsPostgresProvider : public QgsVectorDataProvider
     void disconnectDb();
 
     static int providerIds;
+
+    QString primaryKeyDefault();
+    void parseView();
+
+    /**
+     * Default value for primary key
+     */
+    QString mPrimaryKeyDefault;
 };
 
 #endif
